'******************************************************************************
' File: RapidDBG.inc - part of RapidDBG, a RapidQ Source-line Debugger.
' Copyright (C) 2008-2013 Don Homans
'
'    RapidDBG is free software: you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation, either version 3 of the License, or
'    (at your option) any later version.
'
'    RapidDBG is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with RapidDBG.  If not, see <http://www.gnu.org/licenses/>.
'
'******************************************************************************
' Bug fix for QCOLORDIALOG component in RapidQ2.inc
' This is the original version by D. Glodt with bug fixes
' PLUS a call to Jacques' Multiple Callback Forwarder
' ilo using CODEPTR() for CC.lpfnHook 
'
'   Style cdNormal implemented (default) - compact but expandable.
'
'   Dialog height is fixed, dialog width dependant on style property.
'   Position anywhere on-screen &/or relative to other objects ONCE
'   the Style has been set, using Left, Top, Width & Height.
'   Custom colours updated on exit (colours  retained).
'   Change dialog caption, if required.
'
'      AUTHOR: d_homans@yahoo.com.au
'     VERSION: v1.53 (defined in RapidDBG.fbb)
'        DATE: 25 May 2013
' CONTRIBUTOR: Jacques Philippe
'------------------------------------------------------------------------------------------
$DEFINE cdNormal        0                                       ' Default compact style (expandable)
$DEFINE cdFullOpen      1                                       ' Customisable color style
$DEFINE cdNoFullOpen    2                                       ' Compact style, no custom color dialog

$DEFINE CC_HEIGHT       335                                     ' Height of ColorChooser window
$DEFINE CC_WIDTH_STD    225                                     ' Width of window when in compact style
$DEFINE CC_WIDTH_FULL   460                                     ' Width of window when in full style

' This MUST be declared here, as the function is in the PRIVATE area of the code!
DECLARE FUNCTION HookProc(hWnd AS long, uMsg AS long, wParam AS long, lParam AS long) AS long

DEFLNG BindColorHook                                            ' This MUST be global!!

TYPE QColorDlg EXTENDS QFORM
PUBLIC:
    Width           AS long PROPERTY SET SetWidth               ' Force READ ONLY of this property
    Height          AS long PROPERTY SET SetHeight              '            ditto
    Style           AS long PROPERTY SET SetStyle               ' cdNormal, cdNoFullOpen, cdFullOpen
    Left            AS long                                     ' NB Screen co-ords
    Top             AS long                                     ' --ditto--
    Caption         AS string                                   ' Configurable caption
    Color           AS long                                     ' PreSet AND then selected color
    Colors(1 TO 16) AS long                                     ' Custom colors - User settable!
    WindowHandle    AS long                                     ' Not used ATT

PRIVATE:
    CC              AS CHOOSECOLOR

WITH QColorDlg

' Set visual properties of the dialog prior to displaying.
' NB Hook Procedure MUST return 0 when finished (terminates)!!!
    FUNCTION HookProc(hWnd AS long, uMsg AS long, wParam AS long, lParam AS long) AS long
    DIM R AS QRECT
    DEFINT X, Y, W, H

        Result=1                                                ' Default stay "active"
        IF uMsg=WM_INITDIALOG THEN                              ' ALL initializing is done here 
            GetWindowRect(hWnd, R)
            W = R.Right-R.Left+1: H = R.Bottom-R.Top+1
            X=IIF((.Left < 0), (Screen.Width-W)\2, .Left)
            Y=IIF((.Top < 0),  (Screen.Height-H)\2, .Top)
            MoveWindow(hWnd, X, Y, W, H, 0) 
            IF .Caption <> "" THEN SetWindowText(hWnd, .Caption)
            Result=0                                            ' Returns 0 on successful completion
        END IF
    END FUNCTION

    SUB SetCustomColors
    DEFINT i
    
        FOR i=1 TO 16
            .Colors(i)=&hC0C0C0
        NEXT i
    END SUB

PUBLIC:

    FUNCTION Execute() AS integer
        Result = False                                          ' Default fail
        BIND BindColorHook TO HookProc                          ' Prepare for setting hook function

        MEMSET(THIS.CC,0,SIZEOF(THIS.CC))                       ' Zero the structure
        .CC.lStructSize = SIZEOF(THIS.CC)
        .CC.hWndOwner = Application.Handle                      ' Keeps the focus on the chooser
        .CC.hInstance = Application.hInstance
        .CC.lpCustColors = VARPTR(THIS.Colors(1))               ' Set the pointer to our custom color array
        .CC.lpfnHook = SetNewCallBack_4(BindColorHook)          ' Set the initialisation hook
        .CC.RGBResult = .Color                                  ' Set initial selected color
        .CC.Flags = CC_RGBINIT OR CC_SHOWHELP OR CC_ENABLEHOOK  ' Some defaults
        SELECT CASE .Style                                      ' Defaults to cdNormal (0)
            CASE cdNoFullOpen
                .CC.Flags = .CC.Flags OR CC_PREVENTFULLOPEN     ' Fixed size - compact
            CASE cdFullOpen
                .CC.Flags = .CC.Flags OR CC_FULLOPEN            ' Fixed size - expanded
        END SELECT
        IF ChooseColorDlg(THIS.CC) THEN                         ' Call the Color select Dialog
            .Color = .CC.RGBResult                              ' Update user selected color
            Result = True
        END IF
    END FUNCTION

    PROPERTY SET SetStyle (ccStyle AS long)
        .Style = ccStyle
        SELECT CASE ccStyle
            CASE cdNoFullOpen
                .Width = CC_WIDTH_STD
            CASE cdFullOpen
                .Width = CC_WIDTH_FULL
            CASE ELSE                                           ' cdNormal or invalid "dont care" value
                .Width = CC_WIDTH_STD                           ' Default: cdNormal, compact but expandable
        END SELECT
    END PROPERTY

    PROPERTY SET SetWidth (W AS long)
    ' READONLY property
    END PROPERTY

    PROPERTY SET SetHeight (H AS long)
    ' READONLY property
    END PROPERTY

    CONSTRUCTOR
        Style = cdNormal                                        ' Default to compact mode, but expandable
        Left = -1                                               ' Defaults center of screen
        Top = -1                                                ' ditto
        Width = CC_WIDTH_STD                                    ' Compact mode width
        Height = CC_HEIGHT                                      ' Default height (either mode)
        Color = 0                                               ' Default initial color
        Caption = ""                                            ' Default caption of chooser ("Color")
        BorderStyle = 0                                         ' Blank form
        WindowHandle = 0                                        ' Not used
        SetCustomColors                                         ' Default all to Lt. Grey
    END CONSTRUCTOR
END WITH
END TYPE
