'-------------------------------------------------------------------------------
'  Toolbar1.bas - Single or multi-image bitmap from either file or resource
'  using QIMAGELIST. 
'  Both AddBMPFile & AddBMPHandle METHODs have been modified from the original.
'  Both now include an image count which allows multi-image bmps to be used.
'
'  This example demonstrates the five (5) RapidFRM bitmaps. Each bitmap has a 
'  different count of images. Each image within the bitmaps is 26x26 pix
'  Load single images as well -  a sixth single image bitmap included
'  IF using as a resource, you MUST include the ".rc" file in the make
'  The digit(s) in the first column of the ".rc" file is the BMP_Resource
'  number. VERY important! They do not need to be consecutive, but must be unique.
'
'  The buttons can be re-sized (up or down) automatically, by changing BTN_WIDTH &/or
'  BTN_HEIGHT. However if using masked transparency then image quality can suffer
'  and not recommended (unless you choose a more "neutral" transparency color?).
'
'  NB The Toolbar is automatically resized to the form in the forms WndProc OnPaint
'
'   d_homans@yahoo.com.au
'   5 Apr 2012
'-------------------------------------------------------------------------------
#INCLUDE "windows.bi"
#INCLUDE "win/commctrl.bi"

InitCommonControls()

#INCLUDE "win/RQInclude.bi"

' Some toolbar stuff...
#DEFINE USE_FILE                                        ' Images from file or resource

#DEFINE MAX_BTNS    15                                  ' Max for this Toolbar
#DEFINE BTN_WIDTH   26                                  ' Default for RapidFRM images
#DEFINE BTN_HEIGHT  26                                  '    ditto

#DEFINE TBAR_ID      1                                  ' <<<<<< toolbar 1-5 or bmp 6

DIM SHARED AS byte tbBtns(1 TO 6) = {15,14,11,4,11,1}     ' Num images in each bitmap
DIM SHARED AS HWND hToolBar
'-------------------------------------------------------------------------------
' Global data
'-------------------------------------------------------------------------------
DIM SHARED AS QFORM Form
DIM SHARED AS QBUTTON Butn
DIM SHARED AS QSTATUSBAR StatBar

DIM SHARED AS QFORM PTR lpForm      = @Form
DIM SHARED AS QBUTTON PTR lpButn    = @Butn
DIM SHARED AS QSTATUSBAR PTR lpStat = @StatBar

'-------------------------------------------------------------------------------
' Forward declares for event Subs
'-------------------------------------------------------------------------------
DECLARE SUB TBar_Click(Btn AS integer)
DECLARE SUB Exit_Click(Sender AS QBUTTON)

'-------------------------------------------------------------------------------
' Initialisation of objects
'-------------------------------------------------------------------------------
SUB ConfigComponents
    lpForm->objIndex=0
    lpForm->Width=505: lpForm->Height=400
    lpForm->Caption=Application.Name
    lpForm->Visible=1

    lpButn->objIndex=MAX_BTNS+2
    lpButn->Left=200: lpButn->Top=320
    lpButn->Width=100: lpButn->Height=25
    lpButn->Caption="Exit"
    lpButn->OnClick=CAST(ANY PTR, @Exit_Click)

    lpStat->objIndex=MAX_BTNS+3
    lpStat->Height=19
    lpStat->SimplePanel=1
    lpStat->SimpleText="Simple Text"
    lpStat->SizeGrip=1
END SUB

'-------------------------------------------------------------------------------
' Creation of objects
' Toolbar button styles: BTNS_CHECK, BTNS_CHECKGROUP ....
'-------------------------------------------------------------------------------
SUB AddToolBar(Parent AS HWND)
DIM AS QIMAGELIST imgList
DIM AS TBBUTTON tbb
DIM AS UINT ExtStyle=0                                          ' 0, WS_EX_DLGMODALFRAME, WS_EX_CLIENTEDGE
DIM AS UINT StdStyle=WS_THICKFRAME OR TBSTYLE_TRANSPARENT                             ' 0, WS_THICKFRAME, TBSTYLE_FLAT, ... 
DIM AS integer numBtns=tbBtns(TBAR_ID)

    ' Create the toolbar
    hToolBar=CreateWindowEx(ExtStyle, "ToolbarWindow32", "", WS_CHILD OR WS_VISIBLE OR StdStyle, _
                                                0, 0, 0, 0, Parent, CAST(HMENU, MAX_BTNS+1), 0, 0)
    SendMessage(hToolBar, TB_BUTTONSTRUCTSIZE, SIZEOF(TBBUTTON), NULL)          ' Identifies which vers of dll

    ' Create & populate the ImageList
    imgList.Width=BTN_WIDTH                                                     ' Set defaults for this toolbar
    imgList.Height=BTN_HEIGHT                                                   '    ditto
'    imgList.Masked=False
#IFDEF USE_FILE
    SELECT CASE TBAR_ID
        CASE 1: imgList.AddBMPFile("images\Toolbar1.bmp", numBtns, &hff00ff)
        CASE 2: imgList.AddBMPFile("images\Toolbar2.bmp", numBtns, &hff00ff)
        CASE 3: imgList.AddBMPFile("images\Toolbar3.bmp", numBtns, &hff00ff)
        CASE 4: imgList.AddBMPFile("images\Toolbar4.bmp", numBtns, &hff00ff)
        CASE 5: imgList.AddBMPFile("images\Toolbar5.bmp", numBtns, &hff00ff)
        CASE 6: imgList.AddBMPFile("images\OneButn.bmp", numBtns, &hff00ff)
    END SELECT
''  Add more buttons????????? Provided that MAX_BTNS doesn't constrain
''    imgList.AddBMPHandle(6,1,&hff00ff)
''    numBtns=imgList.Count
#ELSE
    imgList.AddBMPHandle(TBAR_ID, numBtns, &hff00ff)                            ' Load the one large image
''  Add more buttons????????
''    imgList.AddBMPHandle(6,1,&hff00ff)
''    numBtns=imgList.Count
#ENDIF

    ' Configure & show the Toolbar
    SendMessage(hToolBar, TB_SETIMAGELIST, 0, CAST(LPARAM,imgList.Handle))      ' Set ImgList to toolbar
    FOR i AS integer = 0 TO numBtns-1
        tbb.iBitmap = i                                                         ' Index of image in ImgList
        tbb.idCommand = i+1                                                     ' id in WM_COMMAND msg (1 -> NUM_BTNS)
        tbb.fsStyle = BTNS_BUTTON                                               ' BTNS_CHECKGROUP      ' interesting!!
        tbb.fsState = TBSTATE_ENABLED
        tbb.dwData = 0
        SendMessage(hToolBar, TB_ADDBUTTONS, 1, CAST(LPARAM, @tbb))             ' Add it to toolbar
    NEXT
    SendMessage(hToolBar, TB_AUTOSIZE, 0 ,0)                                    ' Toolbar re-calculates itself
END SUB

SUB AddChildWindows(Parent AS HWND)
    AddToolBar(Parent)

    lpButn->Parent=Parent
    lpButn->Handle=CreateWindowEx(0, "BUTTON", lpButn->Caption, _
                                        WS_VISIBLE OR WS_CHILD OR BS_PUSHBUTTON, _
                                        lpButn->Left, lpButn->Top, lpButn->Width, lpButn->Height, _
                                        Parent, CAST(HMENU,lpButn->objIndex), 0, 0)

    lpStat->Parent=Parent
    lpStat->Handle = CreateWindowEx(0, "msctls_statusbar32", "Status", _
                                        WS_VISIBLE OR WS_CHILD, _
                                        lpStat->Left, lpStat->Top, lpStat->Width, lpStat->Height, _
                                        Parent, CAST(HMENU,lpStat->objIndex), 0, 0)
END SUB

FUNCTION WndProc(hWnd AS HWND, uMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM) AS LRESULT

    SELECT CASE uMsg
        CASE WM_CREATE
            AddChildWindows(hWnd)

        CASE WM_PAINT
            DIM AS RECT rct                                                 ' Auto adjust statusbar to form
            DIM AS integer ht1, ht2
            GetClientRect(hToolBar, @rct)
            ht1 = rct.Bottom-rct.Top+1                                      ' Height of status bar
            GetClientRect(lpStat->Handle, @rct)
            ht2 = rct.Bottom-rct.Top+1                                      ' Height of status bar

            GetClientRect(hWnd, @rct)
            MoveWindow(hToolBar, rct.Left, rct.Bottom-ht1, rct.Right-rct.Left+1, ht1, 1)
            MoveWindow(lpStat->Handle, rct.Left, rct.Bottom-ht2, rct.Right-rct.Left+1, ht2, 1)

        CASE WM_COMMAND
            SELECT CASE LOWORD(wParam)
                CASE 1 TO MAX_BTNS
                    TBar_Click(LOWORD(wParam))

                CASE lpButn->ObjIndex
                    IF lpButn->OnClick>0 THEN lpButn->OnClick(lpButn)
            END SELECT

        CASE WM_DESTROY
            PostQuitMessage(0)

    END SELECT

    RETURN DefWindowProc(hWnd, uMsg, wParam, lParam)
END FUNCTION


' Main program loop
FUNCTION WinMain(hInst AS HINSTANCE, hPrevInstance AS HINSTANCE, _
                            szCmdLine AS string, iCmdShow AS integer) AS integer
DIM AS MSG wMsg
DIM AS WNDCLASSEX wc

WITH wc
    .cbSize=SizeOf(WNDCLASSEX)
    .style=CS_HREDRAW OR CS_VREDRAW
    .lpfnWndProc=@WndProc
    .hInstance=hInst
    .hIcon = LoadImage(hInst, @"Main", IMAGE_ICON, 16, 16, 0)
    .hCursor=LoadCursor(0,IDC_ARROW)
    .hbrBackground=Cast(HBRUSH,COLOR_WINDOW)
    .lpszClassName=@"RQClass"
    .hIconSm=.hIcon
END WITH

    IF RegisterClassEx(@wc)=0 THEN
        ShowMessage("Register class error")
        RETURN 0
    END IF

    ConfigComponents

    lpForm->Parent=0
    lpForm->Handle=CreateWindowEx(0, "RQClass", lpForm->Caption, _
                                    WS_VISIBLE OR WS_OVERLAPPEDWINDOW, _
                                    lpForm->Left, lpForm->Top, lpForm->Width, lpForm->Height, _
                                    0, 0, hInst, 0)
    Application.hMainForm=lpForm->Handle                                ' Need this for the terminate fn
    lpForm->Center

    WHILE GetMessage(@wMsg, NULL, 0, 0) <> FALSE
        TranslateMessage(@wMsg)
        DispatchMessage(@wMsg)
    WEND

    RETURN wMsg.wParam
END FUNCTION
'
' Main Entry Point
'
END WinMain(Application.hInstance, NULL, Command(), SW_NORMAL)

'-------------------------------------------------------------------------------
' User added event Subs
'-------------------------------------------------------------------------------
SUB TBar_Click(Btn AS integer)
    lpStat->SimpleText="Button " & Btn
END SUB

SUB Exit_Click(Sender AS QBUTTON)
    Application.Terminate
END SUB

