'-------------------------------------------------------------------------------
' Toolbar2.bas - Uses QIMAGELIST & BMP FILES
'
'   Load Toolbar with a single (multi-image) bitmap file or
'   4 individual files
'   Playing with different styles of buttons in the toolbar
'   Adding a separator (variable width)
'
'   NB The toolbar is auto-sized in the Forms WndProc - OnPaint
'
'   d_homans@yahoo.com.au
'   5 Apr 2012
'-------------------------------------------------------------------------------
#INCLUDE "windows.bi"
#INCLUDE "win/commctrl.bi"

InitCommonControls()

#INCLUDE "win/RQInclude.bi"

' Some toolbar stuff...
CONST BTN_SEP = -1

#DEFINE xMULTI_IMAGE
#DEFINE NUMBTNS     4

DIM SHARED AS HWND hToolBar

'-------------------------------------------------------------------------------
' Global data
'-------------------------------------------------------------------------------
DIM SHARED AS QFORM Form
DIM SHARED AS QBUTTON Butn
DIM SHARED AS QSTATUSBAR StatBar

DIM SHARED AS QFORM PTR lpForm      = @Form
DIM SHARED AS QBUTTON PTR lpButn    = @Butn
DIM SHARED AS QSTATUSBAR PTR lpStat = @StatBar

'-------------------------------------------------------------------------------
' Forward declares for event Subs
'-------------------------------------------------------------------------------
DECLARE SUB TBar_Click(Btn AS integer)
DECLARE SUB Exit_Click(Sender AS QBUTTON)

'-------------------------------------------------------------------------------
' Initialisation of objects
'-------------------------------------------------------------------------------
SUB ConfigComponents
    lpForm->objIndex=0
    lpForm->Width=500: lpForm->Height=400
    lpForm->Caption=Application.Name
    lpForm->Visible=1

    lpButn->objIndex=NUMBTNS+1
    lpButn->Left=200: lpButn->Top=320
    lpButn->Width=100: lpButn->Height=25
    lpButn->Caption="Exit"
    lpButn->OnClick=CAST(ANY PTR, @Exit_Click)

    lpStat->objIndex=NUMBTNS+2
    lpStat->Height=19
    lpStat->SimplePanel=1
    lpStat->SimpleText="Simple Text"
    lpStat->SizeGrip=1
END SUB

'-------------------------------------------------------------------------------
' Creation of objects
'-------------------------------------------------------------------------------
SUB AddToolBar(Parent AS HWND)
DIM AS QIMAGELIST imgList
DIM AS TBBUTTON tbBtn
DIM AS UINT ExtStyle=0                          ' 0, WS_EX_DLGMODALFRAME, WS_EX_CLIENTEDGE
DIM AS UINT StdStyle=WS_THICKFRAME              ' 0, WS_THICKFRAME, TBSTYLE_FLAT ... more

    ' Create the toolbar
    hToolBar=CreateWindowEx(ExtStyle, "ToolbarWindow32", "", WS_CHILD OR WS_VISIBLE OR StdStyle, _
                                                        0, 0, 0, 0, Parent, CAST(HMENU, NUMBTNS), 0, 0)
    SendMessage(hToolBar, TB_BUTTONSTRUCTSIZE, SIZEOF(TBBUTTON), NULL)          ' Identifies which vers of dll

    ' Create the ImageList -  Actual images are 32x32 pix, auto-sized to fit width & height 
#IFDEF MULTI_IMAGE
    imgList.Width=32
    imgList.Height=16
    imgList.AddBMPFile("images\AllBtns.bmp", 4, &hff00ff)                              ' Image strip of 4 images
#ELSE
    imgList.Width=16
    imgList.Height=16
    imgList.AddBMPFile("images\FileNew.bmp", 1, &hff00ff)    
    imgList.AddBMPFile("images\FileOpen.bmp", 1, &hff00ff)
    imgList.AddBMPFile("images\FileSave.bmp", 1, &hff00ff)
    imgList.AddBMPFile("images\Exit.bmp", 1, &hff00ff)
#ENDIF

    SendMessage(hToolBar, TB_SETIMAGELIST, 0, CAST(LPARAM,imgList.Handle))      ' Set ImgList to toolbar
    FOR i AS integer = 0 TO NUMBTNS-1
        tbBtn.iBitmap = i                                                       ' Index of image in ImgList
        tbBtn.idCommand = i+1                                                   ' id in WM_COMMAND msg (1 -> NUMBTNS)
        IF i<3 THEN
            tbBtn.fsStyle = BTNS_CHECKGROUP                                     ' A Group with dual state buttons
        ELSE
            tbBtn.fsStyle = BTNS_CHECK                                          ' A single dual state button
        END IF
        tbBtn.fsState = TBSTATE_ENABLED
        tbBtn.dwData = 0
        SendMessage(hToolBar, TB_ADDBUTTONS, 1, CAST(LPARAM, @tbBtn))           ' Add it to toolbar
        IF i=2 THEN                                                             ' AFTER 3rd button
            tbBtn.iBitmap = 15                                                  ' <<<< WIDTH of separator
            tbBtn.fsStyle = TBSTYLE_SEP
            tbBtn.idCommand = -1
            SendMessage(hToolBar, TB_ADDBUTTONS, 1, CAST(LPARAM, @tbBtn))       ' Add it to toolbar
        END IF
    NEXT i
    SendMessage(hToolBar, TB_AUTOSIZE, 0 ,0)                                    ' Toolbar re-calculates itself
END SUB

SUB AddChildWindows(Parent AS HWND)
    AddToolBar(Parent)

    lpButn->Parent=Parent
    lpButn->Handle=CreateWindowEx(0, "BUTTON", lpButn->Caption, _
                                        WS_VISIBLE OR WS_CHILD OR BS_PUSHBUTTON, _
                                        lpButn->Left, lpButn->Top, lpButn->Width, lpButn->Height, _
                                        Parent, CAST(HMENU,lpButn->objIndex), 0, 0)

    lpStat->Parent=Parent
    lpStat->Handle = CreateWindowEx(0, "msctls_statusbar32", "Status", _
                                        WS_VISIBLE OR WS_CHILD, _
                                        lpStat->Left, lpStat->Top, lpStat->Width, lpStat->Height, _
                                        Parent, CAST(HMENU,lpStat->objIndex), 0, 0)
END SUB

FUNCTION WndProc(hWnd AS HWND, uMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM) AS LRESULT

    SELECT CASE uMsg
        CASE WM_CREATE
            AddChildWindows(hWnd)

        CASE WM_PAINT
            DIM AS RECT rct                                                 ' Auto adjust statusbar to form
            DIM AS integer ht1, ht2
            GetClientRect(hToolBar, @rct)
            ht1 = rct.Bottom-rct.Top+1                                      ' Height of status bar
            GetClientRect(lpStat->Handle, @rct)
            ht2 = rct.Bottom-rct.Top+1                                      ' Height of status bar

            GetClientRect(hWnd, @rct)
            MoveWindow(hToolBar, rct.Left, rct.Bottom-ht1, rct.Right-rct.Left+1, ht1, 1)
            MoveWindow(lpStat->Handle, rct.Left, rct.Bottom-ht2, rct.Right-rct.Left+1, ht2, 1)

        CASE WM_COMMAND
            SELECT CASE LOWORD(wParam)
                CASE 1 TO NUMBTNS
                    TBar_Click(LOWORD(wParam))

                CASE lpButn->ObjIndex
                    IF lpButn->OnClick>0 THEN lpButn->OnClick(lpButn)
            END SELECT

        CASE WM_DESTROY
            PostQuitMessage(0)

    END SELECT

    RETURN DefWindowProc(hWnd, uMsg, wParam, lParam)
END FUNCTION


' Main program loop
FUNCTION WinMain(hInst AS HINSTANCE, hPrevInstance AS HINSTANCE, _
                            szCmdLine AS string, iCmdShow AS integer) AS integer
DIM AS MSG wMsg
DIM AS WNDCLASSEX wc

WITH wc
    .cbSize=SizeOf(WNDCLASSEX)
    .style=CS_HREDRAW OR CS_VREDRAW
    .lpfnWndProc=@WndProc
    .hInstance=hInst
    .hIcon=LoadIcon(0,IDI_APPLICATION)
    .hCursor=LoadCursor(0,IDC_ARROW)
    .hbrBackground=Cast(HBRUSH,COLOR_WINDOW)
    .lpszClassName=@"RQClass"
    .hIconSm=.hIcon
END WITH

    IF RegisterClassEx(@wc)=0 THEN
        ShowMessage("Register class error")
        RETURN 0
    END IF

    ConfigComponents

    lpForm->Parent=0
    lpForm->Handle=CreateWindowEx(0, "RQClass", lpForm->Caption, _
                                    WS_VISIBLE OR WS_OVERLAPPEDWINDOW, _
                                    lpForm->Left, lpForm->Top, lpForm->Width, lpForm->Height, _
                                    0, 0, hInst, 0)
    Application.hMainForm=lpForm->Handle                                ' Need this for the terminate fn
    lpForm->Center

    WHILE GetMessage(@wMsg, NULL, 0, 0) <> FALSE
        TranslateMessage(@wMsg)
        DispatchMessage(@wMsg)
    WEND

    RETURN wMsg.wParam
END FUNCTION
'
' Main Entry Point
'
END WinMain(Application.hInstance, NULL, Command(), SW_NORMAL)

'-------------------------------------------------------------------------------
' User added event Subs
'-------------------------------------------------------------------------------
SUB TBar_Click(Btn AS integer)
    lpStat->SimpleText="Button " & Btn
END SUB

SUB Exit_Click(Sender AS QBUTTON)
    Application.Terminate
END SUB

