'-----------------------------------------------------------------------------------------------------------------
'  PROJECT: RapidFRM.exe
'  Form Designer/Editor for RapidQ Basic (with limited support for FreeBasic)
'  Copyright (c) 2009-13 Don Homans
'
'  CONTRIBUTORS:    Jordi Ramos - original code base (v1.00 assigned)
'                   JohnK - Floating component panel
'  AUTHOR:          d_homans@yahoo.com.au
'  VERSION:         1.85
'  DATE:            27 June 2013
'  COMMENT:
'-----------------------------------------------------------------------------------------------------------------
$APPTYPE GUI
$TYPECHECK ON
$OPTION ICON "images\RapidFRM.ico"                                  ' Using RapidQ icon

$INCLUDE <RapidQ2.inc>

$INCLUDE "RapidFRM.inc"                                             ' My MouseHook, ColorDialog & TabControl stuff
$DEFINE MAXTOOLBTNS     15
$INCLUDE "QToolBarEX.inc"                                           ' My toolbar component (with QImageListEX.inc)

$RESOURCE ObjData AS "component.data"                               ' Resource(0) used in initialise()
$RESOURCE HelpData AS "Help.data"                                   ' Resource(1) used in initialise()
$RESOURCE BlankIco AS "images\Blank.ico"                            ' Blank form icon
$RESOURCE TBar1 AS "images\ToolBar1.bmp"
$RESOURCE TBar2 AS "images\ToolBar2.bmp"
$RESOURCE TBar3 AS "images\ToolBar3.bmp"
$RESOURCE TBar4 AS "images\ToolBar4.bmp"
$RESOURCE TBar5 AS "images\ToolBar5.bmp"
$RESOURCE XBtns AS "images\ToolBar6.bmp"

CONST VER="v1.85"
CONST CRLF=CHR$(13)+CHR$(10)
CONST QU=CHR$(34)

$DEFINE STAND_ALONE     0                                           '
$DEFINE INPUT_FILE      1                                           ' Modes of operation
$DEFINE INPUT_CLIPBRD   2                                           '

$DEFINE MAX_OBJECTS     64                                          ' Limit of components (including forms)
$DEFINE MAX_FORMS       8                                           ' This limit seems reasonable
$DEFINE MAX_MENUS       8                                           ' Overkill?

$DEFINE NON_VISIBLE     30                                          ' 0->29 Visible, 30->55 non-visible
$DEFINE MAX_TOOLBTNS    55                                          ' 5 TABs: 15 + 14 + 11 + 4 + 11

$DEFINE MAINMENU        1
$DEFINE POPMENU         2

$DEFINE SWP_NOSIZE      1
$DEFINE SWP_NOMOVE      2
$DEFINE SWP_NOZORDER    4
$DEFINE SWP_DRAWFRAME   &H20

CONST FLAGS=(SWP_NOSIZE + SWP_NOMOVE)
CONST SW_FLAGS=(SWP_NOZORDER + SWP_DRAWFRAME)
CONST SWP_FLAGS=(SW_FLAGS + SWP_NOMOVE)

''$DEFINE DBG_INPUT_STREAM

'------------------- Forward Declarations ---------------------------------------------
DECLARE FUNCTION WaitOnHelp LIB "kernel32" ALIAS "WaitForSingleObject" (hHandle AS long, _
                                                                        dwMilliseconds AS long) AS long

' General stuff
DECLARE SUB Initialise
DECLARE FUNCTION GetScreenHeight() AS integer
DECLARE SUB FormPopUp(X AS integer, Y AS integer)
DECLARE SUB ObjPopUp(X AS integer, Y AS integer)
DECLARE SUB NonVisPopup(X AS integer, Y AS integer)
DECLARE SUB DesignTabChange                                         ' Active design window
DECLARE SUB SelectComboChange                                       ' Component selection combo
DECLARE SUB ComponentTabChange                                      ' Component properties/methods/events
DECLARE SUB MainClose(Action AS integer)
DECLARE SUB NoClose(Action AS integer)
DECLARE SUB HelpBtnClick
DECLARE FUNCTION FindHelpFile() AS integer
DECLARE SUB ProgramEnd
' Project file import/export
DECLARE SUB LoadProject                                             ' Load & parse the project file/clipboard
DECLARE SUB ShowProject                                             ' Display project data
DECLARE SUB BuildFormCode                                           ' Output RapidQ code (Default)
DECLARE SUB BuildFBCode                                             ' Output FreeBasic code
' Button stuff
DECLARE SUB LoadSingleButtons                                       ' Additional separate buttons
DECLARE SUB LoadToolbarHints                                        ' 'TAB'ed Toolbar buttons
DECLARE SUB ButnCmd(idx AS integer)
DECLARE SUB ToolTabChange
DECLARE SUB AboutClick
DECLARE SUB Deselect
' PopUp menu options
DECLARE SUB HelpOnObject
DECLARE SUB AlignObject                                             ' Visual editing options
DECLARE SUB MouseCopyObject                                         ' ditto
DECLARE SUB MousePasteObject                                        ' ditto
DECLARE SUB DeleteComponent                                            ' ditto
DECLARE SUB CopyToClipBrd                                           ' STAND_ALONE mode options
DECLARE SUB SelectionToClipBrd                                      ' ditto
DECLARE SUB SaveTemplate                                            ' FreeBASIC save RQ template file
DECLARE SUB SaveFile                                                ' ditto
DECLARE SUB AskExit
DECLARE SUB ExitNoUpdate                                            ' ditto
DECLARE SUB SaveAndExit                                             ' Existing Project option
DECLARE SUB FB_SaveAndExit                                          ' FreeBasic save to clipboard
' Forms() - ALL forms in the array
DECLARE SUB FormsMouseMove(X AS long, Y AS long, Shift AS long, Sender AS QFORM)
DECLARE SUB ReSizeForm(Sender AS QFORM)
DECLARE SUB PaintFormGrid(Sender AS QFORM)
' Caret manipulation
DECLARE SUB CaretMseDown(MouseButton AS long, X AS long, Y AS long, Shift AS long, Sender AS QPANEL)
DECLARE SUB CaretMseUp(MouseButton AS long, X AS long, Y AS long, Shift AS long, Sender AS QPANEL)
DECLARE SUB CaretsOnTop
DECLARE SUB ShowCarets(tgtRect AS QRECT)
DECLARE SUB HideCarets
' Object - Mouse editing
DECLARE SUB ScanMouse
DECLARE SUB MouseResizeObject(hWnd AS long)
DECLARE SUB MouseMoveObject(hWnd AS long)
DECLARE FUNCTION HandleToIndex(hWnd AS long) AS integer
' Object - Grid editing
DECLARE SUB UpdatePropGrid(idx AS integer)
DECLARE SUB EditPropertyField(Col AS integer, Row AS integer, CanSelect AS integer, Sender AS QSTRINGGRID)
DECLARE SUB EditMethodField(Col AS integer, Row AS integer, CanSelect AS integer, Sender AS QSTRINGGRID)
DECLARE SUB EditEventField(Col AS integer, Row AS integer, CanSelect AS integer, Sender AS QSTRINGGRID)
DECLARE SUB FieldUpdateCombo(Sender AS QCOMBOBOX)
DECLARE SUB FieldUpdateEdit(Key AS word, Shift AS integer, Sender AS QEDIT)
DECLARE SUB ReDrawComponent
DECLARE SUB HideEditCtls(Col AS integer, Row AS integer, State AS integer, Rect AS QRect, Sender AS QSTRINGGRID)

' MainMenu & PopupMenu Editing
DECLARE SUB MenuEditor
DECLARE SUB AddMainMenu
DECLARE SUB AddPopupMenu
DECLARE SUB UpdateProjectMenu
DECLARE SUB ParentChange
DECLARE SUB OptionChange
' Adding new components
DECLARE SUB AddNewObject
DECLARE SUB AddNewForm
DECLARE FUNCTION GetDefaultProperties(ComponentID AS integer) AS string

'---------------------------- Globals -------------------------------------------------------
' General vars
DEFINT ProjectMode                                                  ' New, existing project file or clipboard
DEFINT ErrorCondition                                               ' Error True/False flag
DEFINT ComponentCount                                               ' Counter for save/exit action

DEFSTR RQHelpFile                                                   ' Full path to Rapidq.chm
DEFSTR ProjectFile                                                  ' Filename parameter input - existing project
DIM ProjectStream AS QMEMORYSTREAM                                  ' Holds a Project file/clipboard text if needed
DIM ComponentRef AS QMEMORYSTREAM                                   ' Stream to hold component default properties
DIM HelpRef AS QMEMORYSTREAM                                        ' Contains the help text

DIM FB_InputList AS QSTRINGLIST

' Vars for parsing CREATE .. END CREATE blocks of code
DEFINT StartOfBlocks, EndOfBlocks                                   ' Markers in Project file/clipboard text
DEFSTR FormsList                                                    ' List of Forms to display
DEFSTR KnownsList                                                   ' List of objects outside of forms (known parents)
DEFSTR MainMenuList                                                 ' List of QMAINMENU components found
DEFSTR PopMenuList                                                  ' List of QPOPUPMENU components found
DEFSTR UnknownList                                                  ' List of "unknown" components found eg ...Ex types
DEFSTR FwdDeclares                                                  ' New component events list
DEFSTR ExistDeclares                                                ' Existing declares if input file or clipboard
DEFSTR MenuUpdateStr

' Vars for visual display
DIM ObjectList(0 TO MAX_OBJECTS) AS string                          ' Components added (with all properties)...
                                                                    ' ...0=Main form -> Forms(1)

DIM Forms(1 TO MAX_FORMS) AS QFORM
DEFINT ThemeOffsetY                                                 ' Active TitleBar height adjustment

DEFSTR ToolbarHint                                                  ' Hints for all Toolbar buttons
DIM xtraBtns(0 TO 3) AS QCOOLBTN                                    ' 4 Additional single buttons
DIM NonVisBMP AS QBITMAP                                            ' "Non-visibles" temp bmp

DIM Carets(1 TO 8) AS QPANEL                                        ' Carets used in moving/resizing objects
DEFINT ActiveCaret, ReSizing                                        ' Resizing an object
DIM CaretsTimer AS QTIMER                                           ' Keeps carets() "on top"

DIM Scan AS QTIMER                                                  ' Main timer for visual editing
DIM DelayTimer AS QTIMER                                            ' One-shot timer used with input file or clipboard

DIM ColorPick AS QColorDlg                                          ' MY temp version (see RapidFRM.inc)
DEFINT MouseSaveX, MouseSaveY                                       ' Needed by copy/paste with QPopupMenus

DEFINT CurrentObject, CurrentForm                                   ' Currently active components
DEFINT NewObjectID, NonVisCount

' FB stuff
TYPE OBJ_STRUC
    FBLineNum AS integer                                            ' Line # in InList
    FBObjID AS integer                                              ' 0=QFORM, 1=QEDIT etc
    FBObjType AS string*16                                          ' "QFORM" etc
    FBObjName AS string*16                                          ' "Form1" etc
    FBObjPtrName AS string*16                                       ' "lpForm1" etc
    FBObjHandle AS long
    FBObjParent AS long
    FBChildCount AS integer
END TYPE
DIM FB_Objects(0 TO MAX_OBJECTS) AS OBJ_STRUC
DEFINT ObjCount=0
'
DEFINT ObjectIndex, FrmIndex, NonVIndex

' Visible components indices (QOvalBtn done differently - see RapidFRM.inc)
DEFINT EdIndex, RichIndex, LabelIndex, ButnIndex, CoolIndex
DEFINT CheckIndex, RadioIndex, ComboIndex, ListBIndex, PanIndex, GrpBoxIndex
DEFINT TabContIndex, ScrollBIndex, ScrllBarIndex, StringGIndex, ListVIndex
DEFINT HeadIndex, SplitIndex, StatIndex, GlassIndex, GaugeIndex, TrackBIndex
DEFINT OutLnIndex, DirTIndex, FileLIndex, TreeVIndex, OLEContIndex

' Non-visible components indices
DEFINT ImageIndex, ImgListIndex, CanvasIndex, OleObjIndex
DEFINT TimerIndex, SockIndex, FStreamIndex, MStreamIndex, FontIndex, SQLIndex
DEFINT BmpIndex, RegIndex, StrListIndex
' Dialog components
DEFINT OpenDlgIndex, SaveDlgIndex, FontDlgIndex, ColorDlgIndex
' DirectX components
DEFINT DXScreenIdx,DXImgListIdx,DXTimerIdx,DXSoundIdx,D3DFaceIdx
DEFINT MeshBuilderIdx,D3DFrameIdx,D3DLightIdx,TextureIdx,VectorIdx,D3DWrapIdx
' Menu indices
DEFINT MainMenuIndex, PopUpIndex

DIM fDialog AS QFileDialog

'------------------------------------ Main form -------------------------------------------
CREATE designMenu AS QPOPUPMENU
    CREATE mnuAlign AS QMENUITEM
        Caption="Align to Grid"
        OnClick=AlignObject
    END CREATE
    CREATE mnuCopy AS QMENUITEM
        Caption="Copy Component"
        OnClick=MouseCopyObject
    END CREATE
    CREATE mnuPaste AS QMENUITEM
        Caption="Paste Component"
        OnClick=MousePasteObject
    END CREATE
    CREATE mnuSep1 AS QMENUITEM
        Caption="-"
    END CREATE
    CREATE mnuDelete AS QMENUITEM
        Caption="Delete Component"
        OnClick=DeleteComponent
    END CREATE
    CREATE mnuSep2 AS QMENUITEM
        Caption="-"
    END CREATE
    CREATE mnuHelp AS QMENUITEM
        Caption="Help on Component"
        Enabled=False
        OnClick=HelpOnObject
    END CREATE
    CREATE mnuSep3 AS QMENUITEM
        Caption="-"
    END CREATE
    CREATE mnuQuit AS QMENUITEM
        Caption="Exit Designer"
        OnClick=AskExit
    END CREATE
END CREATE

CREATE codeMenu AS QPOPUPMENU
    CREATE edCopy AS QMENUITEM                                      ' Stand-alone version option
        Caption="Copy all to clipboard"
        OnClick=CopyToClipBrd
    END CREATE
    CREATE partCopy AS QMENUITEM                                    ' Stand-alone version
        Caption="Copy selection to clipboard"
        OnClick=SelectionToClipBrd
    END CREATE
    CREATE mnuSep4 AS QMENUITEM
        Caption="-"
    END CREATE
    CREATE tplSave AS QMENUITEM                                     ' FB all versions save as RQ src file
        Caption="Save FB template file"
        OnClick=SaveTemplate
    END CREATE
    CREATE edSave AS QMENUITEM                                      ' Stand-alone version
        Caption="Save to File"
        OnClick=SaveFile
    END CREATE
    CREATE mnuSep5 AS QMENUITEM
        Caption="-"
    END CREATE
    CREATE edPsave AS QMENUITEM                                     ' Project version option
        Caption="Save Project && Exit"
        OnClick=SaveAndExit
    END CREATE
    CREATE mnuSep6 AS QMENUITEM
        Caption="-"
    END CREATE
    CREATE edQuit AS QMENUITEM
        Caption="Exit Designer Without Changes"
        OnClick=ExitNoUpdate
    END CREATE
END CREATE

CREATE MainForm AS QFORM
    Width=Screen.Width
    Height=GetScreenHeight()                      'Screen.Height-28    ' GetSystemMetrics(SM_CYFULLSCREEN)?
    Left=0
    Top=0
    Caption="RapidFRM "+VER
    BorderStyle=bsSingle
    OnShow=Initialise
    OnClose=MainClose
    CREATE stat AS QSTATUSBAR
        SizeGrip=False
        AddPanels "", "", "", "", ""
        Panel(0).Width=MainForm.ClientWidth\2
        Panel(1).Width=MainForm.ClientWidth\12
        Panel(2).Width=MainForm.ClientWidth\12
        Panel(3).Width=MainForm.ClientWidth\12
    END CREATE
    CREATE ToolBox AS QPANEL
        Width=MainForm.ClientWidth
        Height=60
        Align=alTop
        Caption=""
        CREATE ToolBarTab AS QTABCONTROL
            Width=445                                               ' 15*29+10 - 15 btns plus
            Height=54
            Left=5
            Top=2
            Tabheight=16
            TabWidth=87                                             ' 15*29\5 - 5 tabs
            TabInactiveColor=&hbfbfbf
            Font.Size=8
            Font.Color=&h1f1fbf
            Addtabs "Standard","Additional","Non-visible","Dialogs","DirectX"
            OnChange=ToolTabChange
            CREATE ToolBar1 AS QToolBarEX
                BtnCount=15
                Cursor=crHandPoint
                Tag=1                                               ' Button # of first button
                BMPHandle=3                                         ' Resource(3)
                OnSelect=ButnCmd
            END CREATE
            CREATE ToolBar2 AS QToolBarEX
                BtnCount=14
                Cursor=crHandPoint
                Tag=16                                              ' Button # of first button
                Visible=False
                BMPHandle=4                                         ' Resource(4)
                OnSelect=ButnCmd
            END CREATE
            CREATE ToolBar3 AS QToolBarEX
                BtnCount=11
                Cursor=crHandPoint
                Tag=30                                              ' Button # of first button
                Visible=False
                BMPHandle=5                                         ' Resource(5)
                OnSelect=ButnCmd
            END CREATE
            CREATE ToolBar4 AS QToolBarEX
                BtnCount=4
                Cursor=crHandPoint
                Tag=41                                              ' Button # of first button
                Visible=False
                BMPHandle=6                                         ' Resource(6)
                OnSelect=ButnCmd
            END CREATE
            CREATE ToolBar5 AS QToolBarEX
                BtnCount=11
                Cursor=crHandPoint
                Tag=45                                              ' Button # of first button
                Visible=False
                BMPHandle=7                                         ' Resource(7)
                OnSelect=ButnCmd
            END CREATE
        END CREATE
        CREATE NonVisPanel AS QGROUPBOX                             ' Container for non-visible components
            Left=450+6*30                                           ' Toolbar tabs + 4 * buttons + 2 * button gap
            Top=5
            Width=ToolBox.Width-NonVisPanel.Left-45
            Height=50
            Caption="Project non-visible components"
        END CREATE
    END CREATE
    CREATE DesignPan AS QPANEL
        Width=Screen.Width*3\4
        Height=MainForm.ClientHeight-ToolBox.Height-stat.Height
        Left=0
        Top=ToolBox.Height
        CREATE DesignTab AS QTABCONTROL
            Width=DesignPan.ClientWidth
            Height=DesignPan.ClientHeight
            Align=alTop
            TabHeight=24
            TabWidth=100
            TabInactiveColor=&hbfbfbf
            Font.Color=&h7f7f00
            AddTabs "Form Designer","RapidQ Coder","Menus","FreeBasic Coder","Help"
            OnChange=DesignTabChange
            CREATE FormsPanel AS QPANEL                             ' Container for form designer
                Align=alClient
                BevelWidth=2
                BorderStyle=bvLowered
                Color=&hdfdfdf
            END CREATE
            CREATE CodeText AS QRichEdit                            ' Code text screen RapidQ
                Align=alClient
                ScrollBars=ssBoth
                Color=&hffffef
                Font.Name="courier"
                Font.Size=10
                Font.Color=&hbf1f1f
                PlainText=True
                PopUpMenu=codeMenu
                Visible=False
            END CREATE
            ' NB For Menu Designer TAB - see MenuPanel below
            CREATE CodeText2 AS QRichEdit                            ' Code text screen FreeBasic
                Align=alClient
                ScrollBars=ssBoth
                Color=&hffffef
                Font.Name="courier"
                Font.Size=10
                Font.Color=&hbf1f1f
                PlainText=True
                PopUpMenu=codeMenu
                Visible=False
            END CREATE
            CREATE HelpText AS QRichEdit                            ' Help screen
                Align=alClient
                ScrollBars=ssVertical
                Color=&hefffef
                Font.Name="courier"
                Font.Size=10
                Font.Color=&h004f00
                PlainText=True
                ReadOnly=True
                Visible=False
            END CREATE
        END CREATE
    END CREATE
    CREATE ComponentPan AS QPANEL
        Left=DesignPan.Width
        Top=DesignPan.Top
        Width=Screen.Width-DesignPan.Width
        Height=DesignPan.ClientHeight
        CREATE SelectionCombo AS QCOMBOBOX
            Align=alTop
            Height=24
            Width=ComponentPan.ClientWidth-4
            Style=csDropDownList
            OnChange=SelectComboChange
        END CREATE
        CREATE ComponentTab AS QTABCONTROL
            Align=alClient
            AddTabs "Properties","Methods","Events"
            OnChange=ComponentTabChange
            CREATE PropertyGrid AS QSTRINGGRID                      ' Properties of selected component
                Align=alClient
                RowCount=100
                DefaultRowHeight=16
                FixedRows=1
                FixedCols=1
                ColCount=2
                ColWidths(0)=100
                ColWidths(1)=ComponentTab.ClientWidth-120
                ColumnStyle(0)=gcsNone
                ColumnStyle(1)=gcsNone
                Cell(0,0)="PROPERTY"
                Cell(1,0)="VALUE"
                ScrollBars=ssVertical
                OnSelectCell=EditPropertyField
                OnDrawCell=HideEditCtls
            END CREATE
            CREATE MethodGrid AS QSTRINGGRID                        ' Methods of selected component
                Align=alClient
                RowCount=100
                DefaultRowHeight=16
                FixedRows=1
                FixedCols=1
                ColCount=2
                ColWidths(0)=100
                ColWidths(1)=ComponentTab.ClientWidth-120
                ColumnStyle(0)=gcsNone
                ColumnStyle(1)=gcsNone
                Cell(0,0)="METHOD"
                Cell(1,0)="ASSIGNMENT"
                ScrollBars=ssVertical
                OnSelectCell=EditMethodField
                OnDrawCell=HideEditCtls
            END CREATE
            CREATE EventGrid AS QSTRINGGRID                         ' Events of selected component
                Align=alClient
                RowCount=100
                DefaultRowHeight=16
                FixedRows=1
                FixedCols=1
                ColCount=2
                ColWidths(0)=100
                ColWidths(1)=ComponentTab.ClientWidth-120
                ColumnStyle(0)=gcsNone
                ColumnStyle(1)=gcsNone
                Cell(0,0)="EVENT"
                Cell(1,0)="SUBROUTINE"
                ScrollBars=ssNone
                Visible=False
                OnSelectCell=EditEventField
                OnDrawCell=HideEditCtls
            END CREATE
            CREATE FieldEdit AS QEDIT                               ' Editing a single field
                Left=108
                Width=PropertyGrid.ColWidths(1)-24
                Height=PropertyGrid.DefaultRowHeight
                OnKeyDown=FieldUpdateEdit
                Visible=False
            END CREATE
            CREATE FieldBox AS QCOMBOBOX                            ' Editing a multi-choice field
                Left=108
                Width=PropertyGrid.ColWidths(1)-24
                Height=PropertyGrid.DefaultRowHeight
                DropDownCount=5
                Style=csDropDownList
                OnChange=FieldUpdateCombo
                Visible=False
            END CREATE
        END CREATE
    END CREATE
END CREATE

CREATE MenuPanel AS QPANEL                                          ' Simple menu template screen
    Parent=DesignTab
    Align=alClient
    BevelWidth=2
    BorderStyle=bvLowered
    Color=&hdfffff
    Visible=False
    CREATE MenuText AS QRichEdit
        Width=MenuPanel.ClientWidth*2\3
        Height=MenuPanel.ClientHeight-150
        Left=(MenuPanel.Width-MenuText.Width)\2
        Top=MenuPanel.Top+100
        ScrollBars=ssVertical
        Color=&hcfefef
        Font.Name="courier"
        Font.Size=10
        Font.Color=&h1f3f3f
        PlainText=True
        ReadOnly=True
    END CREATE
    CREATE btnMMenu AS QBUTTON
        Width=145
        Left=MenuText.Left
        Top=20
        Caption="Add QMainMenu"
        Font.Color=&h009f00
        Font.Bold=3
        OnClick=AddMainMenu
    END CREATE
    CREATE cbMMenu AS QCOMBOBOX
        Left=MenuText.Left
        Top=50
        Font.Color=0
        OnChange=ParentChange
    END CREATE
    CREATE btnPMenu AS QBUTTON
        Width=145
        Left=MenuText.Left+MenuText.Width-145
        Top=20
        Caption="Add QPopupMenu"
        Font.Color=&h009f00
        Font.Bold=3
        OnClick=AddPopupMenu
    END CREATE
    CREATE btnUpdate AS QBUTTON
        Width=100
        Height=60
        Left=MenuText.Left+MenuText.Width\2 - 50
        Top=20
        Caption="Insert code"
        Font.Color=&hbf
        Font.Bold=3
        Tag=0
        OnClick=UpdateProjectMenu
    END CREATE
    CREATE chkMenu AS QCHECKBOX
        Width=180
        Left=MenuText.Left+MenuText.Width\2 - 90
        Top=110
        Caption="Add 'non-defaults' as comments"
        Font.Color=0
        Checked=False
        OnClick=OptionChange
    END CREATE
END CREATE

$INCLUDE "RapidFRM_Mods.inc"                                        ' Courtesy of JohnK

MainForm.ShowModal

'--------------------------------Common functions-----------------------------------------
FUNCTION TRIM$(inpString AS string) AS string
    Result=LTRIM$(RTRIM$(inpString))
END FUNCTION

' Which index of ObjectList() belongs to this component handle
FUNCTION HandleToIndex(hWnd AS long) AS integer
DEFINT i
DEFSTR tstr

    Result = -1
    FOR i=0 TO MAX_OBJECTS
        IF ObjectList(i)="" THEN EXIT FOR
        tstr=FIELD$(ObjectList(i),"~",1)
        IF VAL(FIELD$(tstr,"|",3))=hWnd THEN
            Result=i
            EXIT FUNCTION
        END IF
    NEXT i
END FUNCTION

' Update StatusBar with CurrentObject Info & CurrentForm
SUB UpdateStatus
DEFSTR tstr, tok, strCaption
DEFINT ObjType
DEFLNG hWnd

    tstr=FIELD$(ObjectList(CurrentObject),"~",1)
    strCaption="Selection("+STR$(CurrentObject)+") :: Type="+FIELD$(tstr,"|",2)+ _
                " :: Name="+FIELD$(tstr,"|",5)+" :: Handle="
    ObjType=VAL(FIELD$(tstr,"|",1))
    SELECT CASE ObjType
        CASE Q_COOLBTN,Q_OVALBTN,Q_TABCONTROL,Q_DIRTREE,Q_FILELISTBOX,Q_GAUGE, _
                        Q_CANVAS,Q_IMAGE,Q_SPLITTER,NON_VISIBLE TO MAX_TOOLBTNS     ' Objects without "valid" handle
            strCaption=strCaption+"Undefined"
        CASE ELSE
            strCaption=strCaption+FIELD$(tstr,"|",3)
            IF objType>0 THEN
                hWnd=VAL(FIELD$(tstr,"|",3))
                hWnd=GetParent(hWnd)
                strCaption=strCaption+" :: Parent="+STR$(hWnd)
            END IF
    END SELECT
    stat.Panel(0).Caption=strCaption
    stat.Panel(1).Caption="  Active Form = "+STR$(CurrentForm)
END SUB

' Deselect everything - Current Form active
SUB Deselect
DEFLNG hWnd
DEFINT i
DEFSTR tstr

    HideCarets                                                      ' If visible
    FieldEdit.Visible=False
    FieldBox.Visible=False
    DesignTab.TabIndex=0
    DesignTabChange
    IF designMenu.Tag>0 THEN                                        ' Copying something?
        designMenu.Tag=0                                            ' Cancel it
        FOR i=1 TO MAX_FORMS
            Forms(i).Cursor=crDefault                               ' Cursor back to default
        NEXT i
    END IF
    hWnd=Forms(CurrentForm).Handle                                  ' Default to current form
    FOR i=0 TO ObjectIndex-1
        tstr=FIELD$(ObjectList(i),"~",1)
        IF VAL(FIELD$(tstr,"|",3))=hWnd THEN
            IF CurrentObject<>i THEN                                ' Not current form?
                CurrentObject=i                                     ' Set to current form
                UpdatePropGrid(i)                                   ' Update prop grid
                UpdateStatus
            END IF
            EXIT FOR
        END IF
    NEXT i
    NewObjectID=-1
END SUB

' Returns true for objects that can have child objects (excluding forms)
FUNCTION IsTypeParent(ThisIdx AS integer) AS integer
DEFSTR tstr

    tstr=FIELD$(ObjectList(ThisIdx),"~",1)                          ' First line of this object string
    SELECT CASE FIELD$(tstr,"|",2)
        CASE "QPANEL","QTABCONTROL","QGROUPBOX","QSCROLLBOX"        ' NB do NOT include QFORMs
            Result=True
        CASE ELSE
            Result=False
    END SELECT
END FUNCTION

' Get all direct siblings of this "start" object (non-parent types first)
FUNCTION FillSiblingList(StartIndex AS integer) AS string
DEFINT i
DEFSTR tstr, strParent, sibResult

    sibResult=""
    tstr=FIELD$(ObjectList(StartIndex),"~",1)
    strParent=FIELD$(tstr,"|",3)                                    ' Get handle of the start object
    FOR i=StartIndex+1 TO ObjectIndex-1
        IF IsTypeParent(i)=False THEN
            tstr=FIELD$(ObjectList(i),"~",1)
            IF strParent=FIELD$(tstr,"|",6) THEN                    ' Is this a sibling?
                sibResult=sibResult+STR$(i)+"|"                     ' Add its index to the list
            END IF
        END IF
    NEXT i
    FOR i=StartIndex+1 TO ObjectIndex-1
        IF IsTypeParent(i)=True THEN
            tstr=FIELD$(ObjectList(i),"~",1)
            IF strParent=FIELD$(tstr,"|",6) THEN                    ' Is this a sibling?
                sibResult=sibResult+STR$(i)+"|"                     ' Add its index to the list
            END IF
        END IF
    NEXT i
    Result=sibResult
END FUNCTION

' Recursively called to delete parent/sibling chains
SUB DeleteThisChain(StartIdx AS integer)
DEFINT i, cnt, nxt
DEFSTR tstr, sibString
DEFLNG hWnd

    sibString=FillSiblingList(StartIdx)                             ' Get list of child object indices
    cnt=TALLY(sibString,"|")
    IF cnt>0 THEN
        FOR i=1 TO cnt                                              ' Loop all child objects
            nxt=VAL(FIELD$(sibString,"|",i))                        ' Get next object index
            tstr=FIELD$(ObjectList(nxt),"~",1)
            SELECT CASE FIELD$(tstr,"|",2)
                CASE "QPANEL","QTABCONTROL","QGROUPBOX","QSCROLLBOX"    ' Not deleted ATT (may have siblings)
                CASE ELSE
                    hWnd=VAL(FIELD$(tstr,"|",3))
                    ShowWindow(hWnd,0)                              ' Delete the object
                    ObjectList(nxt)=""
            END SELECT
        NEXT i
        FOR i=1 TO cnt                                              ' Loop all "parent type" child objects
            nxt=VAL(FIELD$(sibString,"|",i))                        ' Get next object index
            tstr=FIELD$(ObjectList(nxt),"~",1)
            SELECT CASE FIELD$(tstr,"|",2)
                CASE "QPANEL","QTABCONTROL","QGROUPBOX","QSCROLLBOX"
                    DeleteThisChain(nxt)                            ' Recursive call to self
            END SELECT
        NEXT i
    END IF
    tstr=FIELD$(ObjectList(StartIdx),"~",1)                         ' Delete the initial parent object
    hWnd=VAL(FIELD$(tstr,"|",3))
    ShowWindow(hWnd,0)
    ObjectList(StartIdx)=""
END SUB

' Update ObjectList property (pField) with its new property value (vField) in string position
SUB PropertyFieldUpdate(pField AS string, vField AS string, position AS integer)
DEFSTR propString, token, tstr, resString
DEFINT i, j, k, cnt

    propString=ObjectList(CurrentObject)
    cnt=TALLY(propString,"~"): resString=""
    FOR i=1 TO cnt
        token=FIELD$(propString,"~",i)
        IF FIELD$(token,"|",1)=pField THEN                          ' Found this property string
            k=TALLY(token,"|")+1
            tstr=""
            FOR j=1 TO k
                IF j=position THEN                                  ' This value replaced
                    tstr=tstr+vField+"|"
                ELSE
                    tstr=tstr+FIELD$(token,"|",j)+"|"
                END IF
            NEXT j
            token=LEFT$(tstr,LEN(tstr)-1)                           ' Remove trailing "|"
        END IF
        resString=resString+token+"~"
    NEXT i
    ObjectList(CurrentObject)=resString                             ' Update the global properties string
END SUB

'--------------------- Editing objects visually ----------------------------------------
SUB RefreshObject(hWnd AS long, nuRect AS QRECT)
DEFINT Width, Height
DEFLNG Parnt
DIM tPoint AS PointAPI

    Width=nuRect.Right-nuRect.Left
    Height=nuRect.Bottom-nuRect.Top
    tPoint.x=nuRect.Left: tPoint.y=nuRect.Top
    Parnt=GetParent(hWnd)
    ScreenToClient(Parnt,tPoint)

    PropertyFieldUpdate("Left", STR$(tPoint.x),4)
    PropertyFieldUpdate("Top", STR$(tPoint.y),4)
    PropertyFieldUpdate("Width", STR$(Width),4)
    PropertyFieldUpdate("Height", STR$(Height),4)
    UpdatePropGrid(CurrentObject)
    ReDrawComponent
END SUB

SUB AlignObject
DEFINT i, Top, Left, deltaX, deltaY
DEFSTR tstr, propStr

    HideCarets                                                      ' If visible
    tstr=ObjectList(CurrentObject)                                  ' Component global string data
    FOR i=1 TO TALLY(tstr,"~")                                      ' For each property this component
        propStr=FIELD$(tstr,"~",i)
        SELECT CASE FIELD$(propStr,"|",1)                           ' Get current left/top values
            CASE "Left": Left=VAL(FIELD$(propStr,"|",4))
            CASE "Top": Top=VAL(FIELD$(propStr,"|",4))
        END SELECT
    NEXT i
    deltaX=(Left MOD 10): Left=Left-deltaX                          ' Adjust each value to grid
    IF deltaX>4 THEN Left=Left+10
    deltaY=(Top MOD 10): Top=Top-deltaY
    IF deltaY>4 THEN Top=Top+10

    PropertyFieldUpdate("Left",STR$(Left),4)                        ' Update global string data
    PropertyFieldUpdate("Top",STR$(Top),4)
    UpdatePropGrid(CurrentObject)                                   ' Update property grid fields
    ReDrawComponent                                                 ' Redraw in new position
END SUB

SUB MouseCopyObject
DEFINT i
DEFSTR tstr

    tstr=FIELD$(ObjectList(CurrentObject),"~",1)                    ' Get the objectID of this component
    designMenu.Tag=VAL(FIELD$(tstr,"|",1))                          ' Store the ObjectID in the Tag property
    IF designMenu.Tag>0 THEN                                        ' NOT Forms (ID=0)!
        FOR i=1 TO MAX_FORMS
            Forms(i).Cursor=crDrag                                  ' Show we are copying
        NEXT i
    END IF
END SUB

SUB MousePasteObject
DEFINT i

    FOR i=1 TO MAX_FORMS
        Forms(i).Cursor=crDefault                                   ' Cursor back to default
    NEXT i
    IF designMenu.Tag>0 THEN                                        ' NOT Forms (ID=0)!
        NewObjectID=designMenu.Tag                                  ' The pre-set object we are adding
        designMenu.Tag=0                                            ' Re-set this flag
        SetCursorPos(MouseSaveX,MouseSaveY)                         ' Re-adjust for popup menu position
        MouseSaveX=0: MouseSaveY=0
        AddNewObject                                                ' Add the object to the active form
    END IF
END SUB

SUB ReDrawNonVisibles
DEFINT i, j, x, objType
DEFLNG hWnd
DEFSTR tstr

    IF NonVisCount=0 THEN EXIT SUB
    j=0
    FOR i=1 TO MAX_OBJECTS                                          ' Start at second object
        tstr=FIELD$(ObjectList(i),"~",1)
        IF tstr="" THEN EXIT FOR
        objType=VAL(FIELD$(tstr,"|",1))                             ' Object type
        IF objType>=NON_VISIBLE THEN
            hWnd=VAL(FIELD$(tstr,"|",3))
            x=j*32 + 5: INC j                                       ' Left property
            SetWindowPos(hWnd, 0, x, 16, 30, 30, SW_FLAGS)          ' Reposition object
        END IF
    NEXT i
END SUB

' Currently forms cannot be deleted
SUB DeleteComponent
DEFINT i, j, objType
DEFLNG hWnd
DEFSTR tstr

    IF CurrentObject=0 THEN                                         ' Just in case! NEVER the initial Form
        MessageBox("You cannot delete this form!","Delete Component",&h10)
        EXIT SUB
    END IF
    HideCarets                                                      ' Hide if visible

    tstr=FIELD$(ObjectList(CurrentObject),"~",1)                    ' Check for non-visible objects
    objType=VAL(FIELD$(tstr,"|",1))                                 ' Object type
    IF objType>=NON_VISIBLE THEN
        hwnd=VAL(FIELD$(tstr,"|",3))
        ShowWindow(hWnd,0)                                          ' Delete the object
        ObjectList(CurrentObject)=""
        IF NonVisCount>0 THEN DEC NonVisCount                       ' Should ALWAYS be >0 to get here (but safety)
    ELSE
        DeleteThisChain(CurrentObject)                              ' Deletes this object plus any siblings
    END IF

    i=CurrentObject
    FOR j=i+1 TO ObjectIndex
        IF ObjectList(j)<>"" THEN                                   ' Tidy the ObjectList()...
            ObjectList(i)=ObjectList(j)                             '...compacting all remaining objects
            ObjectList(j)=""
            INC i
        END IF
    NEXT j

' NB: Do this AFTER the ObjectList array has been tidied (above)!
    IF objType>=NON_VISIBLE THEN ReDrawNonVisibles

' Fix the select combobox
    SelectionCombo.Clear                                            ' Clear selection combo
    FOR i=0 TO MAX_OBJECTS                                          ' Rebuild selection combo
        tstr=ObjectList(i)
        IF tstr="" THEN EXIT FOR
        tstr=FIELD$(tstr,"~",1)
        SelectionCombo.AddItems FIELD$(tstr,"|",5)+" ["+FIELD$(tstr,"|",2)+"]"
    NEXT i
    ObjectIndex=i                                                   ' Update this

    hWnd=Forms(CurrentForm).Handle                                  ' Set focus to the underlying form
    FOR i=0 TO ObjectIndex-1
        tstr=FIELD$(ObjectList(i),"~",1)
        IF hWnd=VAL(FIELD$(tstr,"|",3)) THEN
            CurrentObject=i                                         ' The NEW current object
            UpdatePropGrid(CurrentObject)
            EXIT FOR
        END IF
    NEXT i
END SUB

SUB MouseResizeObject(hWnd)
DEFINT OldX, OldY, DeltaX, DeltaY
DIM ObjRect AS QRECT

    OldX=screen.MouseX: OldY=screen.MouseY
    GetWindowRect(hWnd,ObjRect)
    DO
        DeltaX=OldX-screen.MouseX: DeltaY=OldY-screen.MouseY
        IF DeltaX<>0 OR DeltaY<>0 THEN
            SELECT CASE ActiveCaret
                CASE 1: ObjRect.Left=ObjRect.Left-DeltaX: ObjRect.Top=ObjRect.Top-DeltaY
                CASE 2: ObjRect.Left=ObjRect.Left-DeltaX
                CASE 3: ObjRect.Left=ObjRect.Left-DeltaX: ObjRect.Bottom=ObjRect.Bottom-DeltaY          ' 1 8 7
                CASE 4: ObjRect.Bottom=ObjRect.Bottom-DeltaY                                            ' 2   6
                CASE 5: ObjRect.Right=ObjRect.Right-DeltaX: ObjRect.Bottom=ObjRect.Bottom-DeltaY        ' 3 4 5
                CASE 6: ObjRect.Right=ObjRect.Right-DeltaX
                CASE 7: ObjRect.Right=ObjRect.Right-DeltaX: ObjRect.Top=ObjRect.Top-DeltaY
                CASE 8: ObjRect.Top=ObjRect.Top-DeltaY
            END SELECT
            ShowCarets(ObjRect)
            OldX=screen.MouseX: OldY=screen.MouseY
        END IF
        DoEvents
    LOOP UNTIL ReSizing=False                                       ' Finished re-sizing
    RefreshObject(hWnd,ObjRect)
END SUB

SUB MouseMoveObject(hWnd)
DEFINT OldX, OldY, DeltaX, DeltaY
DEFINT i, saveX
DEFSTR tstr
DIM ObjRect AS QRECT

    GetWindowRect(hWnd,ObjRect)
    saveX=ObjRect.Top
    ShowCarets(ObjRect)
    OldX=screen.MouseX: OldY=screen.MouseY
    DO
        DeltaX=OldX-screen.MouseX: DeltaY=OldY-screen.MouseY
        IF DeltaX<>0 OR DeltaY<>0 THEN
            ObjRect.Left=ObjRect.Left-DeltaX
            ObjRect.Right=ObjRect.Right-DeltaX
            ObjRect.Top=ObjRect.Top-DeltaY
            ObjRect.Bottom=ObjRect.Bottom-DeltaY
            ShowCarets(ObjRect)
            OldX=screen.MouseX: OldY=screen.MouseY
        END IF
        DoEvents
    LOOP UNTIL gMouseBtns=0                                         ' Finished moving object
    ' Fudge for QSPLITTER
    i=HandleToIndex(hWnd)
    IF i>0 THEN
        tstr=ObjectList(i)
        IF FIELD$(tstr,"|",2)="QSPLITTER" THEN
            i=ObjRect.Bottom-ObjRect.Top+1               ' FIX our default splitter to top of form
            ObjRect.Top=saveX: ObjRect.Bottom=saveX+i
        END IF
    END IF
    RefreshObject(hWnd,ObjRect)
END SUB

FUNCTION WhichFormIsActive(hWnd AS long) AS integer
DEFINT i
    Result=0
    FOR i=1 TO MAX_FORMS
        IF hWnd=Forms(i).Handle THEN
            Result=i                                                ' Return index in Forms()
            EXIT FOR
        END IF
    NEXT i
END FUNCTION

FUNCTION IsObjectNonVis(ObjIndex AS integer) AS integer
DEFSTR tstr=FIELD$(ObjectList(ObjIndex),"~",1)

    Result=False
    IF VAL(FIELD$(tstr,"|",1))>=NON_VISIBLE THEN Result=True
END FUNCTION

FUNCTION IsObjectAForm(ObjIndex AS integer) AS integer
DEFSTR tstr=FIELD$(ObjectList(ObjIndex),"~",1)

    Result=False
    IF VAL(FIELD$(tstr,"|",1))=0 THEN Result=True
END FUNCTION

' Right mouse click on a non-selected object
SUB ForceSelection(hWnd AS long)
DEFINT i
DIM ObjRect AS QRECT

    FOR i=1 TO 8
        Carets(i).Color=&hbf                                        ' Dk Red highlight - alert user to change
    NEXT i
    GetWindowRect(hWnd,ObjRect)
    ShowCarets(ObjRect)
    Sleep.MS 150                                                    ' Delay for user to see the change
    FOR i=1 TO 8
        Carets(i).Color=&h3f7f00                                    ' Back to original color
    NEXT i
END SUB

FUNCTION IsObjectMovable(ObjIndex AS integer) AS integer
DEFINT id
DEFSTR tstr=FIELD$(ObjectList(ObjIndex),"~",1)

    Result=True
    id=VAL(FIELD$(tstr,"|",1))
    IF (id=0) OR (id>=NON_VISIBLE) THEN Result=False
END FUNCTION

SUB ResetComponentTab
    FieldEdit.Visible=False: FieldBox.Visible=False                 ' Ensure these not visible
    ComponentTab.TabIndex=0: ComponentTabChange
END SUB

SUB ScanMouse
DEFLNG hWnd
DEFINT ObjIndex

    scan.Enabled=False
    IF ReSizing THEN                                                ' Global flag set by carets()
        stat.Panel(3).Caption="L"
        stat.Panel(0).Caption="Resizing "+FIELD$(ObjectList(CurrentObject),"|",5)
        hWnd=VAL(FIELD$(ObjectList(CurrentObject),"|",3))
        MouseResizeObject(hWnd)                                     ' Do the resizing
        UpdateStatus
    ELSEIF gMouseBtns=LMOUSE_BTN THEN                               ' Place or Select an object?
        stat.Panel(3).Caption="L"
        hWnd=WindowFromPoint(Screen.MouseX, Screen.MouseY)
        ObjIndex=HandleToIndex(hWnd)
        IF (ObjIndex >= 0) AND (ObjIndex <= MAX_OBJECTS) THEN       ' Design object or form?
            BringWindowToTop(Forms(CurrentForm).Handle)             ' In case (partially) hidden
            ResetComponentTab
            IF NewObjectID>0 THEN                                   ' Adding a new object?
                AddNewObject
            ELSE
                IF ObjIndex<>CurrentObject THEN
                    CurrentObject=ObjIndex                          ' Set current object selected
                    UpdatePropGrid(ObjIndex)                        ' Fill in the property string grid
                END IF
                IF IsObjectMovable(CurrentObject) THEN
                    MouseMoveObject(hWnd)                           ' Not forms or non-visible objects
                ELSE
                    HideCarets
                END IF
            END IF
        END IF
    ELSEIF gMouseBtns=RMOUSE_BTN THEN
        stat.Panel(3).Caption="R"
        ResetComponentTab
        hWnd=WindowFromPoint(Screen.MouseX, Screen.MouseY)
        ObjIndex=HandleToIndex(hWnd)
        IF (ObjIndex >= 0) AND (ObjIndex <= MAX_OBJECTS) THEN
            IF ObjIndex<>CurrentObject THEN
                CurrentObject=ObjIndex                              ' Set current object selected
                UpdatePropGrid(ObjIndex)                            ' Fill in the property string grid
                IF IsObjectMovable(CurrentObject) THEN
                    ForceSelection(hWnd)                            ' Change current object
                END IF
            END IF
            HideCarets                                              ' If visible - hide in any case
            IF IsObjectAForm(ObjIndex) THEN
                FormPopUp(Screen.MouseX, Screen.MouseY)
            ELSEIF IsObjectNonVis(ObjIndex) THEN
                NonVisPopup(Screen.MouseX, Screen.MouseY)
            ELSE
                ObjPopUp(Screen.MouseX, Screen.MouseY)
            END IF
        END IF
    ELSEIF gMouseBtns=0 THEN
        stat.Panel(3).Caption=""
    END IF
    DoEvents
    scan.Enabled=True
END SUB

'-------------------- Carets used while editing -------------------------------------
SUB CaretMseDown(MouseButton,X,Y,Shift,Sender)
    ActiveCaret=Sender.Tag: ReSizing=True
END SUB

SUB CaretMseUp(MouseButton,X,Y,Shift,Sender)
    ActiveCaret=0: ReSizing=False
END SUB

SUB ShowCarets(tgtRect)
DIM tPoint AS PointAPI
DEFINT i, Left, Top, Width, Height
DEFINT midX, midY

    tPoint.x=tgtRect.Left: tPoint.y=tgtRect.Top
    ScreenToClient(Forms(CurrentForm).Handle, tPoint)
    Left=tPoint.x: Top=tPoint.y
    Width=tgtRect.Right-tgtRect.Left
    Height=tgtRect.Bottom-tgtRect.Top
    midX=Width\2-3: midY=Height\2-3

    Carets(1).Top=Top-6: Carets(1).Left=Left-6: Carets(1).Visible=True
    Carets(2).Top=Top+midY: Carets(2).Left=Left-6: Carets(2).Visible=True             ' Carets 6x6 pixels
    Carets(3).Top=Top+Height: Carets(3).Left=Left-6: Carets(3).Visible=True           '     1   8   7
    Carets(4).Top=Top+Height: Carets(4).Left=Left+midX: Carets(4).Visible=True        '     2       6
    Carets(5).Top=Top+Height: Carets(5).Left=Left+Width: Carets(5).Visible=True       '     3   4   5
    Carets(6).Top=Top+midY: Carets(6).Left=Left+Width: Carets(6).Visible=True
    Carets(7).Top=Top-6: Carets(7).Left=Left+Width: Carets(7).Visible=True
    Carets(8).Top=Top-6: Carets(8).Left=Left+midX: Carets(8).Visible=True
    CaretsTimer.Enabled=True                                        ' Ensure this is running
END SUB

SUB HideCarets
DEFINT i

    CaretsTimer.Enabled=False
    FOR i=1 TO 8
        Carets(i).Visible=False
    NEXT i
    ActiveCaret=0
END SUB

SUB CaretsOnTop
DEFINT i

    FOR i=1 TO 8
        SetWindowPos(Carets(i).Handle,0,0,0,0,0,FLAGS)
    NEXT i
END SUB

'--------------------- General subs/fns ------------------------------------------
'---------------------------------------------------------------------------------
' Height of Active TitleBar - (Form.Height-Form.ClientHeight-2*Border-1)
SUB GetThemeOffset()
DEFINT h1,h2,w1,w2,TwoBrdr

    h1=Forms(1).Height: w1=Forms(1).Width
    h2=Forms(1).ClientHeight: w2=Forms(1).ClientWidth
    TwoBrdr=w1-w2
    ThemeOffsetY = (h1-h2-TwoBrdr-1) - 18                           ' Windows Classic TitleBar Height=18
END SUB

SUB Initialise
DEFSTR tstr
DEFINT i, wStyle
DEFINT curs(1 TO 8)={-8, -9, -6, -7, -8, -9, -6, -7}                ' Cursors for caret movement

    SetWindowLong(MainForm.Handle, -8, 0)                           ' MainForm minimize correctly
    SetWindowLong(Application.Handle, -8, MainForm.Handle)

    ComponentRef.ExtractRes(Resource(0))                            ' Load all component text data
    ComponentRef.Position=0                                         ' Initialise component reference source
    tstr=UCASE$(ComponentRef.ReadLine)                              ' Read top line (QFORM...)
    IF INSTR(tstr,"RAPIDQ")>0 THEN
        MOD_ComponentForm.Caption="RapidQ Component Support"
    ELSE
        MOD_ComponentForm.Caption="FreeBASIC Component Support"
    END IF

    HelpRef.Position=0                                              ' Initialise the help reference stream
    HelpRef.ExtractRes(Resource(1))                                 ' Load the help text data

    LoadSingleButtons                                               ' Set the individual buttons
    LoadToolBarHints                                                ' Set the toolbar hints
    NonVisBMP.Width=27                                              ' Holds selected non-visible bmp
    NonVisBMP.Height=27
    DesignTab.TabIndex=0
    DesignTabChange

    FOR i=1 TO MAX_FORMS
        wStyle=GetWindowLong(Forms(i).Handle, GWL_STYLE)
        SetWindowLong(Forms(i).Handle, GWL_STYLE, (wStyle OR WS_CHILD))
        SetParent(Forms(i).Handle, FormsPanel.Handle)
        Forms(i).Left=0
        Forms(i).Top=0
        Forms(i).Caption="Form"+STR$(i)
        Forms(i).Visible=False
        Forms(i).AutoScroll=False                                   ' NOT default - but problems if changed
        Forms(i).IcoHandle=BlankIco
        Forms(i).OnPaint=PaintFormGrid
        Forms(i).OnResize=ReSizeForm
        Forms(i).OnMouseMove=FormsMouseMove
        Forms(i).OnClose=NoClose
        Forms(i).PopUpMenu=designMenu
    NEXT i
    Forms(1).Caption="Main Form"
    GetThemeOffset()                                                ' X offset from Windows Classic theme

    ColorPick.Style=cdFullOpen                                      ' Initialise the color chooser window
    ColorPick.Left=ComponentPan.Left-ColorPick.Width
    ColorPick.Top=(Screen.Height-ColorPick.Height)\2

    FwdDeclares=" ": FormsList=" ": KnownsList=" "                  ' Initialise the global string data
    MainMenuList=" ": PopMenuList=" ": UnknownList=" "

    FrmIndex=1: ObjectIndex=0: ComponentCount=0

    IF FindHelpFile THEN mnuHelp.Enabled=True

    FOR i=1 TO 8
        Carets(i).Tag=i
        Carets(i).Height=6: Carets(i).Width=6
        Carets(i).Cursor=curs(i): Carets(i).Color=&h3f7f00
        Carets(i).Visible=False
        Carets(i).OnMouseDown=CaretMseDown
        Carets(i).OnMouseUp=CaretMseUp
    NEXT i
    ActiveCaret=0                                                   ' Used for re-positioning/re-scaling
    CaretsTimer.Interval=250
    CaretsTimer.Enabled=False
    CaretsTimer.OnTimer=CaretsOnTop                                 ' Keeps carets visible when needed

    Scan.Interval=50
    Scan.Enabled=False
    Scan.OnTimer=ScanMouse                                          ' Main loop in design form

    HookTheMouse()                                                  ' In RapidFRM.inc

    IF CommandCount=0 THEN                                          ' New project?
        ProjectMode=STAND_ALONE
        edPsave.Enabled=False                                       ' Fix the popup menu
        MainForm.Caption=MainForm.Caption+"    [New project]"
        AddNewForm                                                  ' Add an initial form
        INC ComponentCount
    ELSE                                                            ' Editing an existing project?
        ProjectFile=LTRIM$(RTRIM$(COMMAND$(1)))
        IF UCASE$(ProjectFile)="CLIPBOARD" THEN
            MainForm.Caption=MainForm.Caption+"    [Input clipboard]"
            ProjectMode=INPUT_CLIPBRD                               ' Read the clipboard for input
        ELSE
            MainForm.Caption=MainForm.Caption+"    [Input "+ProjectFile+"]"
            ProjectMode=INPUT_FILE                                  ' Read a file for input
        END IF
        edCopy.Enabled=False                                        '|
        partCopy.Enabled=False                                      '| Fix the popup menu
        tplSave.Enabled=True                                        '|
        edSave.Enabled=False                                        '|
        LoadProject                                                 ' Load and parse the project code
' NB NEED to wait until AFTER MainForm is showing B4 we can start DISPLAYING components.
        DelayTimer.Interval=200                                     ' Set a delay of 200 mSecs
        DelayTimer.Enabled=True
        DelayTimer.OnTimer=ShowProject                              ' Allow time for initial screen paint
    END IF
    MOD_ComponentForm.Show
END SUB

FUNCTION GetScreenHeight() AS integer
DEFINT rct(0 TO 3)

    SystemParametersInfo(48, 0, VARPTR(rct(0)), 0)                  ' SPI_GETWORKAREA=48
    Result=rct(3)-rct(0)                                            ' Screen work area
END FUNCTION

' "X" of MainForm
SUB MainClose(Action)
    IF ObjectIndex>ComponentCount THEN                              ' Changes?
        Action=False
        IF MessageBox("QUIT without saving changes?", _
                "Project updated...",&h124) = mrNo THEN EXIT SUB    ' Keep going
    END IF
    UnhookWindowsHookEx(hHook)                                      ' Remove our windows mouse hook
    Action=True                                                     ' Ok we are finished!
END SUB

' Disable "X" in user generated form(s)
SUB NoClose(Action)
    Action=False
END SUB

FUNCTION FindHelpFile() AS integer
DEFSTR tstr, curPath

    Result=False
    curPath=Application.Path
    tstr=LEFT$(curPath,RINSTR(curPath,"\"))+"help\rapidq.chm"       ' FreeQ help folder?
    IF FileExists(tstr) THEN
        RQHelpFile=tstr
        Result=True
        EXIT FUNCTION
    END IF
    tstr=curPath+"\rapidq.chm"                                      ' Current folder?
    IF FileExists(tstr) THEN
        RQHelpFile=tstr
        Result=True
        EXIT FUNCTION
    END IF
    tstr=curPath+"\Help\rapidq.chm"                                  ' Mine c:\RapidQ\Help
    IF FileExists(tstr) THEN
        RQHelpFile=tstr
        Result=True
        EXIT FUNCTION
    END IF
END FUNCTION

SUB HelpOnObject
DEFSTR tstr, helpStr
DEFLNG PID

' From the doco:
'   ms-its:Helpfile.chm::/Topic.htm[>Window name]
'   mk:@MSITStore:Helpfile.chm::/Topic.htm[>Window name]
'
    tstr=ObjectList(CurrentObject)
    tstr=FIELD$(tstr,"|",2)+".html"                                 ' Object page in the help file
'    helpStr="HH.exe mk:@MSITStore:"+RQHelpFile+"::/"+tstr
    helpStr="HH.exe ms-its:"+RQHelpFile+"::/"+tstr
    PID=SHELL(helpStr)
    WaitOnHelp(PID,-1)
END SUB

SUB AboutClick
DEFSTR tstr

    tstr="RapidFRM - Form Designer/Editor for RapidQ & FreeBasic"+CRLF
    tstr=tstr+"Copyright (c) 2009-2013 Don Homans"+CRLF
    tstr=tstr+"Contact: d_homans@yahoo.com.au"+CRLF+CRLF
    tstr=tstr+"Thanks to:"+CRLF
    tstr=tstr+"        William Yu"+CRLF
    tstr=tstr+"        RapidQ contributions from Jordi Ramos & JohnK"+CRLF
    tstr=tstr+"        FreeBasic guidance from the FreeBasic community"+CRLF+CRLF
    tstr=tstr+"You may freely copy/modify the source code of this program"+CRLF
    tstr=tstr+"for your own purposes, provided that you give me and this"+CRLF
    tstr=tstr+"original program reasonable recognition in your production."+CRLF+CRLF
    tstr=tstr+"This program is FREE.  Use at your own risk!"
    MessageBox(tstr,"RapidFRM "+VER,&h40)
END SUB

' General help using the single butn at top of screen
SUB HelpBtnClick
    DesignTab.Tag=0
    DesignTab.TabIndex=4
    DesignTabChange
END SUB

SUB FillHelp(idx AS integer)
DEFINT i
DEFSTR tstr, token

    HelpText.Clear                                                  ' Clear help screen
    token="#"+STR$(idx)                                             ' Which section of help
    HelpRef.Position=0
    FOR i=1 TO HelpRef.LineCount                                    ' Position within the mem stream
        tstr=HelpRef.ReadLine
        IF FIELD$(tstr," ",1)=token THEN EXIT FOR
    NEXT i
    tstr=tstr-token
    HelpText.AddStrings tstr
    INC i
    FOR i=i TO HelpRef.LineCount                                    ' Fill the edit box
        tstr=HelpRef.ReadLine
        IF LEFT$(tstr,1)="#" THEN EXIT FOR                          ' Next section .. so exit
        HelpText.AddStrings tstr
    NEXT i
END SUB

SUB NonVisPopup(X,Y)
    mnuAlign.Enabled=False
    mnuCopy.Enabled=False
    mnuPaste.Enabled=False
    mnuDelete.Enabled=True
    designMenu.Popup(X,Y)
END SUB

' Right click on an object in Forms()
SUB ObjPopUp(X, Y)
    mnuAlign.Enabled=True
    mnuCopy.Enabled=True
    IF designMenu.Tag>0 THEN                                        ' Something copied?
        mnuPaste.Enabled=True
        MouseSaveX=X: MouseSaveY=Y
    ELSE
        mnuPaste.Enabled=False
        MouseSaveX=0: MouseSaveY=0
    END IF
    mnuDelete.Enabled=True
    designMenu.Popup(X,Y)
END SUB

' Right click on Forms() itself
SUB FormPopUp(X, Y)
    mnuAlign.Enabled=False
    mnuCopy.Enabled=False
    IF designMenu.Tag>0 THEN                                        ' Something copied?
        mnuPaste.Enabled=True
        MouseSaveX=X: MouseSaveY=Y
    ELSE
        mnuPaste.Enabled=False
        MouseSaveX=0: MouseSaveY=0
    END IF
    mnuDelete.Enabled=False
    designMenu.Popup(X,Y)
END SUB

' Recursively tunnel down to find which FORM this (objIndx) object belongs to
FUNCTION GetParentForm(objIndx AS integer) AS integer
DEFINT i, max=objIndx
DEFSTR tstr, parntStr, objStr=""

    WHILE objStr<>"0"
        tstr=FIELD$(ObjectList(max),"~",1)
        parntStr=FIELD$(tstr,"|",6)                                 ' Parent handle string
        FOR i=0 TO max-1
            tstr=FIELD$(ObjectList(i),"|",3)                        ' Find the index of the parent
            IF tstr=parntStr THEN
                objStr=FIELD$(ObjectList(i),"|",1)                  ' Parent object type, looking for a form
                max=i                                               ' reset max
                EXIT FOR
            END IF
        NEXT i
    WEND
    Result=max
END FUNCTION

' Select a component from combo dropdown list
SUB SelectComboChange
DEFINT i, frmIdx
DEFLNG hWnd
DEFSTR tstr
DIM rect AS QRECT

    IF SelectionCombo.ItemIndex=CurrentObject THEN EXIT SUB
    HideCarets
    CurrentObject=SelectionCombo.ItemIndex                          ' Set the current object

    tstr=FIELD$(ObjectList(CurrentObject),"~",1)
    IF VAL(FIELD$(tstr,"|",1))<NON_VISIBLE THEN
        IF FIELD$(tstr,"|",1)<>"0" THEN                             ' Not a form?
            frmIdx=GetParentForm(CurrentObject)                     ' Drill down - find the associated form
            tstr=FIELD$(ObjectList(frmIdx),"~",1)
        END IF
        hWnd=VAL(FIELD$(tstr,"|",3))                                ' Handle of current form
        FOR i=1 TO FrmIndex-1
            IF hWnd=Forms(i).Handle THEN
                IF i<>CurrentForm THEN
                    CurrentForm=i                                   ' Set current form
                    FOR i=1 TO 8
                        Carets(i).Parent=Forms(CurrentForm)         ' Set carets to current form
                    NEXT i
                END IF
                EXIT FOR
            END IF
        NEXT i
        tstr=FIELD$(ObjectList(CurrentObject),"~",1)
        IF FIELD$(tstr,"|",1)<>"0" THEN
            hWnd=VAL(FIELD$(tstr,"|",3))                            ' Handle of current object
            GetWindowRect(hWnd,rect)
            ShowCarets(rect)                                        ' Show carets
        END IF
    END IF
    UpdatePropGrid(CurrentObject)                                   ' Update String grid data
END SUB

' Change between component properties and events
SUB ComponentTabChange
    FieldEdit.Visible=False                                         ' Ensure hidden
    FieldBox.Visible=False                                          ' ditto
    SELECT CASE ComponentTab.TabIndex
        CASE 0
            PropertyGrid.Visible=True
            MethodGrid.Visible=False
            EventGrid.Visible=False
        CASE 1
            PropertyGrid.Visible=False
            MethodGrid.Visible=True
            EventGrid.Visible=False
        CASE 2
            PropertyGrid.Visible=False
            MethodGrid.Visible=False
            EventGrid.Visible=True
    END SELECT
END SUB

SUB DesignTabChange
DEFINT i

    ComponentTab.TabIndex=0
    ComponentTabChange
    FOR i=1 TO FrmIndex-1
        Forms(i).Visible=False
    NEXT i
    FormsPanel.Visible=False
    CodeText.Visible=False
    MenuPanel.Visible=False
    CodeText2.Visible=False
    HelpText.Visible=False
    SELECT CASE DesignTab.TabIndex
        CASE 0                                                      ' Form designer
            DesignTab.Tag=1                                         ' Preset tag for relevant help info
            FormsPanel.Visible=True
            FOR i=1 TO FrmIndex-1
                Forms(i).Visible=True
            NEXT i
        CASE 1                                                      ' RapidQ coder
            DesignTab.Tag=2                                         ' Preset tag for help info
            CodeText.Visible=True
            tplSave.Enabled=False
            BuildFormCode
        CASE 2                                                      ' Menu code copy/paste
            DesignTab.Tag=3                                         ' Preset tag for help info
            MenuEditor
        CASE 3                                                      ' FreeBasic coder
            DesignTab.Tag=4                                         ' Preset tag for help info
            CodeText2.Visible=True
            tplSave.Enabled=True
            BuildFBCode                                             ' FreeBasic coder
        CASE 4
            HelpText.Visible=True
            FillHelp(DesignTab.Tag)
    END SELECT
END SUB

'------------------------ Buttons & Toolbars ------------------------------------------------
SUB LoadSingleButtons
DEFINT i

    ButtonImages.ExtractRes(8,4,clNone)                             ' Resource(8)
    FOR i=0 TO 3
        xtraBtns(i).Parent=ToolBox
        xtraBtns(i).Left=480+(i*30)                                 ' Button width + spacing
        xtraBtns(i).Top=20
        xtraBtns(i).Width=28
        xtraBtns(i).Height=28
        xtraBtns(i).Cursor=crHandPoint
        xtraBtns(i).Spacing=0
        xtraBtns(i).Flat=True
        xtraBtns(i).ShowHint=True
        xtraBtns(i).BMP=ButtonImages.GetBMP(i)                      ' Assign the bmp
        SELECT CASE i
            CASE 0: xtraBtns(i).Hint="ADD A FORM": xtraBtns(i).OnClick=AddNewForm
            CASE 1: xtraBtns(i).Hint="ABOUT": xtraBtns(i).OnClick=AboutClick
            CASE 2: xtraBtns(i).Hint="GENERAL HELP": xtraBtns(i).OnClick=HelpBtnClick
            CASE 3: xtraBtns(i).Hint="EXIT": xtraBtns(i).OnClick=AskExit
        END SELECT
    NEXT i
END SUB

SUB LoadToolbarHints
DEFINT i, lines, cnt=0
DEFSTR tstr

    ToolbarHint=""                                                  ' A global used in several places
    ComponentRef.Position=0
    lines=ComponentRef.LineCount
    tstr=ComponentRef.ReadLine                                      ' Junk <# 0 QFORM>
    FOR i=2 TO lines
        tstr=ComponentRef.ReadLine
        IF LEFT$(tstr,1)="#" THEN
            ToolbarHint=ToolbarHint+FIELD$(tstr," ",3)+"~"
            INC cnt
            IF cnt>MAX_TOOLBTNS THEN EXIT FOR
        END IF
    NEXT i
    tstr=""
    FOR i=1 TO cnt
        tstr=tstr+FIELD$(ToolBarHint,"~",i)+","
        SELECT CASE i
            CASE 15: ToolBar1.Hint=tstr: tstr=""                    ' Btns  1-15
            CASE 29: ToolBar2.Hint=tstr: tstr=""                    ' Btns 16-29
            CASE 40: ToolBar3.Hint=tstr: tstr=""                    ' Btns 30-40
            CASE 44: ToolBar4.Hint=tstr: tstr=""                    ' Btns 41-44
            CASE 55: ToolBar5.Hint=tstr: tstr=""                    ' Btns 45-55
        END SELECT
    NEXT i
END SUB

SUB ButnCmd(idx)                                                    ' Clicked on toolbar button
    Scan.Enabled=False
    FieldEdit.Visible=False
    FieldBox.Visible=False
    DesignTab.TabIndex=0: DesignTabChange                           ' Ensure design tab visible
    ComponentTab.TabIndex=0: ComponentTabChange
    HideCarets
    SELECT CASE ToolBarTab.TabIndex
        CASE 0: NewObjectID=ToolBar1.Tag+idx                        ' Btns  1-15
        CASE 1: NewObjectID=ToolBar2.Tag+idx                        ' Btns 16-29
        CASE 2: NewObjectID=ToolBar3.Tag+idx                        ' Btns 30-40
        CASE 3: NewObjectID=ToolBar4.Tag+idx                        ' Btns 41-44
        CASE 4: NewObjectID=ToolBar5.Tag+idx                        ' Btns 45-55
    END SELECT
    IF NewObjectID >= NON_VISIBLE THEN                              ' Add immediately to non-vis area
        SELECT CASE ToolBarTab.TabIndex
            CASE 0: NonVisBMP.BMP=ToolBar1.tbButtons(idx).BMP       '
            CASE 1: NonVisBMP.BMP=ToolBar2.tbButtons(idx).BMP       '
            CASE 2: NonVisBMP.BMP=ToolBar3.tbButtons(idx).BMP       ' Grab a copy of button image
            CASE 3: NonVisBMP.BMP=ToolBar4.tbButtons(idx).BMP       '
            CASE 4: NonVisBMP.BMP=ToolBar5.tbButtons(idx).BMP       '
        END SELECT
        AddNewObject
    END IF
    Scan.Enabled=True                                               ' Else wait for user to place on form
END SUB

SUB ToolTabChange
    Deselect
    ToolBar1.Visible=False: ToolBar2.Visible=False: ToolBar3.Visible=False
    ToolBar4.Visible=False: ToolBar5.Visible=False
    SELECT CASE ToolBarTab.TabIndex
        CASE 0: ToolBar1.Visible=True
        CASE 1: ToolBar2.Visible=True
        CASE 2: ToolBar3.Visible=True
        CASE 3: ToolBar4.Visible=True
        CASE 4: ToolBar5.Visible=True
    END SELECT
END SUB

'--------------------- Form(s) stuff -------------------------------------------------
FUNCTION UpdateDefProperties(defString AS string, X AS integer, Y AS integer, _
                        Width AS integer, Height AS integer, strText AS string) AS string
DEFINT i, cnt
DEFSTR token, resStr

    cnt=TALLY(defString,"~")
    resStr=" "
    FOR i=1 TO cnt
        token=FIELD$(defString,"~",i)
        SELECT CASE UCASE$(FIELD$(token,"|",1))
            CASE "LEFT": token=token-FIELD$(token,"|",4)+STR$(X)
            CASE "TOP": token=token-FIELD$(token,"|",4)+STR$(Y)
            CASE "WIDTH": token=token-FIELD$(token,"|",4)+STR$(Width)
            CASE "HEIGHT": token=token-FIELD$(token,"|",4)+STR$(Height)
            CASE "CAPTION": token=token-FIELD$(token,"|",4)+strText
            CASE "PARENT"
                IF INSTR(strText,"Form")=0 THEN                     ' If not a form object
                    token=token-FIELD$(token,"|",4)
                    IF CurrentForm=0 THEN
                        token=token+"MainForm"
                    ELSE
                        token=token+"Form"+STR$(CurrentForm)
                    END IF
                END IF
        END SELECT
        resStr=resStr+token+"~"
    NEXT i
    Result=LTRIM$(resStr)
END FUNCTION

' Currently only a visual fudge for QSplitter AND only vertically (default)!!
SUB UpdateChildObjects(Sender AS QFORM)
DEFINT i, j, splitLeft, splitWidth
DEFLNG hWnd
DEFSTR objStr, tstr, tok

    FOR i=CurrentObject+1 TO ObjectIndex
        objStr=ObjectList(i)
        IF objStr="" THEN EXIT FOR
        tstr=FIELD$(objStr,"~",1)
        tok=FIELD$(tstr,"|",6)                                      ' Parent handle
        IF (VAL(tok)=Sender.Handle) AND (FIELD$(tstr,"|",2)="QSPLITTER") THEN
            hWnd=VAL(FIELD$(tstr,"|",3))                            ' Object handle
            FOR j=2 TO TALLY(objStr,"~")
                tstr=UCASE$(FIELD$(objStr,"~",j))
                tok=TRIM$(FIELD$(tstr,"|",1))
                SELECT CASE tok
                    CASE "LEFT": tok=TRIM$(FIELD$(tstr,"|",4)): splitLeft=VAL(tok)
                    CASE "WIDTH": tok=TRIM$(FIELD$(tstr,"|",4)): splitWidth=VAL(tok)
                END SELECT
            NEXT j
            SetWindowPos(hWnd, 0, splitLeft, 0, splitWidth, Sender.ClientHeight, SW_FLAGS)
            EXIT FOR
        END IF
    NEXT i
END SUB

SUB ReSizeForm(Sender)
DEFSTR tstr, saveStr
DEFINT idx

    HideCarets
    CurrentForm=WhichFormIsActive(Sender.Handle)                    ' Index in Forms()
    idx=HandleToIndex(Sender.Handle)                          ' Get index in ObjectList()
    IF (idx >= 0) AND (idx <= MAX_OBJECTS) THEN
        CurrentObject=idx
        tstr=ObjectList(idx)
        saveStr=FIELD$(tstr,"~",1)+"~"
        tstr=tstr-saveStr
        tstr=UpdateDefProperties(tstr, Sender.Left, Sender.Top, _
                                        Sender.Width, Sender.Height, Sender.Caption)
        ObjectList(idx)=saveStr+tstr                                ' Update global string
        UpdatePropGrid(idx)                                         ' Update String grid
        UpdateChildObjects(Sender)
    END IF
END SUB

SUB FormsMouseMove(X, Y, Shift, Sender)
DEFINT i

    i=WhichFormIsActive(Sender.Handle)
    IF i<>CurrentForm THEN
        CurrentForm=i
        HideCarets                                                  ' If visible
        FOR i=1 TO 8
            Carets(i).Parent=Sender                                 ' Set carets to this form
        NEXT i
    END IF
    stat.Panel(1).Caption="  Active Form = "+STR$(CurrentForm)
    stat.Panel(2).Caption="  X="+STR$(X)+": Y="+STR$(Y)
END SUB

SUB PaintFormGrid(Sender)
DEFINT px, py

    FOR py=0 TO Sender.Height STEP 10
        FOR px=0 TO Sender.Width STEP 10
            Sender.Pset(px, py, &h7f7f7f)
        NEXT px
    NEXT py
END SUB

'----------------------- Updating component fields -------------------------------------
'
SUB MethodUpdate(newStrings AS string)
DEFINT i, cnt
DEFLNG hWnd
DEFSTR tstr, token

    IF LEN(newStrings)=0 THEN newStrings=" "                        ' Don't allow to delete ALL tabs!
    tstr=ObjectList(CurrentObject)
    tstr=FIELD$(tstr,"~",1)
    hWnd=VAL(FIELD$(tstr,"|",7))
    SELECT CASE FIELD$(tstr,"|",2)
        CASE "QTABCONTROL"
            DelAllTabs(hWnd)
            i=0: cnt=TALLY(newStrings,",")+1
            DO
                token=FIELD$(newStrings,",",i+1)
                IF token<> "" THEN AddNewTab(hWnd,i,token)
                INC i
            LOOP UNTIL i=cnt
    END SELECT
END SUB

SUB ReDrawComponent
DEFSTR propString, token, tstr, cap
DEFINT i, X, Y, Height, Width, objType, reposn
DEFLNG hWnd
DIM rect AS QRECT

    propString=ObjectList(CurrentObject)
    tstr=FIELD$(propString,"~",1)
    objType=VAL(FIELD$(tstr,"|",1))
    IF objType>=NON_VISIBLE THEN EXIT SUB

    reposn=False
    hWnd=VAL(FIELD$(tstr,"|",3))
    FOR i=2 TO TALLY(propString,"~")
        tstr=FIELD$(propString,"~",i)
        token=UCASE$(FIELD$(tstr,"|",1))
        SELECT CASE token
            CASE "LEFT": X=VAL(FIELD$(tstr,"|",4)): reposn=True
            CASE "TOP": Y=VAL(FIELD$(tstr,"|",4)): reposn=True
            CASE "WIDTH": Width=VAL(FIELD$(tstr,"|",4)): reposn=True
            CASE "HEIGHT": Height=VAL(FIELD$(tstr,"|",4)): reposn=True
            CASE "CAPTION", "TEXT"
                cap=FIELD$(tstr,"|",4)
                SetWindowText(hWnd,cap)                             ' Update object text
                IF objType=Q_GROUPBOX THEN
                    SetWindowPos(hWnd,0,0,0,0,0,SWP_FLAGS)          ' Fudge to fix QGroupbox only!
                END IF
        END SELECT
    NEXT i
    IF objType=Q_OVALBTN THEN                                       ' Handle QOvalBtn differently
        tstr=FIELD$(propString,"~",1)
        i=VAL(FIELD$(tstr,"|",7))                                   ' Array index of the ovalbtn
        OvalRepaint(i,X,Y,Width,Height,cap)
    ELSEIF reposn THEN
        HideCarets                                                  ' If visible
        SetWindowPos(hWnd, 0, X, Y, Width, Height, SW_FLAGS)
        IF objType>0 THEN                                           ' If NOT a form
            GetWindowRect(hWnd,rect)
            ShowCarets(rect)
        END IF
    END IF
END SUB

' FIXME:
SUB SetPropertyColor(colour AS long)
'DEFLNG hWnd, hbr
'DEFSTR tstr
'DIM rct AS QRECT
'
'    tstr=FIELD$(ObjectList(CurrentObject),"~",1)
'    hWnd=VAL(FIELD$(tstr,"|",3))
'    GetClientRect(hWnd, rct)
'    hbr=CreateSolidBrush(colour)
'    DeleteObject(SetClassLong(hWnd, -10, hbr))                 ' GCL_HBRBACKGROUND = -10
'    InvalidateRect(hWnd, rct, TRUE)
END SUB


' Edit a property field using the combobox
SUB FieldUpdateCombo(Sender)
DEFSTR tstr, tok
DEFINT i, Row, ColorChosen=False

    Row=Sender.Tag                                                  ' The saved Row we are editing
    tstr=Sender.Item(Sender.ItemIndex)                              ' The combobox selection
    IF UCASE$(tstr)="<CHOOSE>" THEN                                 ' A change!
        tok=PropertyGrid.Cell(0,Row)+" ["
        tok=tok+FIELD$(SelectionCombo.Item(SelectionCombo.ItemIndex),"[",1)+"]"
        ColorPick.Caption=tok                                       ' Prepare the caption for color chooser
        tok=PropertyGrid.Cell(1,Row)                                ' Last value for color
        IF LEFT$(tok,2)="&H" THEN                                   ' Colour has been set before?
            ColorPick.Color=VAL(CONVBASE$(tok,16,10))               ' Change "default" colour
'       ELSE
'          Uses the LAST set "default" colour!!!
        END IF
        IF ColorPick.Execute THEN
            tstr="&H"+RIGHT$(HEX$(ColorPick.Color),6)               ' Updated value
            ColorChosen=True
        ELSE
            tstr=PropertyGrid.Cell(1,Row)                           ' NO change!
        END IF
    END IF

    IF tstr<>PropertyGrid.Cell(1,Row) THEN                          ' Changed?
        PropertyGrid.Cell(1,Row)=tstr
        PropertyFieldUpdate(PropertyGrid.Cell(0,Row),tstr,4)        ' New current value
        IF ColorChosen THEN
            PropertyFieldUpdate(PropertyGrid.Cell(0,Row),tstr,7)    ' New color choice
            SetPropertyColor(ColorPick.Color)
        END IF
    END IF
    Sender.Tag=-1                                                   ' Clear saved data
    Sender.Visible=False
END SUB

' Return the parameters of the event sub
FUNCTION GetSubParms(pField AS string) AS string
DEFSTR tstr, token
DEFINT i

    Result=""
    tstr=ObjectList(CurrentObject)
    FOR i=1 TO TALLY(tstr,"~")
        token=FIELD$(tstr,"~",i)
        IF FIELD$(token,"|",1)=pField THEN
            Result=FIELD$(token,"|",3)
            EXIT FOR
        END IF
    NEXT i
END FUNCTION

' Edit a property/method/event field using the edit box
SUB FieldUpdateEdit(Key,Shift,Sender)
DEFSTR oldStr, newStr, pField, token
DEFINT Row

    IF Key=13 THEN                                                  ' Finished editing this field?
        Row=Sender.Tag                                              ' The saved Row we are editing
        Sender.Tag=-1
        newStr=LTRIM$(RTRIM$(Sender.Text))
        SELECT CASE ComponentTab.TabIndex
            CASE 0: oldStr=PropertyGrid.Cell(1,Row)
            CASE 1: oldStr=MethodGrid.Cell(1,Row)
            CASE 2: oldStr=EventGrid.Cell(1,Row)
        END SELECT
        IF oldStr=newStr THEN                                       ' No change?
            Sender.Visible=False
            EXIT SUB
        END IF
        SELECT CASE ComponentTab.TabIndex
            CASE 0                                                  ' Component PROPERTY update
                PropertyGrid.Cell(1,Row)=newStr                     ' Update the property grid
                pField=PropertyGrid.Cell(0,Row)
                PropertyFieldUpdate(pField,newStr,4)                ' Update the global object string
                SELECT CASE UCASE$(pField)
                    CASE "LEFT","TOP","WIDTH","HEIGHT","CAPTION","TEXT"
                        ReDrawComponent
                END SELECT
            CASE 1                                                  ' Component METHOD update
                MethodGrid.Cell(1,Row)=newStr                       ' Update the property grid
                pField=MethodGrid.Cell(0,Row)
                PropertyFieldUpdate(pField,newStr,4)                ' Update the global object string
                MethodUpdate(newStr)
            CASE 2                                                  ' Component EVENT update
                EventGrid.Cell(1,Row)=newStr                        ' Update the event grid
                pField=EventGrid.Cell(0,Row)
                PropertyFieldUpdate(pField,newStr,2)                ' Update the global object string
                IF newStr="" THEN                                   ' Cleared an event?
                    oldStr="SUB "+oldStr
                    IF INSTR(FwdDeclares,oldStr)<>0 THEN            ' Old SUB exists in our CURRENT declares?
                        oldStr=oldStr+GetSubParms(pField)+"~"       ' Get parameters of this sub
                        FwdDeclares=FwdDeclares-oldStr              ' Remove the forward declaration...
                    END IF                                          ' ..NB NOT from the entire users code!
                ELSE
                    newStr="SUB "+newStr
                    IF INSTR(FwdDeclares,newStr)=0 THEN             ' New SUB doesn't exist?
                        newStr=newStr+GetSubParms(pField)+"~"       ' Get parameters of this sub
                        FwdDeclares=FwdDeclares+newStr              ' Add the forward declaration
                    END IF
                END IF
        END SELECT
        Sender.Visible=False
    END IF
END SUB

' User selected a field in the property string grid
SUB EditPropertyField(Col, Row, CanSelect, Sender)
DEFINT i, top, cnt, Sel
DEFSTR tstr, tok

    HideCarets                                                      ' Ensure hidden
    FieldEdit.Visible=False                                         ' Ensure the field QEDIT hidden
    FieldBox.Visible=False                                          ' Ensure the field QCOMBOBOX hidden

    tstr=ObjectList(CurrentObject)
    tstr=FIELD$(tstr,"~",Row+1)                                     ' Get the property string we are editing
    IF FIELD$(tstr,"|",3)="R" THEN EXIT SUB                         ' If property is read only then Exit

    i=Sender.DefaultRowHeight+1                                     ' Height + gridline
    top=i*(Row-Sender.TopRow+2)+(i\2)                               ' Adjusted for scroll & TABs height (1.5 * i)

    cnt=TALLY(tstr,"|")+1                                           ' Number items this property
    IF cnt>4 THEN                                                   ' Multi-choice? Use the field COMBOBOX
        FieldBox.Top=top                                            ' Position over correct cell
        FieldBox.Clear: Sel=0
        FOR i=5 TO cnt
            tok=FIELD$(tstr,"|",i)                                  ' Add each possible value to the list
            FieldBox.AddItems tok
            IF tok=Sender.Cell(1,Row) THEN Sel=i-5                  ' CURRENT value? Save its index
        NEXT i
        tstr=Sender.Cell(1,Row)
        IF (LEFT$(tstr,1) <> "<") AND (LEFT$(FieldBox.Item(0),1)="<") THEN      ' Color?
            FieldBox.AddItems tstr                                  ' Previous color selected? Add it to list
            FieldBox.ItemIndex=i-5                                  ' Now select it!
        ELSE
            FieldBox.ItemIndex=Sel                                  ' Select CURRENT value from the list
        END IF
        FieldBox.Tag=Row                                            ' Save Row for updating correct field
        FieldBox.Visible=True
    ELSE                                                            ' Single field use the field EDIT BOX
        FieldEdit.Top=top
        FieldEdit.Text=FIELD$(tstr,"|",4)
        FieldEdit.Tag=Row                                           ' Save Row for updating correct field
        FieldEdit.Visible=True
        SetFocus(FieldEdit.Handle)
    END IF
END SUB

' User selected a field in the method string grid
SUB EditMethodField(Col, Row, CanSelect, Sender)
DEFINT i, top, cnt
DEFSTR objString, tstr, token

    HideCarets                                                      ' Ensure hidden
    FieldEdit.Visible=False                                         ' Likewise
    FieldBox.Visible=False                                          ' ditto

    token=Sender.Cell(0,Row)                                        ' The method name
    objString=ObjectList(CurrentObject)
    cnt=TALLY(objString,"~")                                        ' Number of lines this component
    FOR i=2 TO cnt
        tstr=FIELD$(objString,"~",i)                                ' Find the method fields
        IF LEFT$(tstr,1)="*" THEN EXIT FOR
    NEXT i
    FOR i=i TO cnt
        tstr=FIELD$(objString,"~",i)                                ' Find this method
        IF token=FIELD$(tstr,"|",1) THEN EXIT FOR
    NEXT i
    IF FIELD$(tstr,"|",3)<>"W" THEN EXIT SUB                        ' NOT writable then exit

    token=FIELD$(tstr,"|",4)                                        ' Get the value
    i=Sender.DefaultRowHeight+1                                     ' Height + gridline
    top=i*(Row-Sender.TopRow+2)+(i\2)                               ' Adjusted for scroll & TABs height
    FieldEdit.Top=top
    FieldEdit.Text=token
    FieldEdit.Tag=Row                                               ' Save Row for updating correct field
    FieldEdit.Visible=True
    SetFocus(FieldEdit.Handle)
END SUB

' User selected a field in the event string grid
SUB EditEventField(Col, Row, CanSelect, Sender)
DEFINT i, j, top, cnt
DEFSTR objString, tstr, token

    HideCarets                                                      ' Ensure hidden
    FieldEdit.Visible=False                                         ' Likewise
    FieldBox.Visible=False                                          ' ditto

    token=Sender.Cell(0,Row)                                        ' The event name
    objString=ObjectList(CurrentObject)
    cnt=TALLY(objString,"~")                                        ' Number of lines this component
    j=0
    FOR i=2 TO cnt
        tstr=FIELD$(objString,"~",i)                                ' Find the event fields
        IF LEFT$(tstr,1)="*" THEN INC j
        IF j=2 THEN EXIT FOR
    NEXT i
    FOR i=i TO cnt
        tstr=FIELD$(objString,"~",i)                                ' Find this event
        IF token=FIELD$(tstr,"|",1) THEN EXIT FOR
    NEXT i
    token=FIELD$(tstr,"|",2)                                        ' Get the value

    i=Sender.DefaultRowHeight+1                                     ' Height + gridline
    top=i*(Row-Sender.TopRow+2)+(i\2)                               ' Adjusted for scroll & TABs height
    FieldEdit.Top=top
    FieldEdit.Text=token
    FieldEdit.Tag=Row                                               ' Save Row for updating correct field
    FieldEdit.Visible=True
    SetFocus(FieldEdit.Handle)
END SUB

' Tag property tracks the TopRow so if user "scrolls" component data (Properties)
' then the "Field boxes" are cleared. eg QRichEdit, QTabControl & QStringGrid
SUB HideEditCtls(Col,Row,State,Rect,Sender)
    IF Sender.Tag<>Sender.TopRow THEN                               ' Have we scrolled?
        FieldEdit.Visible=False                                     ' Ensure this hidden
        FieldBox.Visible=False                                      ' ditto
        Sender.Tag=Sender.TopRow                                    ' Update copy of TopRow
    END IF
END SUB


'----------------------- Displaying code in the TextCode Screen -----------------------------------
' Entry point: BuildFormCode()
'
SUB InsertCodeHeader                                                ' STAND_ALONE mode only
DEFINT i
DEFSTR tstr

    CodeText.AddStrings "'", "' <Program Name>", "'"                ' Add a default header
'    CodeText.AddStrings "' RapidFRM "+VER,"'",""
    CodeText.AddStrings "$APPTYPE GUI", "$TYPECHECK ON",""
    CodeText.AddStrings "$INCLUDE <Rapidq2.inc>", ""
    FwdDeclares=LTRIM$(FwdDeclares)
    IF FwdDeclares<>"" THEN                                         ' Add any Forward declarations
        FOR i=1 TO TALLY(FwdDeclares,"~")
            tstr=LTRIM$(RTRIM$(FIELD$(FwdDeclares,"~",i)))
            CodeText.AddStrings "DECLARE "+tstr
        NEXT i
        CodeText.AddStrings ""
    END IF
END SUB

SUB InsertCodeFooter                                                ' STAND_ALONE mode only
DEFINT i
DEFSTR tstr

    tstr=FIELD$(ObjectList(0),"~",1)                                ' Initial form
    tstr=FIELD$(tstr,"|",4)                                         ' The form name
    CodeText.AddStrings "SetWindowLong("+tstr+".Handle, GWL_HWNDPARENT, 0)"
    CodeText.AddStrings "SetWindowLong(Application.Handle, GWL_HWNDPARENT, "+tstr+".Handle)",""
    CodeText.AddStrings tstr+".ShowModal",""
    IF FwdDeclares<>"" THEN                                         ' Add the FwdDEclare sub templates
        FOR i=1 TO TALLY(FwdDeclares,"~")
            tstr=LTRIM$(RTRIM$(FIELD$(FwdDeclares,"~",i)))
            CodeText.AddStrings tstr,"END SUB"
            CodeText.AddStrings
        NEXT i
    END IF
END SUB

SUB InsertNonVisible
DEFINT i, k, cnt, objID
DEFSTR defString, nuString, tstr, token
DEFSTR spacer=SPACE$(4)

    FOR i=1 TO MAX_OBJECTS                                          ' Loop through the ObjectList...
        nuString=ObjectList(i)                                      ' ..for non-visible objects
        IF nuString="" THEN EXIT FOR                                ' End of object list so exit
        tstr=FIELD$(nuString,"~",1)                                 ' First line of object string
        objID=VAL(FIELD$(tstr,"|",1))
        IF objID >= NON_VISIBLE THEN                                ' Interested?
            IF ProjectMode=STAND_ALONE THEN
                token=FIELD$(tstr,"|",4)                            ' Internal object name
            ELSE
                token=FIELD$(tstr,"|",5)                            ' Project object name
            END IF
            tstr="DIM "+token+" AS "+FIELD$(tstr,"|",2)
            CodeText.AddStrings tstr
            defString="Dummy align string~"                         ' Add this to align with ObjectList() string
            defString=defString+GetDefaultProperties(objID)         ' Default properties this object type
            cnt=TALLY(defString,"~")                                ' Num properties this object
            FOR k=2 TO cnt
                tstr=FIELD$(nuString,"~",k)                         ' Loop through each current prop line
                IF FIELD$(defString,"~",k)<>tstr THEN               ' Equal default property line?
                    IF UCASE$(LEFT$(tstr,2))="ON" THEN              ' An event?
                        CodeText.AddStrings spacer+token+"."+FIELD$(tstr,"|",1) _
                                                            +" = "+FIELD$(tstr,"|",2)
                    ELSE                                            ' A property!
                        SELECT CASE FIELD$(tstr,"|",1)              ' No, so include the change
                            CASE "Parent"                           ' Don't include this!
                            CASE ELSE
                                IF FIELD$(tstr,"|",2)="STRING" THEN
                                    CodeText.AddStrings spacer+token+"."+FIELD$(tstr,"|",1) _
                                                                +" = "+QU+FIELD$(tstr,"|",4)+QU
                                ELSE
                                    CodeText.AddStrings spacer+token+"."+FIELD$(tstr,"|",1)+" = "+FIELD$(tstr,"|",4)
                                END IF
                        END SELECT
                    END IF
                END IF
            NEXT k
            CodeText.AddStrings ""
        END IF
    NEXT i
END SUB

' Insert code for QMainMenus and/or QPopupMenus with formatting
SUB InsertMenu(which AS integer)
DEFINT i, j, cnt, lev
DEFSTR mnuString, tstr, token, spacer

    IF which=MAINMENU THEN mnuString=LTRIM$(MainMenuList) ELSE mnuString=LTRIM$(PopMenuList)
    cnt=TALLY(mnuString,"@")
    FOR i=1 TO cnt
        lev=0: spacer=""
        tstr=FIELD$(mnuString,"@",i)+"~"                            ' Each menu block
        FOR j=1 TO TALLY(tstr,"~")
            token=FIELD$(tstr,"~",j)                                ' Each menu line
            IF LEFT$(LTRIM$(token),1)="'" THEN
                CodeText.AddStrings token
            ELSE
                IF INSTR(token,"=")>0 THEN
                    token=RTRIM$(FIELD$(token,"=",1))+" = "+LTRIM$(FIELD$(token,"=",2))
                END IF
                IF INSTR(UCASE$(token),"CAPTION")>0 THEN
                    token=FIELD$(token," ",1)+" = "+QU+LTRIM$(FIELD$(token,"=",2))+QU
                END IF
                IF UCASE$(FIELD$(token," ",1))="END" THEN
                    spacer=LEFT$(spacer,LEN(spacer)-4)              ' Dec spacer then add line
                    CodeText.AddStrings spacer+token
                ELSEIF UCASE$(FIELD$(token," ",1))="CREATE" THEN
                    CodeText.AddStrings spacer+token                ' Add line then inc spacer
                    spacer=spacer+"    "
                ELSE
                    CodeText.AddStrings spacer+token
                END IF
            END IF
        NEXT j
        CodeText.AddStrings ""
    NEXT i
END SUB

FUNCTION ObjectHasChildren(ThisIdx AS integer) AS integer
DEFINT i
DEFLNG hWnd
DEFSTR tstr

    Result=False
    tstr=FIELD$(ObjectList(ThisIdx),"~",1)
    hWnd=VAL(FIELD$(tstr,"|",3))                                    ' Handle of (parent) object
    FOR i=ThisIdx+1 TO ObjectIndex-1                                ' Search after this object
        tstr=FIELD$(ObjectList(i),"~",1)
        IF hWnd=VAL(FIELD$(tstr,"|",6)) THEN                        ' Found an object with this parent
            Result=True
            EXIT FOR
        END IF
    NEXT i
END FUNCTION


' FIXME Component "methods" of type OTHER THAN <Array of strings>!!
SUB InsertObjectCode(ThisIdx AS integer, lev AS integer)
DEFINT i, j, cnt, aligned, addquotes, objType
DEFSTR tstr, token, defString, nuString
DEFSTR spacer

    spacer=SPACE$(lev*4)
    nuString=ObjectList(ThisIdx)                                    ' This object in the list
    IF ProjectMode=STAND_ALONE THEN
        token=FIELD$(nuString,"|",4)                                ' Internal object name
    ELSE
        token=FIELD$(nuString,"|",5)                                ' Project object name
    END IF
    CodeText.AddStrings spacer+"CREATE "+token+" AS "+FIELD$(nuString,"|",2)
    spacer=spacer+"    "
    objType=VAL(FIELD$(nuString,"|",1))
    defString="Dummy align string~"                                 ' Add this to align with ObjectList() string
    defString=defString+GetDefaultProperties(objType)               ' Default properties this object type

    aligned=False                                                   ' Default Align prop=alNone
    cnt=TALLY(defString,"~")
    FOR i=2 TO cnt                                                  ' Check for Align=alClient
        tstr=FIELD$(nuString,"~",i)
        IF FIELD$(tstr,"|",1)="Align" THEN
            IF FIELD$(tstr,"|",4)="alClient" THEN
                aligned=True                                        ' Flag it - alClient=True
            END IF
            EXIT FOR
        END IF
    NEXT i
    addquotes=False
    FOR i=2 TO cnt
        tstr=FIELD$(nuString,"~",i)
        IF FIELD$(defString,"~",i)<>tstr THEN
            token=UCASE$(FIELD$(tstr,"|",1))
            IF (LEFT$(token,2)="ON") OR (token="WNDPROC") THEN      ' An event?
                CodeText.AddStrings spacer+FIELD$(tstr,"|",1)+" = "+FIELD$(tstr,"|",2)
            ELSEIF LEFT$(token,3)="ADD" THEN                        ' Add method?
                token=FIELD$(tstr,"|",4)                            ' The values (comma sep strings)
                IF INSTR(token,QU)=0 THEN addquotes=True
                tstr=spacer+FIELD$(tstr,"|",1)+" "
                FOR j=1 TO TALLY(token,",")+1
                    IF addquotes THEN tstr=tstr+QU
                    tstr=tstr+LTRIM$(RTRIM$(FIELD$(token,",",j)))   ' Add each string
                    IF addquotes THEN tstr=tstr+QU
                    tstr=tstr+","
                NEXT j
                CodeText.AddStrings LEFT$(tstr,LEN(tstr)-1)         ' Remove trailing ","
            ELSE
                SELECT CASE FIELD$(tstr,"|",1)
                    CASE "Parent"                                   ' Don't include this!
                    CASE "Width"
                        ' Fudge for QSplitter only, but there are more!
                        IF aligned=False THEN                       ' If NOT alClient - include this
                            CodeText.AddStrings spacer+FIELD$(tstr,"|",1)+" = "+FIELD$(tstr,"|",4)
                        END IF
                    CASE "Left","Top","Height"
                        ' Fudge for QSplitter only, but there are more!
                        IF (aligned=False) AND (objType<>Q_SPLITTER) THEN       ' If NOT alClient - include these
                            CodeText.AddStrings spacer+FIELD$(tstr,"|",1)+" = "+FIELD$(tstr,"|",4)
                        END IF
                    CASE "ColWidths"                                ' QStringGrid fudge array of widths
                        token=FIELD$(tstr,"|",4)
                        FOR j=1 TO TALLY(token,",")+1
                            CodeText.AddStrings spacer+FIELD$(tstr,"|",1)+"("+STR$(j-1)+") = "+FIELD$(token,",",j)
                        NEXT j
                    CASE ELSE
                        IF FIELD$(tstr,"|",2)="STRING" THEN         ' This property type a string?
                            CodeText.AddStrings spacer+FIELD$(tstr,"|",1)+" = "+QU+FIELD$(tstr,"|",4)+QU
                        ELSE
                            CodeText.AddStrings spacer+FIELD$(tstr,"|",1)+" = "+FIELD$(tstr,"|",4)
                        END IF
                END SELECT
            END IF
        END IF
    NEXT i
END SUB

SUB InsertFormCode(idx AS integer)
DEFINT i, cnt, Maxit
DEFSTR defString, nuString, tstr, token
DEFSTR spacer=SPACE$(4)

    Maxit=False                                                     ' Default - forms are not maximised
    nuString=ObjectList(idx)                                        ' Get our current object (with changes?)
    tstr=FIELD$(nuString,"~",1)
    IF ProjectMode=STAND_ALONE THEN
        token=FIELD$(tstr,"|",4)                                    ' Internal object name
    ELSE
        token=FIELD$(tstr,"|",5)                                    ' Project object name
    END IF
    CodeText.AddStrings "CREATE "+token+" AS QFORM"
    defString="Dummy align string~"                                 ' Add this to align with ObjectList() string
    defString=defString+GetDefaultProperties(0)                     ' Default properties this object
    cnt=TALLY(defString,"~")
    FOR i=2 TO cnt                                                  ' Check for maximised form
        tstr=FIELD$(nuString,"~",i)
        IF FIELD$(tstr,"|",1)="WindowState" THEN
            IF FIELD$(tstr,"|",4)="wsMaximized" THEN Maxit=True
            EXIT FOR
        END IF
    NEXT i
    FOR i=2 TO cnt
        tstr=FIELD$(nuString,"~",i)
        IF FIELD$(defString,"~",i)<>tstr THEN
            token=UCASE$(FIELD$(tstr,"|",1))
            IF (LEFT$(token,2)="ON") OR (token="WNDPROC") THEN      ' A form event?
                CodeText.AddStrings spacer+FIELD$(tstr,"|",1)+" = "+FIELD$(tstr,"|",2)
            ELSE
                SELECT CASE FIELD$(tstr,"|",1)
                    CASE "Left","Top"                               ' Don't need these, will default to center
                    CASE "Height","Width"                           ' Don't need these if maximised
                        IF Maxit=False THEN                         ' Not maximised?
                            CodeText.AddStrings spacer+FIELD$(tstr,"|",1)+" = "+FIELD$(tstr,"|",4)
                        END IF
                    CASE ELSE
                        IF FIELD$(tstr,"|",2)="STRING" THEN
                            CodeText.AddStrings spacer+FIELD$(tstr,"|",1)+" = "+QU+FIELD$(tstr,"|",4)+QU
                        ELSE
                            CodeText.AddStrings spacer+FIELD$(tstr,"|",1)+" = "+FIELD$(tstr,"|",4)
                        END IF
                END SELECT
            END IF
        END IF
    NEXT i
    IF Maxit=False THEN
        CodeText.AddStrings spacer+"Center"
    END IF
END SUB

' Recursively called for each new "parent type" object
SUB InsertThisChain(StartIdx AS integer, lev AS integer)
DEFINT i, cnt, nxt
DEFSTR sibString, spacer

    spacer=SPACE$(lev*4)
    sibString=FillSiblingList(StartIdx)                             ' Get list of child object indices
    cnt=TALLY(sibString,"|")
    IF cnt>0 THEN
        FOR i=1 TO cnt                                              ' Loop all child objects
            nxt=VAL(FIELD$(sibString,"|",i))                        ' Get next object index
            InsertObjectCode(nxt,lev)                               ' Add this object code
            IF IsTypeParent(nxt)=True THEN                          ' Child objects?
                INC lev
                InsertThisChain(nxt,lev)                            ' Recursive call to self
                DEC lev
            END IF
            CodeText.AddStrings spacer+"END CREATE"                 ' Terminate this object code
        NEXT i
    END IF
END SUB

' Entry point to display the code for current form(s) layout
SUB BuildFormCode
DEFINT i, j, lev=1
DEFSTR tstr

    CodeText.Clear
    IF ProjectMode=STAND_ALONE THEN
        InsertCodeHeader                                            ' New project code header
    END IF
    IF LTRIM$(PopMenuList)<>"" THEN                                 ' QPOPUPMENUs B4 main code
        InsertMenu(POPMENU)
    END IF

    FOR i=0 TO ObjectIndex-1                                        ' Loop through the ObjectList...
        tstr=FIELD$(ObjectList(i),"~",1)                            ' ..for form objects
        IF FIELD$(tstr,"|",1)="0" THEN                              ' Type QFORM?
            InsertFormCode(i)                                       ' Add the initial form code
            InsertThisChain(i,lev)                                  ' Add ALL child code
            CodeText.AddStrings "END CREATE",""                     ' Add the form terminate code
        END IF
    NEXT i
    IF LTRIM$(MainMenuList)<>"" THEN                                ' QMAINMEMUs after main code
        InsertMenu(MAINMENU)
    END IF
    InsertNonVisible                                                ' Chuck in any non-visible components
    IF ProjectMode=STAND_ALONE THEN
        InsertCodeFooter                                            ' Add form termination code
    END IF
END SUB

'---------------------- Stand-alone saving code clipboard/file------------------------------------
'NB CodeText.CopyToClipboard not used .. clipboard cleared on exit of program
SUB CopyToClipBrd
    ClipBoard.Open
    ClipBoard.Clear                                                 ' Dump any contents
    IF DesignTab.Tag=2 THEN                                         ' RapidQ TAB page visible
        CodeText.SelectAll
        ClipBoard.Text=CodeText.Text                                ' Throw the lot onto the clipboard
        ClipBoard.Close                                             ' Remains when exiting
        CodeText.SelLength=0                                        ' De-select text
    ELSE
        CodeText2.SelectAll                                         ' FreeBasic code
        ClipBoard.Text=CodeText2.Text
        ClipBoard.Close
        CodeText2.SelLength=0
    END IF
    ComponentCount=ObjectIndex                                      ' Signal we have saved and can exit ok
    MessageBox("Your code has been placed on the clipboard","RapidFRM "+VER,&H40)
END SUB

SUB SelectionToClipBrd
    IF DesignTab.Tag=2 THEN                                         ' RapidQ TAB page visible
        IF CodeText.SelLength>0 THEN
            ClipBoard.Open
            ClipBoard.Clear                                         ' Dump any contents
            ClipBoard.Text=CodeText.SelText                         ' Selection onto the clipboard
            ClipBoard.Close                                         ' Remains when exiting
            CodeText.SelLength=0                                    ' De-select text
            ComponentCount=ObjectIndex                              ' Signal we have saved and can exit ok
            MessageBox("RapidQ code snippet has been placed on the clipboard","RapidFRM "+VER,&H40)
        ELSE
            MessageBox("No code has been selected","RapidFRM "+VER,&H30)
        END IF
    ELSE
        IF CodeText2.SelLength>0 THEN                               ' FreeBasic code
            ClipBoard.Open
            ClipBoard.Clear
            ClipBoard.Text=CodeText2.SelText                        ' Selection onto the clipboard
            ClipBoard.Close
            CodeText2.SelLength=0
            ComponentCount=ObjectIndex                              ' Signal we have saved and can exit ok
            MessageBox("FreeBasic code snippet has been placed on the clipboard","RapidFRM "+VER,&H40)
        ELSE
            MessageBox("No code has been selected","RapidFRM "+VER,&H30)
        END IF
    END IF
END SUB

' Save as either RapidQ or FreeBasic source code
SUB SaveFile
DIM SaveDialog AS QSAVEDIALOG
DEFSTR tstr

    SaveDialog.Filter="BAS file|*.bas|INC file|*.inc|TEXT file|*.txt"
    IF SaveDialog.Execute THEN
        IF INSTR(SaveDialog.FileName,".")=0 THEN
            tstr=FIELD$(Savedialog.Filter,"*",Savedialog.FilterIndex+1)
            SaveDialog.FileName=SaveDialog.FileName+LEFT$(tstr,4)
        END IF
        IF DesignTab.Tag=2 THEN                                     ' RapidQ TAB page visible
            CodeText.SaveToFile SaveDialog.FileName
        ELSE
            CodeText2.SaveToFile SaveDialog.FileName                ' FreeBasic visible
        END IF
        ComponentCount=ObjectIndex                                  ' Signal we have saved and can exit ok
    END IF
END SUB

'----------------------- Visually adding new components -------------------------------------
' Entry points AddNewObject() or AddNewForm()

FUNCTION GetDefaultProperties(ComponentID)
DEFINT i, j
DEFSTR tstr, token

    ComponentRef.Position=0                                         ' Stream contains default data ALL components
    token="# "+STR$(ComponentID): j=LEN(token)                      ' Components start with "# <ID> <Q-COMPONENT_NAME>"

    FOR i=1 TO ComponentRef.LineCount
        tstr=ComponentRef.Readline
        IF LEFT$(tstr,j)=token THEN
            EXIT FOR
        END IF
    NEXT i
    IF i>ComponentRef.LineCount THEN                                ' Just in case!
        tstr="GetDefaultProperties() - Component "+STR$(ComponentID)+" NOT found!"
        MessageBox(tstr,"RapidFRM "+VER,&h30)
        Result=""
        EXIT FUNCTION
    END IF

    tstr=""
    token=ComponentRef.Readline
    WHILE LEFT$(token,1)<>"#"
        tstr=tstr+token+"~"                                         ' All properties this component
        token=ComponentRef.Readline
    WEND
    Result=tstr
END FUNCTION

SUB UpdatePropGrid(idx)
DEFINT i, propCount, methodCount, eventCount, maxStrings
DEFSTR propString, tstr, token

    SelectionCombo.ItemIndex=idx

    propString=ObjectList(idx)                                      ' Properties & events this object
    tstr=FIELD$(propString,"~",1)                                   ' Header field string
    IF ProjectMode=STAND_ALONE THEN
        token=FIELD$(tstr,"|",4)                                    ' Internal object name
    ELSE
        token=FIELD$(tstr,"|",5)                                    ' Project object name
    END IF

    maxStrings=TALLY(propString,"~")
    propCount=0: methodCount=0: eventCount=0
    FOR i=2 TO maxStrings
        tstr=FIELD$(propString,"~",i)
        IF LEFT$(tstr,1)="*" THEN EXIT FOR                          ' Finished with properties?
        INC propCount                                               ' Component property
        IF (propCount+1)=PropertyGrid.RowCount THEN PropertyGrid.InsertRow(propCount)
        PropertyGrid.Cell(0,propCount)=FIELD$(tstr,"|",1)
        PropertyGrid.Cell(1,propCount)=FIELD$(tstr,"|",4)
    NEXT i
    FOR i=i+1 TO maxStrings
        tstr=FIELD$(propString,"~",i)
        IF LEFT$(tstr,1)="*" THEN EXIT FOR                          ' Finished with methods?
        INC methodCount                                             ' Component method
        IF (methodCount+1)=MethodGrid.RowCount THEN MethodGrid.InsertRow(methodCount)
        MethodGrid.Cell(0,methodCount)=FIELD$(tstr,"|",1)
        MethodGrid.Cell(1,methodCount)=FIELD$(tstr,"|",4)
    NEXT i
    FOR i=i+1 TO maxStrings
        tstr=FIELD$(propString,"~",i)
        INC eventCount                                              ' Component Event
        IF (eventCount+1)=EventGrid.RowCount THEN EventGrid.InsertRow(eventCount)
        EventGrid.Cell(0,eventCount)=FIELD$(tstr,"|",1)
        EventGrid.Cell(1,eventCount)=FIELD$(tstr,"|",2)
    NEXT i

    IF propCount=0 THEN                                             ' Some DirectX components!!
        propCount=1
        PropertyGrid.Cell(0,1)="No properties"
        PropertyGrid.Cell(1,1)=""
    END IF
    IF methodCount=0 THEN
        methodCount=1
        MethodGrid.Cell(0,1)="No design methods"
        MethodGrid.Cell(1,1)=""
    END IF
    IF eventCount=0 THEN
        eventCount=1
        EventGrid.Cell(0,1)="No events"
        EventGrid.Cell(1,1)=""
    END IF
    IF PropertyGrid.RowCount>(propCount+1) THEN
        FOR i=(propCount+2) TO PropertyGrid.RowCount
            PropertyGrid.DeleteRow(i)
        NEXT i
    END IF
    IF MethodGrid.RowCount>(methodCount+1) THEN
        FOR i=(methodCount+2) TO MethodGrid.RowCount
            MethodGrid.DeleteRow(i)
        NEXT i
    END IF
    IF EventGrid.RowCount>(eventCount+1) THEN
        FOR i=(eventCount+2) TO EventGrid.RowCount
            EventGrid.DeleteRow(i)
        NEXT i
    END IF
    UpdateStatus
END SUB

SUB AddNewForm
DEFINT i
DEFSTR tstr, token

    IF FrmIndex > MAX_FORMS THEN
        MessageBox("Maximum forms ("+STR$(MAX_FORMS)+") has been reached","RapidFRM "+VER,&H30)
        EXIT SUB
    END IF

    Scan.Enabled=False
    DesignTab.TabIndex=0                                            ' Ensure design tab visible
    DesignTabChange
    CaretsTimer.Enabled=False
    FOR i=1 TO 8
        Carets(i).Parent=Forms(FrmIndex)                            ' Set carets to this form
        Carets(i).Visible=False
    NEXT i

    CurrentForm=FrmIndex: CurrentObject=ObjectIndex
    token="Form"+STR$(FrmIndex)
    tstr=STR$(0)+"|QFORM|"+STR$(Forms(FrmIndex).Handle)+"|"+token+"|"+token+"|0~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(0)

'    BringWindowToTop(Forms(FrmIndex).Handle)
    SetWindowPos(Forms(FrmIndex).Handle,HWND_TOPMOST,0,0,0,0,FLAGS)
    Forms(FrmIndex).Show

    SelectionCombo.AddItems token+" [QFORM]"
    SelectionCombo.ItemIndex=CurrentObject

    INC ObjectIndex: INC FrmIndex: NewObjectID=-1
    Scan.Enabled=True
END SUB

' When ONLY single instance of this object per form allowed
FUNCTION FormHasObject(ComponentID AS integer) AS integer
DEFINT i, j, cnt, thisObject
DEFSTR objStr, tstr

    FOR i=1 TO MAX_OBJECTS
        objStr=ObjectList(i)                                        ' Search for this object type
        IF objStr="" THEN EXIT FOR
        tstr=FIELD$(objStr,"~",1)
        thisObject=VAL(FIELD$(tstr,"|",1))
        IF thisObject=ComponentID THEN
            cnt=TALLY(objStr,"~")
            FOR j=1 TO cnt                                          ' Check if object belongs to this form
                tstr=FIELD$(objStr,"~",j)
                IF FIELD$(tstr,"|",1)="Parent" THEN
                    tstr=FIELD$(tstr,"|",4)                         ' Parent form name
                    IF VAL(RIGHT$(tstr,1))=CurrentForm THEN
                        MessageBox("Only one instance per form allowed!","RapidFRM "+VER,&h30)
                        Result=True
                        EXIT FUNCTION
                    END IF
                END IF
            NEXT j
        END IF
    NEXT i
    Result=False
END FUNCTION

' When ONLY single instance of this object per parent object allowed
FUNCTION ParentHasObject(Parnt AS long, ComponentID AS integer) AS integer
DEFINT i, parntIdx
DEFSTR tstr

    FOR i=1 TO ObjectIndex-1
        tstr=FIELD$(ObjectList(i),"~",1)                            ' Search for any existing ComponentIDs
        IF ComponentID=VAL(FIELD$(tstr,"|",1)) THEN
            IF VAL(FIELD$(tstr,"|",6))=Parnt THEN                   ' This parent has this object type already
                MessageBox("Only one instance per parent object allowed!","RapidFRM "+VER,&h30)
                Result=True
                EXIT FUNCTION
            END IF
        END IF
    NEXT i
    Result=False
END FUNCTION

' Return the name of the object parent
FUNCTION WhichParent(hWnd AS long) AS string
DEFINT i
DEFSTR tstr

    Result=""
    FOR i=0 TO ObjectIndex-1
        tstr=FIELD$(ObjectList(i),"~",1)                            ' Each object in list
        IF hWnd=VAL(FIELD$(tstr,"|",3)) THEN                        ' Is this the handle?
            Result=FIELD$(tstr,"|",5)                               ' Object name
            EXIT FOR
        END IF
    NEXT i
END FUNCTION

SUB AddQEDIT(hWnd AS long)
DIM BoxEdit(EdIndex+1) AS QPANEL
DEFSTR tstr, token
DIM tPoint AS PointAPI

    BoxEdit(EdIndex).Width=100                                      ' Using a QPanel so set these...
    BoxEdit(EdIndex).Height=25                                      ' ...to the defaults of QEdit
    BoxEdit(EdIndex).Color=&hffffff
    BoxEdit(EdIndex).BevelOuter=2
    BoxEdit(EdIndex).Alignment=taLeftJustify
    token="Edit"+STR$(EdIndex+1)
    BoxEdit(EdIndex).Caption=token

    BoxEdit(EdIndex).Parent=Forms(CurrentForm)                      ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(BoxEdit(EdIndex).Handle,hWnd)                     ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)                                     ' Posn relative to parent
    BoxEdit(EdIndex).Left=tPoint.x
    BoxEdit(EdIndex).Top=tPoint.y
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QEDIT|"+STR$(BoxEdit(EdIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Caption",token,4)                          ' ..Width & Height defaults
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QEDIT]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid data
    INC ObjectIndex: INC EdIndex
END SUB

SUB AddQRICHEDIT(hWnd AS long)
DIM BoxRich(RichIndex+1) AS QPANEL
DEFSTR tstr, token
DIM tPoint AS PointAPI

    BoxRich(RichIndex).Width=200                                    ' Using a QPanel so MUST set these...
    BoxRich(RichIndex).Height=100                                   ' ...to the defaults of QRichEdit
    BoxRich(RichIndex).Color=&hffffff
    BoxRich(RichIndex).BevelOuter=2
    BoxRich(RichIndex).Alignment=taLeftJustify
    token="RichEd"+STR$(RichIndex+1)
    BoxRich(RichIndex).Caption=token

    BoxRich(RichIndex).Parent=Forms(CurrentForm)                    ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(BoxRich(RichIndex).Handle,hWnd)
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    BoxRich(RichIndex).Left=tPoint.x
    BoxRich(RichIndex).Top=tPoint.y
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QRICHEDIT|"+STR$(BoxRich(RichIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Caption",token,4)                          ' ..Width & Height defaults
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QRICHEDIT]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC RichIndex
END SUB

SUB AddQLABEL(hWnd AS long)
DIM BoxLabel(LabelIndex+1) AS QPANEL
DEFSTR tstr, token
DIM tPoint AS PointAPI

    BoxLabel(LabelIndex).Width=75                                   ' Using a QPanel so MUST set...
    BoxLabel(LabelIndex).Height=25                                  ' .. these to the defaults of QLabel
    BoxLabel(LabelIndex).BevelOuter=2
    BoxLabel(LabelIndex).Alignment=taLeftJustify
    token="Label"+STR$(LabelIndex+1)
    BoxLabel(LabelIndex).Caption=token

    BoxLabel(LabelIndex).Parent=Forms(CurrentForm)                  ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(BoxLabel(LabelIndex).Handle,hWnd)                 ' Set to the currect parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    BoxLabel(LabelIndex).Left=tPoint.x
    BoxLabel(LabelIndex).Top=tPoint.y
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QLABEL|"+STR$(BoxLabel(LabelIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Caption",token,4)                          ' ..Width & Height defaults
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QLABEL]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC LabelIndex
END SUB

SUB AddQBUTTON(hWnd AS long)
DIM boxBut(ButnIndex+1) AS QPANEL                                   ' Fudge using QPANEL
DEFSTR tstr, token
DIM tPoint AS PointAPI

    boxBut(ButnIndex).Width=75                                      ' Using a QPanel so set these...
    boxBut(ButnIndex).Height=25                                     ' ...to the defaults of QButton
    boxBut(ButnIndex).BevelOuter=2
    token="Butn"+STR$(ButnIndex+1)
    boxBut(ButnIndex).Caption=token

    boxBut(ButnIndex).Parent=Forms(CurrentForm)                     ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(boxBut(ButnIndex).Handle,hWnd)
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    boxBut(ButnIndex).Left=tPoint.x
    boxBut(ButnIndex).Top=tPoint.y
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QBUTTON|"+STR$(boxBut(ButnIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Caption",token,4)                          ' ..Width & Height defaults
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QBUTTON]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC ButnIndex
END SUB

SUB AddQCOOLBTN(hWnd AS long)
DIM BoxCool(CoolIndex+1) AS QPANEL                                  ' Fudge using QPANEL
DEFSTR tstr, token
DIM tPoint AS PointAPI

    BoxCool(CoolIndex).Width=25                                     ' Using a QPanel so MUST set...
    BoxCool(CoolIndex).Height=25                                    ' .. these to the defaults of QCoolBtn
    BoxCool(CoolIndex).BevelOuter=2
    token="Cool"+STR$(CoolIndex+1)
    BoxCool(CoolIndex).Caption=token

    BoxCool(CoolIndex).Parent=Forms(CurrentForm)                    ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(BoxCool(CoolIndex).Handle,hWnd)                   ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    BoxCool(CoolIndex).Left=tPoint.x
    BoxCool(CoolIndex).Top=tPoint.y
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QCOOLBTN|"+STR$(BoxCool(CoolIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Caption",token,4)                          ' ..Width & Height defaults
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QCOOLBTN]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC CoolIndex
END SUB

SUB AddQOVALBTN(hWnd AS long)
STATIC i
DEFSTR tstr, token
DIM tPoint AS PointAPI

    INC i
    REDIM panOval(i) AS QPANEL
    REDIM imgOval(i) AS QCANVAS

    token="OvalBtn"+STR$(i)
    panOval(i).Parent=Forms(CurrentForm)                            ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(panOval(i).Handle,hWnd)                           ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    panOval(i).Left=tPoint.x
    panOval(i).Top=tPoint.y
    panOval(i).Width=75                                             ' Using a QPanel so MUST set...
    panOval(i).Height=40                                            ' .. these to the defaults of QOvalBtn
    panOval(i).BevelWidth=0
    panOval(i).Caption=token

    imgOval(i).Parent=panOval(i)
    imgOval(i).Align=alClient
    imgOval(i).Enabled=False                                        ' Passes clicks thru to panel
    imgOval(i).Tag=i                                                ' Used when drawing the btn
    imgOval(i).OnPaint=OvalPaint
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QOVALBTN|"+STR$(panOval(i).Handle)+"|"+token+ _
                                                        "|"+token+"|"+STR$(hWnd)+"|"+STR$(i)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Caption",token,4)                          ' ..Width & Height defaults
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QOVALBTN]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex
END SUB

SUB AddQCHECKBOX(hWnd AS long)
DIM Check(CheckIndex+1) AS QCHECKBOX
DEFSTR tstr, token
DIM tPoint AS PointAPI

    token="ChkBox"+STR$(CheckIndex+1)
    Check(CheckIndex).Caption=token

    Check(CheckIndex).Parent=Forms(CurrentForm)                     ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(Check(CheckIndex).Handle,hWnd)                    ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    Check(CheckIndex).Left=tPoint.x
    Check(CheckIndex).Top=tPoint.y
    Check(CheckIndex).Width=100
    Check(CheckIndex).Height=20
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QCHECKBOX|"+STR$(Check(CheckIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Caption",token,4)                          ' ..Width & Height defaults
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QCHECKBOX]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC CheckIndex
END SUB

SUB AddQRADIOBUTTON(hWnd AS long)
DIM Radio(RadioIndex+1) AS QRADIOBUTTON
DEFSTR tstr, token
DIM tPoint AS PointAPI

    token="RadioBtn"+STR$(RadioIndex+1)
    Radio(RadioIndex).Caption=token

    Radio(RadioIndex).Parent=Forms(CurrentForm)                     ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(Radio(RadioIndex).Handle,hWnd)                    ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    Radio(RadioIndex).Left=tPoint.x
    Radio(RadioIndex).Top=tPoint.y
    Radio(RadioIndex).Width=100
    Radio(RadioIndex).Height=20
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QRADIOBUTTON|"+STR$(Radio(RadioIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Caption",token,4)                          ' ..Width & Height defaults
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QRADIOBUTTON]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC RadioIndex
END SUB

SUB AddQPANEL(hWnd AS long)
DIM dPan(PanIndex+1) AS QPANEL
DEFSTR tstr, token
DIM tPoint AS PointAPI

    token="Panel"+STR$(PanIndex+1)
    dPan(PanIndex).Caption=token
    dPan(PanIndex).Width=150
    dPan(PanIndex).Height=100

    dPan(PanIndex).Parent=Forms(CurrentForm)                        ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(dPan(PanIndex).Handle,hWnd)                       ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    dPan(PanIndex).Left=tPoint.x
    dPan(PanIndex).Top=tPoint.y
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QPANEL|"+STR$(dPan(PanIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Caption",token,4)                          ' ..Width & Height defaults
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QPANEL]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC PanIndex
END SUB

SUB AddQTABCONTROL(hWnd AS long)
DIM TabCtrl(TabContIndex+1) AS QTABCONTROL
DIM BoxTab(TabContIndex+1) AS QPANEL                                ' QPanel as a container
DIM btn AS QBUTTON
DEFLNG TabHandle
DEFSTR tstr, token
DIM tPoint AS PointAPI

    BoxTab(TabContIndex).Width=150                                  ' Non-Default for a QTabControl...
    BoxTab(TabContIndex).Height=100                                 ' ...Re-sized for convenience
    BoxTab(TabContIndex).Borderstyle=0
    BoxTab(TabContIndex).BevelOuter=2
    token="TabCtrl"+STR$(TabContIndex+1)

    BoxTab(TabContIndex).Parent=Forms(CurrentForm)                  ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(BoxTab(TabContIndex).Handle,hWnd)                 ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    BoxTab(TabContIndex).Left=tPoint.x
    BoxTab(TabContIndex).Top=tPoint.y

    TabCtrl(TabContIndex).Parent=BoxTab(TabContIndex)               '
    TabCtrl(TabContIndex).Align=alClient                            ' NB MUST do these AFTER container initialised
    TabCtrl(TabContIndex).AddTabs("T","A","B")                         '
    btn.Visible=False: btn.Parent=TabCtrl(TabContIndex)
    TabHandle=GetParent(btn.Handle)
    DestroyWindow(btn.Handle)

    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QTABCONTROL|"+STR$(BoxTab(TabContIndex).Handle)+"|" _
                            +token+"|"+token+"|"+STR$(hWnd)+"|"+STR$(TabHandle)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QTABCONTROL]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC TabContIndex
END SUB

' Minor problems with selecting this component at design time
SUB AddQCOMBOBOX(hWnd AS long)
DIM Combo(ComboIndex+1) AS QCOMBOBOX
DEFSTR tstr, token
DIM tPoint AS PointAPI

    token="Combo"+STR$(ComboIndex+1)
    Combo(ComboIndex).Text=token
    Combo(ComboIndex).Width=145
    Combo(ComboIndex).Parent=Forms(CurrentForm)                     ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(Combo(ComboIndex).Handle,hWnd)                    ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    Combo(ComboIndex).Left=tPoint.x
    Combo(ComboIndex).Top=tPoint.y
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QCOMBOBOX|"+STR$(Combo(ComboIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Text",token,4)                             '
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '
    SelectionCombo.AddItems token+" [QCOMBOBOX]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC ComboIndex
END SUB

SUB AddQLISTBOX(hWnd AS long)
DIM ListB(ListBIndex+1) AS QLISTBOX
DEFSTR tstr, token
DIM tPoint AS PointAPI

    token="ListBox"+STR$(ListBIndex+1)
    ListB(ListBIndex).Width=120
    ListB(ListBIndex).Height=100
    ListB(ListBIndex).Parent=Forms(CurrentForm)                     ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(ListB(ListBIndex).Handle,hWnd)                    ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    ListB(ListBIndex).Left=tPoint.x-2                               ' Fudge (-2) for correct positioning
    ListB(ListBIndex).Top=tPoint.y-2                                ' ditto
    ListB(ListBIndex).AddItems token
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QLISTBOX|"+STR$(ListB(ListBIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QLISTBOX]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC ListBIndex
END SUB

SUB AddQGROUPBOX(hWnd AS long)
DIM GrpBox(GrpBoxIndex+1) AS QGROUPBOX
DEFSTR tstr, token
DIM tPoint AS PointAPI

    token="GrpBox"+STR$(GrpBoxIndex+1)
    GrpBox(GrpBoxIndex).Caption=token
    GrpBox(GrpBoxIndex).Width=150
    GrpBox(GrpBoxIndex).Height=100
    GrpBox(GrpBoxIndex).Parent=Forms(CurrentForm)                   ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(GrpBox(GrpBoxIndex).Handle,hWnd)                  ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    GrpBox(GrpBoxIndex).Left=tPoint.x
    GrpBox(GrpBoxIndex).Top=tPoint.y
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QGROUPBOX|"+STR$(GrpBox(GrpBoxIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Caption",token,4)                          '
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QGROUPBOX]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC GrpBoxIndex
END SUB

' An odd-one-out -> Qpanel placed inside to force scroll bars visible
SUB AddQSCROLLBOX(hWnd AS long)
DIM ScrollB(ScrollBIndex+1) AS QSCROLLBOX
DIM BoxInner(ScrollBIndex+1) AS QPANEL                              ' For effect - force scrollbars visible
DEFSTR tstr, token
DIM tPoint AS PointAPI

    ScrollB(ScrollBIndex).Width=150                                 ' Force size of ScrollBox
    ScrollB(ScrollBIndex).Height=100
    BoxInner(ScrollBIndex).Width=500                                ' Force scrollbars visible
    BoxInner(ScrollBIndex).Height=500

    token="ScrlBox"+STR$(ScrollBIndex+1)
    ScrollB(ScrollBIndex).Parent=Forms(CurrentForm)                 ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(ScrollB(ScrollBIndex).Handle,hWnd)                ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    BoxInner(ScrollBIndex).Parent=ScrollB(ScrollBIndex)
    BoxInner(ScrollBIndex).Enabled=False                            ' Don't capture mouse clicks!
    ScrollB(ScrollBIndex).Left=tPoint.x-2                           ' Fudge for correct positioning
    ScrollB(ScrollBIndex).Top=tPoint.y-2                            ' ditto
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QSCROLLBOX|"+STR$(ScrollB(ScrollBIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QSCROLLBOX]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC ScrollBIndex
END SUB

SUB AddQSCROLLBAR(hWnd AS long)
DIM ScrollBar(ScrllBarIndex+1) AS QSCROLLBAR
DEFSTR tstr, token
DIM tPoint AS PointAPI

    token="ScrlBar"+STR$(ScrllBarIndex+1)
    ScrollBar(ScrllBarIndex).Width=100
    ScrollBar(ScrllBarIndex).Height=20
    ScrollBar(ScrllBarIndex).Parent=Forms(CurrentForm)              ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(ScrollBar(ScrllBarIndex).Handle,hWnd)             ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    ScrollBar(ScrllBarIndex).Left=tPoint.x
    ScrollBar(ScrllBarIndex).Top=tPoint.y
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QSCROLLBAR|"+STR$(ScrollBar(ScrllBarIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QSCROLLBAR]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC ScrllBarIndex
END SUB

' Arrays of StringGrid has problems - using QListView instead!!
SUB AddQSTRINGGRID(hWnd AS long)
DIM StrGrid(StringGIndex+1) AS QLISTVIEW
DEFSTR tstr, token
DIM tPoint AS PointAPI

    token="StringGrid"+STR$(StringGIndex+1)
    StrGrid(StringGIndex).Width=150
    StrGrid(StringGIndex).Height=100
    StrGrid(StringGIndex).Parent=Forms(CurrentForm)                 ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(StrGrid(StringGIndex).Handle,hWnd)                ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    StrGrid(StringGIndex).Left=tPoint.x-2                           ' Fudge to correct positioning
    StrGrid(StringGIndex).Top=tPoint.y-2                            ' ditto


    StrGrid(StringGIndex).GridLines=True                            '
    StrGrid(StringGIndex).ViewStyle=vsReport                        ' Some visual presets
    StrGrid(StringGIndex).AddColumns token,""                       '
    StrGrid(StringGIndex).Column(0).Width=74
    StrGrid(StringGIndex).Column(1).Width=74
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QSTRINGGRID|"+STR$(StrGrid(StringGIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QSTRINGGRID]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC StringGIndex
END SUB

SUB AddQLISTVIEW(hWnd AS long)
DIM ListView(ListVIndex+1) AS QLISTVIEW
DEFSTR tstr, token
DIM tPoint AS PointAPI

    token="ListView"+STR$(ListVIndex+1)
    ListView(ListVIndex).Width=150
    ListView(ListVIndex).Height=100
    ListView(ListVIndex).Parent=Forms(CurrentForm)                  ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(ListView(ListVIndex).Handle,hWnd)                 ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    ListView(ListVIndex).Left=tPoint.x-2                            ' Fudge fix positioning
    ListView(ListVIndex).Top=tPoint.y-2                             ' ditto

    ListView(ListVIndex).GridLines=True                             '
    ListView(ListVIndex).ViewStyle=vsReport                         ' Some visual presets
    ListView(ListVIndex).AddColumns token,""                        '
    ListView(ListVIndex).Column(0).Width=74
    ListView(ListVIndex).Column(1).Width=74
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QLISTVIEW|"+STR$(ListView(ListVIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QLISTVIEW]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC ListVIndex
END SUB

' Problems with this so using a QPANEL container
SUB AddQHEADER(hWnd AS long)
DIM Headr(HeadIndex+1) AS QHEADER
DIM BoxHeadr(HeadIndex+1) AS QPANEL
DEFSTR tstr, token
DIM rect AS QRECT
DIM tPoint AS PointAPI

    IF ParentHasObject(hWnd,NewObjectID) THEN EXIT SUB              ' ONLY 1 object per parent allowed!

    token="Header"+STR$(HeadIndex+1)
    BoxHeadr(HeadIndex).Align=alTop
    BoxHeadr(HeadIndex).Height=17
    BoxHeadr(HeadIndex).BevelOuter=2
    BoxHeadr(HeadIndex).Parent=Forms(CurrentForm)                   ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(BoxHeadr(HeadIndex).Handle,hWnd)                  ' Set to correct parent
    END IF
    Headr(HeadIndex).Parent=BoxHeadr(HeadIndex)
    Headr(HeadIndex).Align=alClient
    Headr(HeadIndex).AddSections "Hdr1","Hdr2","Hdr3"
    Headr(HeadIndex).Enabled=False
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QHEADER|"+STR$(BoxHeadr(HeadIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)

    SelectionCombo.AddItems token+" [QHEADER]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC HeadIndex
END SUB

SUB AddQSTATUSBAR(hWnd AS long)
DIM StatBar(StatIndex+1) AS QSTATUSBAR
DIM BoxStat(StatIndex+1) AS QPANEL
DEFSTR tstr, token
DIM tPoint AS PointAPI

    IF ParentHasObject(hWnd,NewObjectID) THEN EXIT SUB              ' ONLY 1 object per parent allowed!

    token="Status"+STR$(StatIndex+1)
    BoxStat(StatIndex).Align=alBottom
    BoxStat(StatIndex).Height=19
    BoxStat(StatIndex).BevelOuter=2
    BoxStat(StatIndex).Parent=Forms(CurrentForm)                    ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(BoxStat(StatIndex).Handle,hWnd)                   ' Set to correct parent
    END IF
    StatBar(StatIndex).Parent=BoxStat(StatIndex)
    StatBar(StatIndex).Align=alClient
    StatBar(StatIndex).SimplePanel=True
    StatBar(StatIndex).SimpleText=token
    StatBar(StatIndex).Enabled=False
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QSTATUSBAR|"+STR$(BoxStat(StatIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)

    SelectionCombo.AddItems token+" [QSTATUSBAR]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC StatIndex
END SUB

SUB AddQDIRTREE(hWnd AS long)
DIM DirTree AS QDIRTREE
DIM BoxDTree AS QPANEL
DEFSTR tstr, token
DIM tPoint AS PointAPI

    IF DirTIndex>0 THEN                                             ' ONLY 1 allowed!
        MessageBox("Internally limited to one instance only!","RapidFRM "+VER,&h30)
        EXIT SUB
    END IF

    INC DirTIndex
    token="DirTree"+STR$(DirTIndex)
    BoxDTree.Width=150
    BoxDTree.Height=150
    BoxDTree.BevelOuter=2
    BoxDTree.Parent=Forms(CurrentForm)                              ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(BoxDTree.Handle,hWnd)                             ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    BoxDTree.Left=tPoint.x: BoxDTree.Top=tPoint.y
    DirTree.Parent=BoxDTree: DirTree.Align=alClient: DirTree.Enabled=False
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QDIRTREE|"+STR$(BoxDTree.Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QDIRTREE]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex
END SUB

SUB AddQFILELISTBOX(hWnd AS long)
DIM dFileL(FileLindex+1) AS QFILELISTBOX
DIM BoxFile(FileLindex+1) AS QPANEL
DEFSTR tstr, token
DIM tPoint AS PointAPI

    token="FileListBox"+ STR$(FileLindex+1)
    BoxFile(FileLindex).Width=150
    BoxFile(FileLindex).Height=150
    BoxFile(FileLindex).BevelOuter=2
    BoxFile(FileLindex).Parent=Forms(CurrentForm)                   ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(BoxFile(FileLindex).Handle,hWnd)                  ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    BoxFile(FileLindex).Left=tPoint.x
    BoxFile(FileLindex).Top=tPoint.y
    dFileL(FileLindex).Parent=BoxFile(FileLindex)
    dFileL(FileLindex).Align=alClient
    dFileL(FileLindex).Enabled=False
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QFILELISTBOX|"+STR$(BoxFile(FileLindex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QFILELISTBOX]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC FileLindex
END SUB

SUB AddQGAUGE(hWnd AS long)                                         ' Fudge using QPANEL
DIM dGauge(GaugeIndex+1) AS QGAUGE
DIM panGauge(GaugeIndex+1) AS QPANEL
DEFSTR tstr, token
DIM tPoint AS PointAPI

    token="Gauge"+STR$(GaugeIndex+1)
    panGauge(GaugeIndex).Width=100
    panGauge(GaugeIndex).Height=25
    panGauge(GaugeIndex).BevelOuter=2
    panGauge(GaugeIndex).Parent=Forms(CurrentForm)                  ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(panGauge(GaugeIndex).Handle,hWnd)                 ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    panGauge(GaugeIndex).Left=tPoint.x
    panGauge(GaugeIndex).Top=tPoint.y
    dGauge(GaugeIndex).Parent=panGauge(GaugeIndex)
    dGauge(GaugeIndex).Align=alClient
    dGauge(GaugeIndex).Position=75
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QGAUGE|"+STR$(panGauge(GaugeIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QGAUGE]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC GaugeIndex
END SUB

SUB AddQTRACKBAR(hWnd AS long)
DIM dTrackBar(TrackBIndex+1) AS QTRACKBAR
DEFSTR tstr, token
DIM tPoint AS PointAPI

    token="TrackBar"+STR$(TrackBIndex+1)
    dTrackBar(TrackBIndex).Width=150
    dTrackBar(TrackBIndex).Height=45
    dTrackBar(TrackBIndex).Parent=Forms(CurrentForm)                ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(dTrackBar(TrackBIndex).Handle,hWnd)               ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    dTrackBar(TrackBIndex).Left=tPoint.x
    dTrackBar(TrackBIndex).Top=tPoint.y
    dTrackBar(TrackBIndex).Max=20
    dTrackBar(TrackBIndex).Position=15
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QTRACKBAR|"+STR$(dTrackBar(TrackBIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QTRACKBAR]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC TrackBIndex
END SUB

' Defaults to a "dummy" vertical splitter
SUB AddQSPLITTER(hWnd AS long)                                      ' Fudge using a QPanel
DIM panSplit(SplitIndex+1) AS QPANEL
DEFSTR tstr, token
DIM tPoint AS PointAPI

    hWnd=Forms(CurrentForm).Handle                                  ' FORCE to CurrentForm!
    IF ParentHasObject(hWnd,NewObjectID) THEN EXIT SUB              ' ONLY 1 object parent allowed!

    token="Split"+STR$(SplitIndex+1)
    panSplit(SplitIndex).Parent=Forms(CurrentForm)                  ' Set parent property to form
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    panSplit(SplitIndex).Left=tPoint.x
    panSplit(SplitIndex).Top=0
    panSplit(SplitIndex).Height=Forms(CurrentForm).ClientHeight
    panSplit(SplitIndex).Width=3
    panSplit(SplitIndex).BorderStyle=1
    panSplit(SplitIndex).Cursor=crHSplit
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QSPLITTER|"+STR$(panSplit(SplitIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top","0",4)                                ' Update new properties in ObjectList..
    PropertyFieldUpdate("Width","3",4)                              '
    PropertyFieldUpdate("Height",STR$(Forms(CurrentForm).ClientHeight),4)    '
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QSPLITTER]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC SplitIndex
END SUB

SUB AddQTREEVIEW(hWnd AS long)
DIM TreeView AS QTREEVIEW
DEFSTR tstr, token
DIM tPoint AS PointAPI

    IF TreeVIndex>0 THEN                                            ' ONLY 1 allowed!
        MessageBox("Only one instance allowed!","RapidFRM "+VER,&h30)
        EXIT SUB
    END IF
    INC TreeVIndex

    token="TreeView"+STR$(TreeVIndex)
    TreeView.Width=120
    TreeView.Height=100
    TreeView.Parent=Forms(CurrentForm)                              ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(TreeView.Handle,hWnd)                             ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    TreeView.Left=tPoint.x-2
    TreeView.Top=tPoint.y-2
    TreeView.ReadOnly=True
    TreeView.AddItems "1","2","3"
    TreeView.AddChildItems 0, "Child A", "Child B", "Child C"
    TreeView.AddChildItems 4, "Child X", "Child Y", "Child Z"
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QTREEVIEW|"+STR$(TreeView.Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QTREEVIEW]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex
END SUB

' FIXME: Problem with arrays of QOutLine
SUB AddQOUTLINE(hWnd AS long)
DIM frmOutLine AS QOUTLINE
DEFSTR tstr, token
DIM tPoint AS PointAPI

    IF OutLnIndex>0 THEN                                            ' ONLY 1 allowed!
        MessageBox("Only one instance allowed!","RapidFRM "+VER,&h30)
        EXIT SUB
    END IF
    INC OutLnIndex

    token="OutLine"+STR$(OutLnIndex)
    frmOutLine.Width=120
    frmOutLine.Height=100
    frmOutLine.Parent=Forms(CurrentForm)                            ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(frmOutLine.Handle,hWnd)                           ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    frmOutLine.Left=tPoint.x-2
    frmOutLine.Top=tPoint.y-2
    frmOutLine.AddLines token,"Child1","Child2"
    frmOutLine.AddLines "Parent2","Child1","Child2"
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QOUTLINE|"+STR$(frmOutLine.Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QOUTLINE]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex
END SUB

SUB AddQOLECONTAINER(hWnd AS long)                                  ' Fudge using QPANEL
DIM oleContainer(OLEContIndex+1) AS QPANEL
DEFSTR tstr, token
DIM tPoint AS PointAPI

    token="OLEContainer"+STR$(OLEContIndex+1)
    oleContainer(OLEContIndex).Width=100
    oleContainer(OLEContIndex).Height=100
    oleContainer(OLEContIndex).BevelOuter=2
    oleContainer(OLEContIndex).Parent=Forms(CurrentForm)            ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(oleContainer(OLEContIndex).Handle,hWnd)           ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    oleContainer(OLEContIndex).Left=tPoint.x
    oleContainer(OLEContIndex).Top=tPoint.y
    oleContainer(OLEContIndex).Caption=token
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QOLECONTAINER|"+STR$(oleContainer(OLEContIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Caption",token,4)                          '
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QOLECONTAINER]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC OLEContIndex
END SUB

SUB AddQCANVAS(hWnd AS long)                                        ' Fudge using QPANEL
DIM panCanv(CanvasIndex+1) AS QPANEL
DEFSTR tstr, token
DIM tPoint AS PointAPI

    token="Canvas"+STR$(CanvasIndex+1)
    panCanv(CanvasIndex).Width=100
    panCanv(CanvasIndex).Height=100
    panCanv(CanvasIndex).Parent=Forms(CurrentForm)                  ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(panCanv(CanvasIndex).Handle,hWnd)                 ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    panCanv(CanvasIndex).Left=tPoint.x
    panCanv(CanvasIndex).Top=tPoint.y
    panCanv(CanvasIndex).Caption=token
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QCANVAS|"+STR$(panCanv(CanvasIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Caption",token,4)                          '
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QCANVAS]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC CanvasIndex
END SUB

SUB AddQIMAGE(hWnd AS long)                                         ' Fudge using QPANEL
DIM panImage(ImageIndex+1) AS QPANEL
DEFSTR tstr, token
DIM tPoint AS PointAPI

    token="Image"+STR$(ImageIndex+1)
    panImage(ImageIndex).Width=100
    panImage(ImageIndex).Height=100
    panImage(ImageIndex).BevelOuter=2
    panImage(ImageIndex).Color=&hffffff
    panImage(ImageIndex).Parent=Forms(CurrentForm)                  ' MUST set a valid parent first!
    IF hWnd<>Forms(CurrentForm).Handle THEN
        SetParent(panImage(ImageIndex).Handle,hWnd)                 ' Set to correct parent
    END IF
    tPoint.x=Screen.MouseX: tPoint.y=Screen.MouseY
    ScreenToClient(hWnd,tPoint)
    panImage(ImageIndex).Left=tPoint.x
    panImage(ImageIndex).Top=tPoint.y
    panImage(ImageIndex).Caption=token
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|QIMAGE|"+STR$(panImage(ImageIndex).Handle)+"|"+token+"|"+token+"|"+STR$(hWnd)+"~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)
    PropertyFieldUpdate("Left",STR$(tPoint.x),4)                    '
    PropertyFieldUpdate("Top",STR$(tPoint.y),4)                     ' Update new properties in ObjectList..
    PropertyFieldUpdate("Caption",token,4)                          '
    PropertyFieldUpdate("Parent",WhichParent(hWnd),4)               '

    SelectionCombo.AddItems token+" [QIMAGE]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC ImageIndex
END SUB

' Non-visible components
SUB AddNonVisible(hWnd AS long)
DIM dNonVis(NonVIndex+1) AS QBUTTON
DEFINT i, ObjIdx
DEFSTR tstr, ObjType, ObjName

    SELECT CASE NewObjectID
        CASE Q_BITMAP
            INC BmpIndex: ObjIdx=BmpIndex
            ObjName="Bitmap"+STR$(ObjIdx): ObjType="QBITMAP"
        CASE Q_IMAGELIST
            INC ImgListIndex: ObjIdx=ImgListIndex
            ObjName="ImageList"+STR$(ObjIdx): ObjType="QIMAGELIST"
        CASE Q_STRINGLIST
            INC StrListIndex: ObjIdx=StrListIndex
            ObjName="StringList"+STR$(ObjIdx): ObjType="QSTRINGLIST"
        CASE Q_TIMER
            INC TimerIndex: ObjIdx=TimerIndex
            ObjName="Timer"+STR$(ObjIdx): ObjType="QTIMER"
        CASE Q_SOCKET
            INC SockIndex: ObjIdx=SockIndex
            ObjName="Socket"+STR$(ObjIdx): ObjType="QSOCKET"
        CASE Q_FILESTREAM
            INC FStreamIndex: ObjIdx=FStreamIndex
            ObjName="FileStream"+STR$(ObjIdx): ObjType="QFILESTREAM"
        CASE Q_MEMORYSTREAM
            INC MStreamIndex: ObjIdx=MStreamIndex
            ObjName="MemStream"+STR$(ObjIdx): ObjType="QMEMORYSTREAM"
        CASE Q_FONT
            INC FontIndex: ObjIdx=FontIndex
            ObjName="Font"+STR$(ObjIdx): ObjType="QFONT"
        CASE Q_MYSQL
            INC SQLIndex: ObjIdx=SQLIndex
            ObjName="MySQL"+STR$(ObjIdx): ObjType="QMYSQL"
        CASE Q_OLEOBJECT
            INC OleObjIndex: ObjIdx=OleObjIndex
            ObjName="OLEObject"+STR$(ObjIdx): ObjType="QOLEOBJECT"
        CASE Q_REGISTRY
            INC RegIndex: ObjIdx=RegIndex
            ObjName="Registry"+STR$(ObjIdx): ObjType="QREGISTRY"
        CASE Q_OPENDIALOG
'            IF OpenDlgIndex>0 THEN EXIT SUB                            ' One instance only?
            INC OpenDlgIndex: ObjIdx=OpenDlgIndex
            ObjName="OpenDialog"+STR$(ObjIdx): ObjType="QOPENDIALOG"
        CASE Q_SAVEDIALOG
'            IF SaveDlgIndex>0 THEN EXIT SUB
            INC SaveDlgIndex: ObjIdx=SaveDlgIndex
            ObjName="SaveDialog"+STR$(ObjIdx): ObjType="QSAVEDIALOG"
        CASE Q_FONTDIALOG
'            IF FontDlgIndex>0 THEN EXIT SUB
            INC FontDlgIndex: ObjIdx=FontDlgIndex
            ObjName="FontDialog"+STR$(ObjIdx): ObjType="QFONTDIALOG"
        CASE Q_COLORDIALOG
'            IF ColorDlgIndex>0 THEN EXIT SUB
            INC ColorDlgIndex: ObjIdx=ColorDlgIndex
            ObjName="ColorDialog"+STR$(ObjIdx): ObjType="QCOLORDIALOG"
        CASE Q_DXTIMER
'            IF DXTimerIdx>0 THEN EXIT SUB
            INC DXTimerIdx: ObjIdx=DXTimerIdx
            ObjName="DXTimer"+STR$(ObjIdx): ObjType="QDXTIMER"
        CASE Q_DXSOUND
'            IF DXSoundIdx>0 THEN EXIT SUB
            INC DXSoundIdx: ObjIdx=DXSoundIdx
            ObjName="DXSound"+STR$(ObjIdx): ObjType="QDXSOUND"
        CASE Q_DXSCREEN
'            IF DXScreenIdx>0 THEN EXIT SUB
            INC DXScreenIdx: ObjIdx=DXScreenIdx
            ObjName="DXScreen"+STR$(ObjIdx): ObjType="QDXSCREEN"
        CASE Q_DXIMAGELIST
'            IF DXImgListIdx>0 THEN EXIT SUB
            INC DXImgListIdx: ObjIdx=DXImgListIdx
            ObjName="DXImgList"+STR$(ObjIdx): ObjType="QDXIMAGELIST"
        CASE Q_D3DFACE
'            IF D3DFaceIdx>0 THEN EXIT SUB
            INC D3DFaceIdx: ObjIdx=D3DFaceIdx
            ObjName="D3DFace"+STR$(ObjIdx): ObjType="QD3DFACE"
        CASE Q_D3DMESHBUILDER
'            IF MeshBuilderIdx>0 THEN EXIT SUB
            INC MeshBuilderIdx: ObjIdx=MeshBuilderIdx
            ObjName="MeshBuilder"+STR$(ObjIdx): ObjType="QD3DMESHBUILDER"
        CASE Q_D3DFRAME
'            IF D3DFrameIdx>0 THEN EXIT SUB
            INC D3DFrameIdx: ObjIdx=D3DFrameIdx
            ObjName="D3DFrame"+STR$(ObjIdx): ObjType="QD3DFRAME"
        CASE Q_D3DLIGHT
'            IF D3DLightIdx>0 THEN EXIT SUB
            INC D3DLightIdx: ObjIdx=D3DLightIdx
            ObjName="D3DLight"+STR$(ObjIdx): ObjType="QD3DLIGHT"
        CASE Q_D3DTEXTURE
'            IF TextureIdx>0 THEN EXIT SUB
            INC TextureIdx: ObjIdx=TextureIdx
            ObjName="D3DTexture"+STR$(ObjIdx): ObjType="QD3DTEXTURE"
        CASE Q_D3DVECTOR
'            IF VectorIdx>0 THEN EXIT SUB
            INC VectorIdx: ObjIdx=VectorIdx
            ObjName="D3DVector"+STR$(ObjIdx): ObjType="QD3DVECTOR"
        CASE Q_D3DWRAP
'            IF D3DWrapIdx>0 THEN EXIT SUB
            INC D3DWrapIdx: ObjIdx=D3DWrapIdx
            ObjName="D3DWrap"+STR$(ObjIdx): ObjType="QD3DWRAP"
        CASE ELSE
            EXIT SUB
    END SELECT

    dNonVis(NonVIndex).Parent=NonVisPanel
    dNonVis(NonVIndex).Left=NonVisCount*32 + 5                      ' Position the button
    dNonVis(NonVIndex).Top=16
    dNonVis(NonVIndex).Width=30
    dNonVis(NonVIndex).Height=30
    dNonVis(NonVIndex).Hint=ObjName                                 ' Current component name
    dNonVis(NonVIndex).ShowHint=True
    dNonVis(NonVIndex).BMP=NonVisBMP.BMP                            ' Copy the bitmap image
    CurrentObject=ObjectIndex

    tstr=STR$(NewObjectID)+"|"+ObjType+"|"+STR$(dNonVis(NonVIndex).Handle)+"|"+ObjName+"|"+ObjName+"|0~"
    ObjectList(CurrentObject)=tstr+GetDefaultProperties(NewObjectID)

    SelectionCombo.AddItems ObjName+" ["+ObjType+"]"
    SelectionCombo.ItemIndex=CurrentObject
    UpdatePropGrid(CurrentObject)                                   ' Update String grid
    INC ObjectIndex: INC NonVIndex: INC NonVisCount
END SUB

' Checks that parent is a valid type
FUNCTION IsParentValid(hWnd AS long) AS integer
DEFINT i
DEFSTR tstr

    FOR i=0 TO ObjectIndex-1
        tstr=ObjectList(i)
        IF hWnd=VAL(FIELD$(tstr,"|",3)) THEN
            tstr=FIELD$(tstr,"|",2)                                 ' Object type?
            SELECT CASE tstr                                        ' Is parent type?
                CASE "QFORM","QPANEL","QTABCONTROL","QGROUPBOX","QSCROLLBOX"
                    Result=True
                CASE ELSE
                    Result=False
            END SELECT
            EXIT FOR
        END IF
    NEXT i
END FUNCTION

SUB AddNewObject
DEFLNG hWnd

    IF ObjectIndex>MAX_OBJECTS THEN
        MessageBox("Maximum components ("+STR$(MAX_OBJECTS)+") has been reached","RapidFRM "+VER,&H30)
        NewObjectID=-1
        EXIT SUB
    END IF
    IF NewObjectID < NON_VISIBLE THEN
        hWnd=WindowFromPoint(Screen.MouseX,Screen.MouseY)
        IF IsParentValid(hWnd)=False THEN
            hWnd=GetParent(hWnd)
            IF IsParentValid(hWnd)=False THEN
                MessageBox("ERROR: Non-valid parent?","RapidFRM "+VER,&H30)
                NewObjectID=-1
                EXIT SUB
            END IF
        END IF
    END IF
    HideCarets
    SELECT CASE NewObjectID
        CASE 1: AddQEDIT(hWnd)
        CASE 2: AddQRICHEDIT(hWnd)
        CASE 3: AddQLABEL(hWnd)
        CASE 4: AddQBUTTON(hWnd)
        CASE 5: AddQCOOLBTN(hWnd)
        CASE 6: AddQOVALBTN(hWnd)
        CASE 7: AddQCHECKBOX(hWnd)
        CASE 8: AddQRADIOBUTTON(hWnd)
        CASE 9: AddQPANEL(hWnd)
        CASE 10: AddQTABCONTROL(hWnd)
        CASE 11: AddQCOMBOBOX(hWnd)
        CASE 12: AddQLISTBOX(hWnd)
        CASE 13: AddQGROUPBOX(hWnd)
        CASE 14: AddQSCROLLBOX(hWnd)
        CASE 15: AddQSCROLLBAR(hWnd)
        CASE 16: AddQSTRINGGRID(hWnd)
        CASE 17: AddQLISTVIEW(hWnd)
        CASE 18: AddQDIRTREE(hWnd)
        CASE 19: AddQFILELISTBOX(hWnd)
        CASE 20: AddQTREEVIEW(hWnd)
        CASE 21: AddQOUTLINE(hWnd)
        CASE 22: AddQGAUGE(hWnd)
        CASE 23: AddQTRACKBAR(hWnd)
        CASE 24: AddQHEADER(hWnd)
        CASE 25: AddQSTATUSBAR(hWnd)
        CASE 26: AddQCANVAS(hWnd)
        CASE 27: AddQIMAGE(hWnd)
        CASE 28: AddQOLECONTAINER(hWnd)
        CASE 29: AddQSPLITTER(hWnd)
        CASE 30 TO 55: AddNonVisible(hWnd)
    END SELECT
    NewObjectID=-1
END SUB

'----------------- Template insertion for QMAINMENU/QPOPUPMENU --------------------
' Entry point MenuEditor()
'
' Update the temp global string with original menu code
' plus the newly selected code. Update screen text.
SUB LoadMenu(which AS integer, index AS integer)
DEFINT i
DEFSTR tstr, token, ThisParent=""

    MenuText.Clear
    IF which=MAINMENU THEN
        MenuUpdateStr=LTRIM$(MainMenuList)                          ' Load current QMAINMENU code
        token="#7"                                                  ' Marker in help/reference file
        ThisParent=LTRIM$(RTRIM$(cbMMenu.Item(cbMMenu.ItemIndex)))  ' A parent selected?
    ELSE
        MenuUpdateStr=LTRIM$(PopMenuList)                           ' Load current QPOPUPMENU code
        token="#8"                                                  ' Marker in help/reference file
    END IF
    HelpRef.Position=0
    FOR i=1 TO HelpRef.LineCount
        tstr=HelpRef.ReadLine
        IF FIELD$(tstr," ",1)=token THEN EXIT FOR                   ' Position within the mem stream
    NEXT i

    token=STR$(index)                                               ' A unique index
    tstr="The following code will be added to your project:"
    MenuText.AddStrings "", tstr, ""
    FOR i=i+1 TO HelpRef.LineCount                                  ' Fill the menu editbox
        tstr=HelpRef.ReadLine-CHR$(13)-CHR$(10)
        IF LEFT$(tstr,1)="#" THEN EXIT FOR                          ' End of section text
        IF INSTR(tstr,"##")>0 THEN tstr=REPLACESUBSTR$(tstr,"##",token)     ' Insert unique ID
        IF INSTR(UCASE$(tstr),"PARENT")>0 THEN
            IF ThisParent<>"" THEN tstr="    Parent = "+ThisParent
        END IF
        IF NOT ((chkMenu.Checked=False) AND (LEFT$(tstr,1)="'")) THEN
            MenuUpdateStr=MenuUpdateStr+LTRIM$(tstr)+"~"            ' Update the new string
            IF INSTR(UCASE$(tstr),"CAPTION")>0 THEN
                tstr=RTRIM$(FIELD$(tstr,"=",1))+" = "+QU+LTRIM$(FIELD$(tstr,"=",2))+QU
            END IF
            MenuText.AddStrings tstr                                ' Update the displayed text
        END IF
    NEXT i
    MenuUpdateStr=LEFT$(MenuUpdateStr,LEN(MenuUpdateStr)-1)+"@"     ' Terminate string with "@"
END SUB

' User selected option
SUB OptionChange
    SELECT CASE btnUpdate.Tag
        CASE MAINMENU: LoadMenu(MAINMENU,MainMenuIndex)             ' Place text in display window
        CASE POPMENU: LoadMenu(POPMENU,PopUpIndex)                  ' ditto
    END SELECT
END SUB

' Update display of QMAINMENU
SUB ParentChange
    IF btnUpdate.Tag=MAINMENU THEN                                  ' Which menu adding?
        btnUpdate.Enabled=True                                      ' Enable the "save" button
        LoadMenu(MAINMENU,MainMenuIndex)                            ' Place text in display window
    END IF
END SUB

' Display a QMAINMENU code template
SUB AddMainMenu
    btnUpdate.Tag=MAINMENU                                          ' Which menu adding?
    btnUpdate.Enabled=True                                          ' Enable the "save" button
    LoadMenu(MAINMENU,MainMenuIndex)                                ' Place text in display window
END SUB

' Display a QPOPUPMENU code template
SUB AddPopupMenu
    btnUpdate.Tag=POPMENU                                           ' Which menu adding?
    btnUpdate.Enabled=True                                          ' Enable the "save" button
    LoadMenu(POPMENU,PopUpIndex)                                    ' Place text in display window
END SUB

SUB UpdateFwdDeclares
DEFSTR tstr, token
DEFINT i

    SELECT CASE btnUpdate.Tag
        CASE MAINMENU: tstr=MenuUpdateStr-MainMenuList
        CASE POPMENU: tstr=MenuUpdateStr-PopMenuList
    END SELECT
    FOR i=1 TO TALLY(tstr,"~")
        token=FIELD$(tstr,"~",i)
        IF RTRIM$(FIELD$(token,"=",1))="OnClick" THEN
            FwdDeclares=FwdDeclares+"SUB "+LTRIM$(FIELD$(token,"=",2))+"~"
        END IF
    NEXT i
END SUB

' Insert selected code into project
SUB UpdateProjectMenu
DEFSTR tstr

    UpdateFwdDeclares
    SELECT CASE btnUpdate.Tag
        CASE MAINMENU
            MainMenuList=MenuUpdateStr                              ' Update the global string
            INC MainMenuIndex                                       ' Update the unique index
            tstr=RTRIM$(cbMMenu.Item(cbMMenu.ItemIndex))            ' Attached to a project form?
            IF tstr<>"" THEN
                cbMMenu.DelItems(cbMMenu.ItemIndex)                 ' Remove form from combo
            END IF
        CASE POPMENU
            PopMenuList=MenuUpdateStr                               ' Update the global string
            INC PopUpIndex                                          ' Update the unique index
    END SELECT
    btnUpdate.Tag=0                                                 ' Clear "saved" menu option
    btnUpdate.Enabled=False
    MenuText.Clear
END SUB

' "Menus" TAB selected
SUB MenuEditor
DEFINT i, j
DEFSTR tstr, token, frmNames=" "

    cbMMenu.Clear                                                   ' Start with an empty combobox
    FOR i=0 TO ObjectIndex-1
        tstr=FIELD$(ObjectList(i),"~",1)
        IF FIELD$(tstr,"|",2)="QFORM" THEN
            frmNames=frmNames+FIELD$(tstr,"|",5)+"~"
        END IF
    NEXT i
    frmNames=LTRIM$(frmNames)

    MainMenuIndex=TALLY(MainMenuList,"@")
    IF MainMenuIndex>0 THEN
        FOR i=1 TO MainMenuIndex                                    ' From QMAINMENUs list
            tstr=LTRIM$(FIELD$(MainMenuList,"@",i))+"~"
            IF INSTR(UCASE$(tstr),"PARENT")>0 THEN                  ' Has a parent form?
                FOR j=1 TO TALLY(tstr,"~")
                    token=FIELD$(tstr,"~",j)
                    IF INSTR(UCASE$(token),"PARENT")>0 THEN
                        token=LTRIM$(FIELD$(token,"=",2))+"~"
                        frmNames=frmNames-token                     ' Add form name to list
                    END IF
                NEXT j
            END IF
        NEXT i
    END IF

    cbMMenu.AddItems(" ")                                           ' Add blank parent entry!
    FOR i=1 TO TALLY(frmNames,"~")
        tstr=FIELD$(frmNames,"~",i)
        cbMMenu.AddItems(tstr)                                      ' Add form name to combobox
    NEXT i
    cbMMenu.Text="<Select a parent form>"                           ' Display default text

    INC MainMenuIndex                                               ' A unique ID?
    PopUpIndex=TALLY(PopMenuList,"@")+1                             ' ditto

    MenuPanel.Visible=True
    MenuPanel.Repaint
    MenuText.Clear
    btnUpdate.Enabled=False                                         ' Disable the save button
END SUB

'
'--------------------- Display loaded project code ----------------------------
' Parse the input stream, then populate the Form Designer screen with user info
' Entry point ParseInputStream()
'------------------------------------------------------------------------------
' Update the ObjectList string with object name
SUB UpdateObjList(objectName AS string)
DEFINT i, cnt
DEFSTR tstr, tok1, tok2

    tstr=ObjectList(CurrentObject)                                  ' This component property strings
    tok1=FIELD$(tstr,"~",1)                                         ' Component identifier string
    tstr=tstr-tok1

    cnt=TALLY(tok1,"|")+1                                           ' Number of fields
    tok2=""
    FOR i=1 TO 4
        tok2=tok2+FIELD$(tok1,"|",i)+"|"                            ' Build first four fields
    NEXT i
    tok2=tok2+objectName                                            ' Add new fifth field
    FOR i=i+1 TO cnt
        tok2=tok2+"|"+FIELD$(tok1,"|",i)                            ' Add remainder of fields
    NEXT i
    tstr=tok2+tstr                                                  ' Replace new identifier string
    ObjectList(CurrentObject)=tstr                                  ' Put back into global string array
END SUB

SUB UpdateObjCombo(newObject AS string)
    SelectionCombo.DelItems(ObjectIndex-1)                          ' Remove the default obj name
    SelectionCombo.AddItems newObject                               ' Add the real object name
END SUB

SUB UpdateNewProperty(srcString AS string)
DEFINT i, offset
DEFSTR ObjString, tstr, token

    ObjString=ObjectList(CurrentObject)                             ' All properties/events this object
    offset=4                                                        ' Default -> properties 4th field
    token=UCASE$(RTRIM$(FIELD$(srcString,"=",1)))                   ' Update this property or event
    IF (LEFT$(token,2)="ON") OR (token="WNDPROC") THEN offset=2     ' An Event? Update 2nd field
    FOR i=1 TO TALLY(ObjString,"~")
        tstr=FIELD$(ObjString,"~",i)
        IF UCASE$(FIELD$(tstr,"|",1))=token THEN                    ' Found the property
            token=LTRIM$(FIELD$(srcString,"=",2))                   ' New value
            PropertyFieldUpdate(FIELD$(tstr,"|",1),token,offset)    ' Update global ObjectList()
            EXIT FOR
        END IF
    NEXT i
END SUB

' Start=second line of CREATE .. END CREATE string block
' NB can be other objects mixed with THIS objects properties!
SUB UpdateObjGrid(Start AS integer, blkString AS string)
DEFINT i, cnt, lev
DEFSTR tstr, token

    lev=1                                                           ' Want all props THIS lev (object)
    cnt=TALLY(blkString,"~")
    FOR i=Start TO cnt
        tstr=FIELD$(blkString,"~",i)                                ' For each line of this new object
        token=UCASE$(FIELD$(tstr," ",1))
        SELECT CASE token
            CASE "CREATE": INC lev                                  ' A different object?
            CASE "END": DEC lev: IF lev=0 THEN EXIT FOR             ' End of the properties this object
            CASE ELSE
                IF (lev=1) AND (INSTR(tstr,"=")>0) THEN             ' "Property = value"?
                    UpdateNewProperty(tstr)
                END  IF
        END SELECT
    NEXT i
    UpdatePropGrid(CurrentObject)
END SUB

FUNCTION ExtractObjectID(objType AS string) AS integer
DEFINT i
DEFSTR tstr

    IF LEFT$(objType,5)="QFORM" THEN                                ' Includes QformEx
        Result=0                                                    ' ObjectID=0
    ELSE
        FOR i=1 TO MAX_TOOLBTNS
            tstr=UCASE$(FIELD$(ToolbarHint,"~",i))
            IF objType=tstr THEN EXIT FOR
        NEXT i
        Result=i
    END IF
END FUNCTION

' Update defaults for this form, then redraw it
SUB RepaintNewForm(frmString AS string)
DEFINT i, lev
DEFINT left, top, width, height, defWidth, defHeight
DEFSTR tstr, token, cap

    left=0: top=0: width=0: height=0                                ' Preset defaults
    defWidth=Forms(CurrentForm).Width: defHeight=Forms(CurrentForm).Height
    cap="Form"+STR$(FrmIndex-1)                                     ' A default caption
    lev=1
    FOR i=2 TO TALLY(frmString,"~")                                 ' Start after the create form line
        tstr=LTRIM$(FIELD$(frmString,"~",i))
        IF UCASE$(LEFT$(tstr,7))="CREATE " THEN
            INC lev
        ELSEIF UCASE$(LEFT$(tstr,7))="END CRE" THEN
            DEC lev
            IF lev=0 THEN EXIT FOR                                  ' End of this Form
        ELSEIF (lev=1) AND (INSTR(tstr,"=")>0) THEN
            token=UCASE$(RTRIM$(FIELD$(tstr,"=",1)))
            SELECT CASE token
                CASE "LEFT": left=VAL(LTRIM$(FIELD$(tstr,"=",2)))
                CASE "TOP": top=VAL(LTRIM$(FIELD$(tstr,"=",2)))
                CASE "WIDTH": width=VAL(LTRIM$(FIELD$(tstr,"=",2)))
                CASE "HEIGHT": height=VAL(LTRIM$(FIELD$(tstr,"=",2)))
                CASE "CAPTION": cap=LTRIM$(FIELD$(tstr,"=",2))
            END SELECT
        END IF
    NEXT i
' FIXME: Check for some sanity values, and fudge if necessary - should log this!
    IF left<0 THEN left=0: IF top<0 THEN top=0
    IF width<=0 THEN width=defWidth: IF height<=0 THEN height=defHeight

    SetWindowPos(Forms(CurrentForm).Handle, 0, left, top, width, height, SW_FLAGS)
    SetWindowText(Forms(CurrentForm).Handle,cap)
END SUB

' Get SCREEN properties of object into a rect, then refresh it visually (NOT forms)
' Added MAJOR fudge v1.45 for QOvalBtn
SUB RepaintNewObject(Start AS integer, frmString AS string)
DEFINT i, lev, OvalIdx=0
DEFINT width, height
DEFLNG hWnd
DEFSTR tstr, token, cap
DIM tRect AS QRECT
DIM tPoint AS PointAPI

    tstr=ObjectList(CurrentObject)                                  ' Default properties this new object
    token=FIELD$(tstr,"~",1)                                        ' Object header info
    hWnd=VAL(FIELD$(token,"|",3))                                   ' Get its handle

    IF VAL(FIELD$(token,"|",1))=Q_OVALBTN THEN                      ' QOvalBtn?
        OvalIdx=VAL(FIELD$(token,"|",7))                            ' NEED this extra info
    END IF
    FOR i=2 TO TALLY(tstr,"~")                                      ' Default width & height of this object
        token=UCASE$(FIELD$(tstr,"~",i))
        SELECT CASE FIELD$(token,"|",1)
            CASE "WIDTH": Width=VAL(FIELD$(token,"|",4))            ' Default if error reading code parms
            CASE "HEIGHT": Height=VAL(FIELD$(token,"|",4))          ' ditto
        END SELECT
    NEXT i
    tPoint.x=0: tPoint.y=0: lev=1: cap=""                           ' Start for this object

' Update with object data
    FOR i=Start TO TALLY(frmString,"~")
        tstr=LTRIM$(FIELD$(frmString,"~",i))
        IF UCASE$(LEFT$(tstr,7))="CREATE " THEN
            INC lev
        ELSEIF UCASE$(LEFT$(tstr,7))="END CRE" THEN
            DEC lev
            IF lev=0 THEN EXIT FOR
        ELSEIF (lev=1) AND (INSTR(tstr,"=")>0) THEN
            token=UCASE$(RTRIM$(FIELD$(tstr,"=",1)))
            SELECT CASE token
                CASE "LEFT": tPoint.x=VAL(LTRIM$(FIELD$(tstr,"=",2)))
                CASE "TOP": tPoint.y=VAL(LTRIM$(FIELD$(tstr,"=",2)))
                CASE "WIDTH": width=VAL(LTRIM$(FIELD$(tstr,"=",2)))
                CASE "HEIGHT": height=VAL(LTRIM$(FIELD$(tstr,"=",2)))
                CASE "CAPTION": cap=LTRIM$(FIELD$(tstr,"=",2))
            END SELECT
        END IF
    NEXT i

    ' Just in case something goes wrong
    IF tPoint.x<0 THEN tPoint.x=0: IF tPoint.y<0 THEN tPoint.y=0
' MS Themes allow different Active TitleBar heights which alters vertical positioning (inexplicably!)
' of objects.  Check if this object's parent is a form and adjust vertical position accordingly.
    IF GetParent(hWnd)=Forms(CurrentForm).Handle THEN               ' Object's parent a form?
        tPoint.y = tPoint.y+ThemeOffsetY                            ' Possible adjustment
    END IF

    IF OvalIdx>0 THEN                                               ' Special case QOvalBtn
        OvalRepaint(OvalIdx,tPoint.x,tPoint.y,width,height,cap)
    ELSE
        ClientToScreen(hWnd,tPoint)                                 ' Get object in screen co-ords..
        tRect.Left=tPoint.x                                         ' ..and fill tRect
        tRect.Top=tPoint.y
        tRect.Right=tRect.Left+width
        tRect.Bottom=tRect.Top+height
        RefreshObject(hWnd,tRect)                                   ' Repaint the object
        IF cap<>"" THEN
            SetWindowText(hWnd,cap)                                 ' Update it's caption
        END IF
    END IF
END SUB

' Need to set the cursor in a "free" area of the parent object
' NB there may be other child objects in this parent!
FUNCTION SetCursorInParent(hWnd AS long, objType AS string) AS integer
DEFINT px, py, y
DIM pRect AS QRECT

    Result=True
    GetWindowRect(hWnd,pRect)                                       ' Current object rectangle
    SELECT CASE objType                                             '
        CASE "QFORM"                                                '|
            px=pRect.Left+4: py=pRect.Top+23                        '|
        CASE "QSCROLLBOX"                                           '| Set "Client area" start posn..
            px=pRect.Left+2: py=pRect.Top+2                         '| ..of this parent object
        CASE "QPANEL","QGROUPBOX","QTABCONTROL"                     '|
            px=pRect.Left: py=pRect.Top                             '|
        CASE ELSE                                                   ' Non-parent object?
            px=pRect.Left: py=pRect.Top
            SetCursorPos(px,py)                                     ' Left-Top of this object
            EXIT FUNCTION
    END SELECT
    FOR px=px TO pRect.Right STEP 5                                 ' Scan across the parent window..
        FOR y=py TO pRect.Bottom STEP 5                             ' ..for "free" area
            IF WindowFromPoint(px,y)=hWnd THEN
                SetCursorPos(px,y)                                  ' Inside this object rectangle
                EXIT FUNCTION
            END IF
        Next y
    NEXT px
    Result=False
END FUNCTION

' Insert object onto the current parent. This fn is recursive
' as it may be a single object or a parent object with siblings
FUNCTION InsertObjBlock(Start AS integer, frmString AS string) AS integer
DEFSTR tstr, objType
DEFLNG hWnd
DEFINT i, cnt

    tstr=FIELD$(frmString,"~",Start)
    objType=UCASE$(FIELD$(tstr," ",4))                              ' Object type to add
    NewObjectID=ExtractObjectID(objType)                            ' Set the global ID B4 adding it
    IF objType="QFORM" THEN
        AddNewForm                                                  ' Add a "default" QForm
        RepaintNewForm(frmString)                                   ' Re-size & re-position it
    ELSE
        AddNewObject                                                ' Add a "default" object to the parent
        RepaintNewObject(Start+1,frmString)                         ' Re-size & re-position it
    END IF
    scan.Enabled=False                                              ' MUST do this after adding a component

    tstr=FIELD$(tstr," ",2)                                         ' Object name
    UpdateObjList(tstr)                                             ' Update ObjectList string
    UpdateObjCombo(tstr+" ["+objType+"]")                           ' Replace default with actual name
    UpdateObjGrid(Start+1,frmString)                                      ' Update PropertyGrid info

    tstr=ObjectList(ObjectIndex-1)                                  ' Get new object properties
    hWnd=VAL(FIELD$(tstr,"|",3))                                    ' Get new object handle

    cnt=TALLY(frmString,"~")                                        ' Parse for any child objects
    FOR i=Start+1 TO cnt
        tstr=LTRIM$(FIELD$(frmString,"~",i))
        SELECT CASE UCASE$(LEFT$(tstr,6))
            CASE "END CR"                                           ' Test for exit condition FIRST!
                Result=i                                            ' MUST return THIS field number!
                EXIT FUNCTION                                       ' Oddity: Cannot do <EXIT FOR> here!
            CASE "CREATE"
                IF SetCursorInParent(hWnd,objType) THEN             ' Set cursor inside the current object
                    i=InsertObjBlock(i,frmString)                   ' RECURSIVELY call THIS function
                    IF i = -1 THEN                                  ' Failed?
                        Result = -1                                 ' Abort the chain
                        EXIT FUNCTION
                    END IF
                ELSE
                    tstr="Internal eror!"+CRLF
                    tstr=tstr+"SetCursorInParent() failed for parent of "+objType
                    MessageBox(tstr,"Unknown parse error",&h30)
                    Result = -1                                     ' Failed? Abort the chain
                    EXIT FUNCTION
                END IF
        END SELECT
    NEXT i
END FUNCTION

' frmString is the CREATE ... END CREATE of a FORM with included components
' Oddity: Need this "stub" function to call the recursive function InsertObjBlock()
' Stack must get corrupted and returning FROM THIS stub "tidies" it all up(????)
' InsertObjBlock() is a function, but not interested in the Result returned!
SUB InsertThisForm(frmString AS string)
    InsertObjBlock(1,frmString)                                     ' Start from first field - the form
END SUB

' Get all event sub names from current ObjectList() into FwdDeclarations string
SUB BuildFwdDeclarations
DEFINT i, j
DEFSTR tstr, tok1, tok2

    FwdDeclares=" "
    FOR i=0 TO MAX_OBJECTS                                          ' Add forward declarations..
        tstr=ObjectList(i)                                          ' ..for any object events found
        IF tstr="" THEN EXIT FOR                                    ' End of object list so exit
        FOR j=1 TO TALLY(tstr,"~")
            tok1=FIELD$(tstr,"~",j)                                 ' Each property/event line
            tok2=UCASE$(FIELD$(tok1,"|",1))
            IF (LEFT$(tok2,2)="ON") OR (tok2="WNDPROC") THEN        ' An event?
                tok2=FIELD$(tok1,"|",2)                             ' Get sub name if exist
                IF (tok2<>"") AND (INSTR(FwdDeclares,tok2)=0) THEN  ' Does NOT exist already!
                    tok2=tok2+FIELD$(tok1,"|",3)                    ' Get parameters
                    FwdDeclares=FwdDeclares+"SUB "+tok2+"~"         ' Save this for adding a template sub
                END IF
            END IF
        NEXT j
    NEXT i
    FwdDeclares=LTRIM$(FwdDeclares)
END SUB

'FIXME: Code here ok but incomplete (non-default properties not updated)
SUB ParseNonVisible
DEFINT i, j, cnt, found
DEFSTR tstr, objType, resString=" "

    cnt=TALLY(KnownsList,"@")
    FOR i=1 TO cnt
        found=False
        tstr=TRIM$(FIELD$(KnownsList,"@",i))
        tstr=FIELD$(tstr,"~",1)
        objType=UCASE$(TRIM$(FIELD$(tstr," ",4)))
        FOR j=1 TO MAX_TOOLBTNS
            IF objType=UCASE$(FIELD$(ToolBarHint,"~",j)) THEN
                IF j >= NON_VISIBLE THEN
                    NewObjectID=j
                    AddNewObject
'FIXME: Need to update properties of each new object here!!!!!!
                    found=True
                END IF
                EXIT FOR
            END IF
        NEXT j
        IF found=False THEN
            tstr=TRIM$(FIELD$(KnownsList,"@",i))
            resString=resString+tstr+"@"
        END IF
    NEXT i
    KnownsList=LTRIM$(resString)                                    ' Anything not accounted for
END SUB

SUB ShowProject
DEFINT i, cnt
DEFSTR tstr, exForms

    DelayTimer.Enabled=False                                        ' Start-up delay timer disabled
    scan.Enabled=False                                              ' Just in case!
    ExistDeclares=" ": exForms=" "

    cnt=TALLY(FormsList,"@")
    IF cnt>MAX_FORMS THEN
        tstr="The maximum number of forms has been exceeded!"+CRLF
        tstr=tstr+"Only the FIRST "+STR$(MAX_FORMS)+" forms can be edited!"
        MessageBox(tstr,"RapidFRM "+VER,&h30)
    END IF
    IF cnt=0 THEN                                                   ' Didn't find any forms?
        tstr="NO forms found in input "
        IF ProjectMode=INPUT_FILE THEN
            tstr=tstr+"file!"+CRLF+CRLF
        ELSE
            tstr=tstr+"clipboard data!"+CRLF+CRLF
        END IF
        tstr=tstr+"Generating one 'Default' form."
        MessageBox(tstr,"RapidFRM "+VER,&h30)
        AddNewForm                                                  ' Add a "default" QForm
    ELSE
        i=1
        DO
            tstr=LTRIM$(FIELD$(FormsList,"@",i))
            IF i>MAX_FORMS THEN
                exForms=exForms+tstr+"@"                            ' List of excess forms
            ELSE
                InsertThisForm(tstr+"~")
            END IF
            INC i
        LOOP UNTIL i>cnt
        exForms=LTRIM$(exForms)
        UnKnownList=exForms+LTRIM$(UnKnownList)                     ' Anything > MAX_FORMS -> UnKnownList
    END IF
' FIXME: currently works but properties not updated!
'    ParseNonVisible                                                 ' Add final objects in the KnownsList

    BuildFwdDeclarations                                            ' From the current Design Screen objects
    FOR i=1 TO TALLY(FwdDeclares,"~")
        tstr=FIELD$(FwdDeclares,"~",i)
        IF INSTR(tstr,"(")>0 THEN tstr=RTRIM$(FIELD$(tstr,"(",1))   ' Remove any parameters
        ExistDeclares=ExistDeclares+tstr+"~"                        ' Update global ExistDeclares
    NEXT i
    ExistDeclares=LTRIM$(ExistDeclares)
    FwdDeclares=" "                                                 ' Clear ready to add/update

    scan.Enabled=True                                               ' Ready to start editing
END SUB

' No TABs, no comments, no quote chars and ensure single spaced word separaters
FUNCTION ReformatLine(inpStr AS string) AS string
DEFSTR tstr, tok1, tok2, resString

    tstr=REPLACESUBSTR$(inpStr,CHR$(9)," ")                         ' Replace any TABs
    IF (UCASE$(LEFT$(tstr,4))="REM ") OR (LEFT$(tstr,1)="'") OR _
                                        (LTRIM$(tstr)="") THEN      ' Entire line a comment or blank?
        Result=""
        EXIT FUNCTION
    END IF
    IF INSTR(tstr,"'")>0 THEN                                       ' Remove any comments in line
        tstr=RTRIM$(FIELD$(tstr,"'",1))                             ' FIXME: Single ' inside double " string?
    END IF
    tstr=REPLACESUBSTR$(tstr,QU," ")                                ' Remove "quote chars" around strings
    tstr=LTRIM$(RTRIM$(tstr))                                       ' Trim the line

    resString=" "
    IF INSTR(tstr,"=")>0 THEN
        tok1=RTRIM$(FIELD$(tstr,"=",1))+" = "
        tok2=LTRIM$(FIELD$(tstr,"=",2))
        resString=tok1+tok2
    ELSE
        WHILE tstr<>""
            tok1=FIELD$(tstr," ",1)
            resString=resString+tok1+" "
            tstr=LTRIM$(RIGHT$(tstr,LEN(tstr)-LEN(tok1)))
        WEND
    END IF
    Result=LTRIM$(RTRIM$(resString))
END FUNCTION

' Extract menus and "unknowns" from form strings
FUNCTION ExtractOther(StartIdx AS integer, inpString AS string) AS integer
DEFSTR tstr, token, ParentName, objString
DEFINT i, cnt, lev

    ParentName=FIELD$(inpString," ",2)                              ' The current form's name
    objString=FIELD$(inpString,"~",StartIdx)+"~"                    ' The "create" line of this object
    IF UCASE$(FIELD$(objString," ",4))<>"QPOPUPMENU~" THEN
        tstr="Parent = "+ParentName+"~"                             ' MainMenus & unknowns only
        objString=objString+tstr
    END IF

    lev=1: cnt=TALLY(inpString,"~")+1
    FOR i=StartIdx+1 TO cnt
        tstr=FIELD$(inpString,"~",i)
        token=UCASE$(FIELD$(tstr," ",1))
        IF token="CREATE" THEN
            INC lev
        ELSEIF token="END" THEN
            DEC lev
        END IF
        objString=objString+tstr+"~"
        IF lev=0 THEN EXIT FOR
    NEXT i

    objString=LEFT$(objString,LEN(objString)-1)                     ' Remove trailing "~"
    tstr=FIELD$(inpString,"~",StartIdx)                             ' First line "create"
    token=UCASE$(FIELD$(tstr," ",4))                                ' This object type
    SELECT CASE token                                               ' Add to the appropriate list
        CASE "QMAINMENU": MainMenuList=MainMenuList+objString+"@"
        CASE "QPOPUPMENU": PopMenuList=PopMenuList+objString+"@"
        CASE ELSE
            UnknownList=UnknownList+objString+"@"
    END SELECT
    Result=i                                                        ' Line index of the "end create"
END FUNCTION

' Parse form string storing in formslist, removing menus & unknowns
SUB ParseForm(inpString AS string)
DEFINT i, cnt, objType
DEFSTR tstr, token, frmString

    frmString=FIELD$(inpString,"~",1)+"~"
    cnt=TALLY(inpString,"~")
    FOR i=2 TO cnt
        tstr=FIELD$(inpString,"~",i)
        token=UCASE$(FIELD$(tstr," ",1))
        IF token="CREATE" THEN
            token=UCASE$(FIELD$(tstr," ",4))
            objType=ExtractObjectID(token)                          ' Not forms or menus or unknowns!
            IF (objType > 0) AND (objType < NON_VISIBLE) THEN
                frmString=frmString+tstr+"~"
            ELSE
                i=ExtractOther(i,inpString)                         ' Collect these incrementing our counter
            END IF
        ELSE
            frmString=frmString+tstr+"~"
        END IF
    NEXT i

    frmString=LEFT$(frmString,LEN(frmString)-1)                     ' Remove trailing "~"
    FormsList=FormsList+frmString+"@"                               ' Add to global list
END SUB

' Check an "external" component block (not inside a form) for known parent form
' If it belongs to a "DIM"ed form, then put it with "unknowns" list to ignore
FUNCTION IsParentKnown(inpString AS string) AS integer
DEFINT i, cnt
DEFSTR tstr, FormName=""

    Result=False
    cnt=TALLY(inpString,"~")+1
    FOR i=2 TO cnt
        tstr=FIELD$(inpString,"~",i)
        IF UCASE$(LTRIM$(RTRIM$(FIELD$(tstr,"=",1))))="PARENT" THEN
            FormName=UCASE$(LTRIM$(RTRIM$(FIELD$(tstr,"=",2))))     ' Get this components parent name
            EXIT FOR
        END IF
    NEXT i
    IF FormName<>"" THEN                                            ' Check our form names
        cnt=TALLY(FormsList,"@")
        FOR i=1 TO cnt                                              ' Loop each of "known" forms
            tstr=LTRIM$(FIELD$(FormsList,"@",i))
            tstr=UCASE$(LTRIM$(RTRIM$(FIELD$(tstr," ",2))))         ' The form name
            IF tstr=FormName THEN
                Result=True                                         ' One of our "known" forms
                EXIT FOR
            END IF
        NEXT i
    END IF
END FUNCTION

' Add "known" list object to form list
SUB AddToFormList(frmName AS string, objString AS string)
DEFINT i, j, count, found=False
DEFSTR tstr, frmString, resString=" "

    count=TALLY(FormsList,"@")
    FOR i=1 TO count                                                ' For each form in FormList
        frmString=TRIM$(FIELD$(FormsList,"@",i))
        tstr=UCASE$(TRIM$(FIELD$(frmString," ",2)))                 ' This form name
        IF tstr=frmName THEN                                        ' Need to add to this form?
            tstr=TRIM$(FIELD$(frmString,"~",1))                     ' First "CREATE..." field
            resString=resString+tstr+"~"
            FOR j=2 TO TALLY(frmString,"~")+1                       ' Get each field
                tstr=TRIM$(FIELD$(frmString,"~",j))
                SELECT CASE UCASE$(LEFT$(tstr,6))
                    CASE "CREATE"                                   ' Child objects?
                        IF found=False THEN                         ' First child object found?
                            resString=resString+objString+"~"       ' Insert NEW object
                            found=True
                        END IF
                        resString=resString+tstr+"~"                ' Continue with child object
                    CASE "END CR"                                   ' End of Form code?
                        IF found=False THEN                         ' NO child objects found?
                            resString=resString+objString+"~"       ' Insert new object
                            found=True
                        END IF
                        resString=resString+tstr+"~"                ' Add the terminater
                    CASE ELSE
                        resString=resString+tstr+"~"                ' Keep filling out the form string
                END SELECT
            NEXT j
        ELSE                                                        ' NOT parent form we are looking for
            resString=resString+frmString+"@"                       ' Add to resString
        END IF
        resString=LEFT$(resString,LEN(resString)-1)                 ' Remove trailing "~"
        resString=resString+"@"
    NEXT i
    FormsList=LTRIM$(resString)
END SUB

' Move any objects from "known" list with Form parent to relevant formlist
SUB ParseTheKnowns
DEFINT i, j, count
DEFSTR tstr, objString, resString=" "

    count=TALLY(KnownsList,"@")
    FOR i=1 TO count
        objString=TRIM$(FIELD$(KnownsList,"@",i))
        IF IsParentKnown(objString) THEN                            ' Has it a parent form?
            FOR j=2 TO TALLY(objString,"~")+1
                tstr=UCASE$(TRIM$(FIELD$(objString,"~",j)))
                IF RTRIM$(FIELD$(tstr,"=",1))="PARENT" THEN
                    tstr=UCASE$(LTRIM$(FIELD$(tstr,"=",2)))         ' Parent Form name
                    objString=objString-(TRIM$(FIELD$(objString,"~",j))+"~")        ' Remove "parent=..." field
                    AddToFormList(tstr,objString)                   ' Move it into the FormsList
                    EXIT FOR
                END IF
            NEXT j
        ELSE
            resString=resString+objString+"@"                       ' Parent not specified - keep in knownsList
        END IF
    NEXT i
    KnownsList=LTRIM$(resString)
END SUB

SUB CreateDebugFile
DEFINT i
DEFSTR tstr
DIM sList AS QStringList

    sList.Clear
    FOR i=1 TO TALLY(FormsList,"@")
        tstr=FIELD$(FormsList,"@",i)
        sList.AddItems(tstr)
    NEXT i
    FOR i=1 TO TALLY(MainMenuList,"@")
        tstr=FIELD$(MainMenuList,"@",i)
        sList.AddItems(tstr)
    NEXT i
    FOR i=1 TO TALLY(PopMenuList,"@")
        tstr=FIELD$(PopMenuList,"@",i)
        sList.AddItems(tstr)
    NEXT i
    FOR i=1 TO TALLY(KnownsList,"@")
        tstr=FIELD$(KnownsList,"@",i)
        sList.AddItems(tstr)
    NEXT i
    FOR i=1 TO TALLY(UnknownList,"@")
        tstr=FIELD$(UnknownList,"@",i)
        sList.AddItems(tstr)
    NEXT i
    sList.SaveToFile("DbgInputStream.txt")
    sList.Clear
    ShowMessage "DBG: DbgInputStream.txt created!"
END SUB

' Parse and separate blocks of "create .. end create" in the first instance.
' Can be of type Form, Mainmenu, Popupmenu, standard component (with/out parent!) or "unknowns"
' Fill global string lists, ready for displaying forms & initialising menu editor etc
SUB ParseInputStream
DEFINT i, lev, cnt, objType
DEFSTR tstr, blkString
DEFSTR CreateBlocks(1 TO MAX_OBJECTS)

    FOR cnt=1 TO MAX_OBJECTS
        CreateBlocks(cnt)=""
    NEXT cnt
    blkString=" ": lev=0: cnt=0
    ProjectStream.Position=0
    FOR i=1 TO ProjectStream.LineCount
        tstr=ProjectStream.ReadLine
        tstr=ReformatLine(tstr)                                     ' "Pretty" the line for parsing
        IF tstr="" THEN                                             ' Do nothing
        ELSEIF UCASE$(FIELD$(tstr," ",1))="CREATE" THEN
            IF (cnt=0) AND (lev=0) THEN StartOfBlocks=i             ' Global Start Marker - line num of project code
            INC lev
            blkString=blkString+tstr+"~"                            ' Add the line to the block string
        ELSEIF lev>0 THEN                                           ' Inside CREATE .. END CREATE
            blkString=blkString+tstr+"~"
            IF (UCASE$(FIELD$(tstr," ",1))="END") THEN              ' End of current object
                DEC lev
                IF lev=0 THEN                                       ' Was it the end of a block?
                    EndOfBlocks=i                                   ' Global End Marker updated until last block parsed
                    INC cnt
                    blkString=LEFT$(blkString,LEN(blkString)-1)+"@"     ' Replace trailing '~' with '@'
                    CreateBlocks(cnt)=LTRIM$(blkString)             ' Save until ready to display
                    blkString=" "                                   ' Start again
                END IF
            END IF
        END IF
    NEXT i
' Parse all Form blocks first!
    FOR i=1 TO cnt
        blkString=CreateBlocks(i)                                   ' Each create .. end create block
        tstr=FIELD$(blkString,"~",1)
        tstr=UCASE$(FIELD$(tstr," ",4))
        IF LEFT$(tstr,5)="QFORM" THEN                               ' Includes QFormEX!!
            tstr=LEFT$(blkString,LEN(blkString)-1)+"~"
            ParseForm(tstr)                                         ' Updates FormList global string
        END IF
    Next i

' Update remainder of global lists
    FOR i=1 TO cnt
        blkString=CreateBlocks(i)                                   ' Each create .. end create block
        tstr=FIELD$(blkString,"~",1)
        tstr=UCASE$(FIELD$(tstr," ",4))
        IF LEFT$(tstr,5)="QFORM" THEN                               ' Do nothing!
        ELSEIF tstr="QMAINMENU" THEN
            MainMenuList=LTRIM$(MainMenuList)+blkString             ' Main meus
        ELSEIF tstr="QPOPUPMENU" THEN
            PopMenuList=LTRIM$(PopMenuList)+blkString               ' Popup menus
        ELSE
            objType=ExtractObjectID(tstr)
            SELECT CASE objType
                CASE 1 TO (NON_VISIBLE-1)                           ' Visible objects (but not forms or menus)
                    IF IsParentKnown(blkString) THEN
                        KnownsList=KnownsList+blkString             ' Temporarily put into "knowns" list
                    ELSE
                        UnknownList=UnknownList+blkString           ' Put into the "unknowns" list
                    END IF
                CASE NON_VISIBLE TO MAX_TOOLBTNS                    ' Non visible stuff
                    KnownsList=KnownsList+blkString                 ' Put into "knowns" list
                CASE ELSE
                    UnknownList=UnknownList+blkString               ' Put into the "unknowns" list
            END SELECT
        END IF
    NEXT i
    ParseTheKnowns                                                  ' Move objects with "known" parents into formslist

$IFDEF DBG_INPUT_STREAM
    CreateDebugFile
$ENDIF
END SUB

'--------------------------------Load Existing Project  --------------------------------
' Entry point LoadProject()
'---------------------------------------------------------------------------------------
SUB LoadClipboardText
DEFSTR tstr

    ClipBoard.Open
    tstr=ClipBoard.Text                                             ' Read contents
    ClipBoard.Clear                                                 ' Clear the clipboard!!
    ClipBoard.Close                                                 ' Finished with it for now
    DoEvents
    IF tstr="" THEN
        MessageBox("The clipboard is empty - terminating!","RapidFRM "+VER,&h30)
        ErrorCondition=True
        ProgramEnd
    ELSE
        ProjectStream.Close                                         ' Ensure initialised
        ProjectStream.WriteStr(tstr,LEN(tstr))
        ProjectStream.Position=0
    END IF
END SUB

SUB LoadProjectFile
DIM file AS QFILESTREAM
DEFSTR tstr

    IF FileExists(ProjectFile)=0 THEN
        MessageBox("File "+ProjectFile+" does not exist - terminating!","RapidFRM "+VER,&h30)
        ErrorCondition=True
        ProgramEnd
    END IF
    tstr=UCASE$(RIGHT$(ProjectFile,4))
    SELECT CASE tstr
        CASE ".BAS",".RQB",".INC",".FRM"                            ' These file types accepted
    CASE ELSE
        tstr="Usage:"+CRLF+"RapidFRM.exe [<source file>]"+CRLF
        tstr=tstr+"Optional source file to be of type BAS, RQB, INC or FRM"
        MessageBox(tstr,"RapidFRM "+VER,&h40)
        ErrorCondition=True
        ProgramEnd
    END SELECT
    ProjectStream.Close                                             ' Ensure initialised
    file.Open(ProjectFile,fmOpenRead)
    ProjectStream.CopyFrom(file,0)                                  ' Copy all into memstream
    file.Close
    DoEvents
    ProjectStream.Position=0
END SUB

SUB LoadProject
    IF ProjectMode=INPUT_CLIPBRD THEN                               ' Which input?
        LoadClipboardText
    ELSE
        LoadProjectFile
    END IF
    ParseInputStream
END SUB

'---------------------------- Exit program -----------------------------------------
' NB STAND_ALONE mode takes care of it's own Forward declarations and sub templates
'-----------------------------------------------------------------------------------
' Not sure if this necessary, but...
SUB CleanUp
DEFINT i
DEFLNG hWnd
DEFSTR tstr

    FOR i=1 TO ObjectIndex-1                                        ' Do not think RapidQ "knows" about...
        tstr=ObjectList(i)                                          ' ...the system memory used by component...
        IF tstr<>"" THEN                                            ' ...objects created at the "local" level
            tstr=FIELD$(tstr,"~",1)
            IF FIELD$(tstr,"|",1)<>"0" THEN                         ' NOT global forms, RapidQ will fix these
                hWnd=VAL(FIELD$(tstr,"|",3))
                DestroyWindow(hWnd)                                 ' Return memory to system
            END IF
        END IF
    NEXT i
END SUB

' Default (button) for yes/no: 1=yes mask=0: 2=no mask=&h100
SUB ProgramEnd
DEFSTR tstr
DEFLNG mask=&h100                                                   ' Default to button 2 - NO

    IF ErrorCondition=False THEN
        IF ObjectIndex>ComponentCount THEN
            tstr="QUIT without saving changes?"
            IF MessageBox(tstr,"Project updated...",mask+&h24)=mrNo THEN EXIT SUB
        END IF
    END IF
    UnhookWindowsHookEx(hHook)                                      ' Remove our windows mouse hook
    CleanUp
    Application.Terminate
END SUB


SUB AskExit
    IF MessageDlg("Save work before exiting?", mtWarning, mbYes OR mbNo, 0) = mrNo THEN
        ExitNoUpdate
    ELSE
        SaveAndExit
    END IF
END SUB


SUB ExitNoUpdate
DEFINT i
DEFSTR tstr=""

    SELECT CASE ProjectMode
        CASE STAND_ALONE, INPUT_FILE
            ProgramEnd                                              ' Exit with confirmation
        CASE ELSE                                                   ' Clipboard must be re-instated
            ProjectStream.Position=0                                ' The original project stream
            FOR i=1 TO ProjectStream.LineCount
                tstr=tstr+ProjectStream.ReadLine+CRLF               ' Build original code
            NEXT i
            ClipBoard.Open: ClipBoard.Clear
            ClipBoard.Text=tstr                                     ' Add to the clipboard
            ClipBoard.Close
            DoEvents
            ErrorCondition=False: ObjectIndex=ComponentCount        ' 2 Conditions set to exit immediately
            ProgramEnd
    END SELECT
END SUB

'---------------------------- Saving a Project --------------------------------------
' Entry point SaveAndExit()
'------------------------------------------------------------------------------------
' Format Create..End Create blocks that are not QFORMs
FUNCTION FmtOtherBlocks(inpStr AS string) AS string
DEFINT i, cnt
DEFSTR tstr, token, spacer="    "
DEFSTR blkString

    cnt=TALLY(inpStr,"~")-1                                         ' (Lines-1) of code to add
    blkString=FIELD$(inpStr,"~",1)+CRLF                             ' Start with the create line
    FOR i=2 TO cnt                                                  ' Do the lines between create..end create
        tstr=FIELD$(inpStr,"~",i)
        token=FIELD$(tstr," ",1)
        SELECT CASE UCASE$(token)                                   ' Check for string data
            CASE "CAPTION", "TEXT", "HINT"
                token=LTRIM$(FIELD$(tstr,"=",2))                    ' Get the string value (no quotes!)
                token=QU+token+QU                                   ' Add the quotes
                tstr=RTRIM$(FIELD$(tstr,"=",1))                     ' Remove old
                tstr=tstr+" = "+token                               ' Insert new
        END SELECT
        tstr=spacer+tstr+CRLF                                       ' Add with indentation
        blkString=blkString+tstr
    NEXT i
    tstr=FIELD$(inpStr,"~",i)                                       ' Do last line
    blkString=blkString+tstr+CRLF
    Result=blkString
END FUNCTION

SUB SaveAndExit
DIM newStream AS QMEMORYSTREAM
DIM NewFile AS QFILESTREAM
DEFINT i, lev
DEFSTR tstr, token

    IF DesignTab.TabIndex=3 THEN                                    ' FreeBasic?
        FB_SaveAndExit                                              ' Call separate code...
        EXIT SUB
    END IF

' On entry we have the entire project file/clipboard data in the ProjectStream
    IF ProjectMode=INPUT_FILE THEN
        tstr=LEFT$(ProjectFile, RINSTR(ProjectFile,"."))+"bup"      ' Create a backup file name
        IF FileExists(tstr) THEN                                    ' Backup file exists?
            KILL tstr                                               ' Delete it
            DoEvents
        END IF
        RENAME(ProjectFile,tstr)                                    ' Backup original file
        DoEvents
    END IF

    ProjectStream.Position=0                                        ' The original project stream
    newStream.Close                                                 ' Ensure initialised
    FOR i=1 TO StartOfBlocks-1
        tstr=ProjectStream.ReadLine                                 ' Copy lines up to first block declaration
        newStream.WriteLine(tstr)
    NEXT i
    FOR i=i TO EndOfBlocks
        tstr=ProjectStream.ReadLine                                 ' Discard EXISTING block data
    NEXT i

' Add any new/changed forward declarations B4 we add the new code
    IF LTRIM$(FwdDeclares)<>"" THEN
        IF ProjectMode=INPUT_CLIPBRD THEN
            newStream.WriteLine("'FWD_DECL")                        ' Marker for IDE
        ELSE
            newStream.WriteLine("' Forward declarations")
        END IF
        FOR i=1 TO TALLY(FwdDeclares,"~")
            tstr=LTRIM$(RTRIM$(FIELD$(FwdDeclares,"~",i)))
            newStream.WriteLine("DECLARE "+tstr)                    ' Add the new line
        NEXT i
        newStream.WriteLine("")
    END IF
' Add the new form code
    IF ProjectMode=INPUT_CLIPBRD THEN
        newStream.WriteLine("'FRM_CODE")                            ' Marker for IDE
'        newStream.WriteLine("' Code generated by RapidFRM "+VER)    ' Mark code with version info
        newStream.WriteLine("")
    END IF
    FOR i=0 TO CodeText.LineCount-1
        tstr=CodeText.Line(i)
        newStream.WriteLine(tstr)                                   ' Add the new code
    NEXT i
' Add any "Knowns" found (components without parents etc..)
    IF KnownsList<>"" THEN
        FOR i=1 TO TALLY(KnownsList,"@")
            tstr=FIELD$(KnownsList,"@",i)+"~"
            tstr=FmtOtherBlocks(tstr)                               ' Format the code
            newStream.WriteLine(tstr)                               ' Add the new code
        NEXT i
    END IF
' Re-insert any "unknowns" found
    IF UnknownList<>"" THEN
        FOR i=1 TO TALLY(UnknownList,"@")
            tstr=LTRIM$(FIELD$(UnknownList,"@",i))+"~"
            tstr=FmtOtherBlocks(tstr)                               ' Format the code
            newStream.WriteLine(tstr)                               ' Add the new code
        NEXT i
    END IF
' Add remainder of original project
    i=EndOfBlocks+1
    FOR i=i TO ProjectStream.LineCount
        tstr=ProjectStream.ReadLine
        newStream.WriteLine(tstr)
    NEXT i
' Add any new/changed event sub() stubs at end of file code
    IF LTRIM$(FwdDeclares)<>"" THEN                                 ' Anything to add?
        newStream.WriteLine("")
        IF ProjectMode=INPUT_CLIPBRD THEN
            newStream.WriteLine("'FRM_STUBS")                       ' Marker for IDE
        END IF
        FOR i=1 TO TALLY(FwdDeclares,"~")
            tstr=FIELD$(FwdDeclares,"~",i)
            newStream.WriteLine(LTRIM$(tstr))                       ' Add the sub..end sub
            newStream.WriteLine("END SUB")
            newStream.WriteLine("")
        NEXT i
    END IF

    IF ProjectMode=INPUT_FILE THEN
        NewFile.Open(ProjectFile, fmCreate)
        NewFile.CopyFrom(newStream,0)                               ' Copy all code into file
        NewFile.Close                                               ' Done

        tstr="Your project file has been saved."+CRLF+"Your original file was archived"
        MessageBox(tstr,"RapidFRM "+VER,&h40)
    ELSE
        newStream.Position=0: tstr=""
        FOR i=1 TO newStream.LineCount
            tstr=tstr+newStream.ReadLine+CRLF
        NEXT i
        ClipBoard.Open
        ClipBoard.Clear
        ClipBoard.Text=tstr
        ClipBoard.Close
    END IF
    ErrorCondition=False: ObjectIndex=ComponentCount                ' 2 Conditions set to exit immediately
    ProgramEnd
END SUB

'-----------------------------------------------------------------------------------------
'-----------------------------FreeBasic additions-----------------------------------------
' Main entry point for outputting FreeBasic code BuildFBCode()
'-----------------------------------------------------------------------------------------
'-----------------------------------------------------------------------------------------
' Which components currently supported
FUNCTION FB_HasSupport(ObjType AS string) AS integer
    Result=True
    SELECT CASE ObjType
        CASE "QOUTLINE","QOLECONTAINER","QSPLITTER", _
            "QMYSQL","QOLEOBJECT","QREGISTRY", _
            "QDXTimer","QDXSound","QDXScreen","QDXImageList", _
            "QD3DFace","QD3DMeshBuilder","QD3DFrame","QD3DLight", _
            "QD3DTexture","QD3DVector","QD3DWrap"
            Result=False
    END SELECT
END FUNCTION

' This component coded into project?
FUNCTION FB_HasComponent(ObjType AS string) AS integer
DEFINT i

    FOR i=0 TO ObjCount-1
        IF ObjType=RTRIM$(FB_Objects(i).FBObjType) THEN
            Result=True
            EXIT FUNCTION
        END IF
    NEXT i
    Result=False
END FUNCTION

' Match parent to child handles, even though they are only
' REAL at design time, we still match the actual objects ok!
FUNCTION FB_GetParentIdx(ChildIdx AS integer) AS integer
DEFLNG pHandle
DEFINT i

    Result=0                            ' Defaults to main form
    pHandle=FB_Objects(ChildIdx).FBObjParent
    FOR i=0 TO ChildIdx-1
        IF FB_Objects(i).FBObjHandle=pHandle THEN
            Result=i
            EXIT FOR
        END IF
    NEXT i
END FUNCTION

FUNCTION FB_HasObjCaption(OType AS string) AS integer
DEFSTR tstr

    Result=False
    tstr=UCASE$(LTRIM$(RTRIM$(OType)))
    SELECT CASE tstr
        CASE "QFORM","QLABEL","QBUTTON","QCOOLBTN","QOVALBTN", _
                  "QGROUPBOX","QCHECKBOX","QRADIOBUTTON","QPANEL"
            Result=True
    END SELECT
END FUNCTION

FUNCTION FB_ObjectVisible(idx AS integer) AS integer
    Result=False
    IF FB_Objects(idx).FBObjID < NON_VISIBLE THEN
        Result=True
    END IF
END FUNCTION

SUB FB_FillHeader
DEFSTR tstr

    CodeText2.AddStrings "'-------------------------------------------------------------------------------"
    CodeText2.AddStrings "' FreeBasic code generated by RapidFRM "+VER
    CodeText2.AddStrings "'-------------------------------------------------------------------------------"
    CodeText2.AddStrings "#INCLUDE "+QU+"windows.bi"+QU
    CodeText2.AddStrings "#INCLUDE "+QU+"win/commctrl.bi"+QU
    CodeText2.AddStrings "InitCommonControls()",""
    IF FB_HasComponent("QRICHEDIT") THEN
        CodeText2.AddStrings "#DEFINE __QRICHEDIT"
        CodeText2.AddStrings "#INCLUDE ONCE "+QU+"win/richedit.bi"+QU
        CodeText2.AddStrings "DyLibLoad("+QU+"Riched32.dll"+QU+")",""
    END IF
    IF FB_HasComponent("QEDIT") THEN
        CodeText2.AddStrings "#DEFINE __QEDIT"
        CodeText2.AddStrings "#INCLUDE ONCE "+QU+"win/richedit.bi"+QU
    END IF
    IF FB_HasComponent("QDIRTREE") THEN
        CodeText2.AddStrings "#DEFINE __QDIRTREE"
        CodeText2.AddStrings "#INCLUDE ONCE "+QU+"win/shellapi.bi"+QU
    END IF
    IF FB_HasComponent("QSOCKET") THEN
        CodeText2.AddStrings "#DEFINE __QSOCKET"
        CodeText2.AddStrings "#INCLUDE ONCE "+QU+"win/winsock.bi"+QU
    END IF
    IF FB_HasComponent("QCOOLBTN") THEN CodeText2.AddStrings "#DEFINE __QCOOLBTN"
    IF FB_HasComponent("QGAUGE") THEN CodeText2.AddStrings "#DEFINE __QGAUGE"
    IF FB_HasComponent("QOVALBTN") THEN CodeText2.AddStrings "#DEFINE __QOVALBTN"
    IF FB_HasComponent("QPANEL") THEN CodeText2.AddStrings "#DEFINE __QPANEL"
    IF FB_HasComponent("QSCROLLBOX") THEN CodeText2.AddStrings "#DEFINE __QSCROLLBOX"
    IF FB_HasComponent("QGROUPBOX") THEN CodeText2.AddStrings "#DEFINE __QGROUPBOX"
    IF FB_HasComponent("QTABCONTROL") THEN CodeText2.AddStrings "#DEFINE __QTABCONTROL"
    IF FB_HasComponent("QSTRINGGRID") THEN CodeText2.AddStrings "#DEFINE __QSTRINGGRID"

    tstr=Application.Path
    IF FileExists(tstr+"\RQInclude.bi") THEN
        tstr=REPLACESUBSTR$(tstr,"\","/")
        CodeText2.AddStrings "#INCLUDE "+QU+tstr+"/RQInclude.bi"+QU         ' RapidFRM path
    ELSE
        CodeText2.AddStrings "#INCLUDE "+QU+"win/RQInclude.bi"+QU           ' Default FreeBASIC path
    END IF
END SUB

SUB FB_AddGlobals
DEFINT i
DEFSTR tstr

    CodeText2.AddStrings "", "'-------------------------------------------------------------------------------"
    CodeText2.AddStrings "' Global data from Form designer sheet"
    CodeText2.AddStrings "'-------------------------------------------------------------------------------"
    FOR i=0 TO ObjCount-1
        tstr="DIM SHARED AS "+RTRIM$(FB_Objects(i).FBObjType)
        CodeText2.AddStrings tstr+" "+RTRIM$(FB_Objects(i).FBObjName)
        tstr=tstr+" PTR "+RTRIM$(FB_Objects(i).FBObjPtrName)
        CodeText2.AddStrings tstr+" = @"+RTRIM$(FB_Objects(i).FBObjName)
    NEXT i
END SUB

' Replace Event parameter 'BYREF Var AS QRECT' with FBs 'lpVar RECT PTR'
FUNCTION FixQRect(Txt AS string) AS string
DEFSTR tstr, tok
DEFINT i
    tstr=""
    FOR i=1 TO TALLY(Txt,",")+1
        tok=FIELD$(Txt,",",i)
        IF INSTR(tok,"QRECT")>0 THEN
            tok=LTRIM$(RTRIM$(FIELD$(tok,"AS",1)))
            tok=" lp"+tok+" AS RECT PTR"
        END IF
        tstr=tstr+tok+","
    NEXT i
    tstr=LEFT$(tstr,LEN(tstr)-1)+")"                ' Replace final comma with close sub bracket
    Result=tstr
END FUNCTION

SUB FB_AddFwdDeclares
DEFINT i, j, idx
DEFSTR tstr, tok1, tok2
DEFSTR MyFuncList=""

    CodeText2.AddStrings "", "'-------------------------------------------------------------------------------"
    CodeText2.AddStrings "' Forward declarations for component events "
    CodeText2.AddStrings "'-------------------------------------------------------------------------------"
    i=0
    DO
        j=FB_Objects(i).FBLineNum+1
        FOR idx=j TO FB_InputList.ItemCount-1
            tstr=LTRIM$(RTRIM$(FB_InputList.Item(idx)))
            IF LEFT$(tstr,1)="#" THEN EXIT FOR
            IF LEFT$(tstr,2)="On" THEN
                tok1=FIELD$(tstr,"|",2)
                tok2=FIELD$(tstr,"|",3)
                IF tok1<>"" THEN
                    IF INSTR(MyFuncList,tok1)=0 THEN
                        MyFuncList=MyFuncList+","+tok1
                        tok1="DECLARE SUB "+tok1
                        IF tok2="" THEN
                            tok1=tok1+"(Sender AS "+RTRIM$(FB_Objects(i).FBObjType)+")"
                        ELSE
                            IF INSTR(tok2,"QRECT")>0 THEN tok2=FixQRect(tok2)       ' Replace RapidQ QRECT with RECT    
                            tok1=tok1+LEFT$(tok2,LEN(tok2)-1)
                            tok1=tok1+", Sender AS "+RTRIM$(FB_Objects(i).FBObjType)+")"
                        END IF
                        CodeText2.AddStrings tok1
                    END IF
                END IF
            END IF
        NEXT idx
        INC i
    LOOP UNTIL i >= ObjCount
END SUB

FUNCTION FB_GetParam(startIdx AS integer, pField AS string) AS integer
DEFSTR tstr
DEFINT i

    Result=0
    FOR i=startIdx TO FB_InputList.ItemCount-1
        tstr=UCASE$(FB_InputList.Item(i))
        IF LEFT$(tstr,1)="#" THEN EXIT FOR
        IF FIELD$(tstr,"|",1)=pField THEN
            Result=i
            EXIT FOR
        END IF
    NEXT i
END FUNCTION

SUB FB_FillConfiguration(idx AS integer)
DEFINT i, newIdx, memIdx, Found=False
DEFSTR newLine, memLine, newTok, fldTok, ObjType, lpObjName

    ObjType=RTRIM$(FB_Objects(idx).FBObjType)               ' Component type to add
    lpObjName=RTRIM$(FB_Objects(idx).FBObjPtrName)          ' and the ptr
    CodeText2.AddStrings ""
    CodeText2.AddStrings SPACE$(4)+lpObjName+"->ObjIndex = "+STR$(idx)
    newIdx=FB_Objects(idx).FBLineNum+1                      ' First property at line 2
    ComponentRef.Position=0: memIdx=0                       ' Reset the memorystream
    DO
        INC memIdx
        IF memIdx>ComponentRef.LineCount THEN
            ShowMessage("INTERNAL ERROR: Component not found!")
            EXIT SUB
        END IF
        memLine=ComponentRef.ReadLine
        IF FIELD$(memLine," ",3)=ObjType THEN Found=True    ' Next line is first property
    LOOP UNTIL Found=True
    DO
        newLine=RTRIM$(FB_InputList.Item(newIdx))           ' Property for NEW component
        IF LEFT$(newLine,1)="#" THEN EXIT SUB               ' New component property?
        IF LEFT$(newLine,1)="*" THEN EXIT DO                ' Property info finished
        memLine=ComponentRef.ReadLine                       ' DEFAULT property this component

        fldTok=FIELD$(newLine,"|",1)
        IF fldTok<>FIELD$(memLine,"|",1) THEN
            ShowMessage("INTERNAL ERROR: Property field mis-align")
            EXIT SUB
        END IF
        newTok=FIELD$(newLine,"|",4)
        IF newTok<>FIELD$(memLine,"|",4) THEN
            SELECT CASE UCASE$(RTRIM$(fldTok))
                CASE "PARENT"
                    GOTO SkipProperty
                CASE "CAPTION","TEXT","HINT","SIMPLETEXT","DIRECTORY","DRIVE","MASK", _
                        "FILENAME","INITIALDIR","FILTER","FONTNAME","NAME","BMP", _
                        "ICON", "SEPARATOR"
                    newTok=REPLACESUBSTR$(newTok,QU,"")     ' Remove IF exist then...
                    newTok=QU+newTok+QU                     ' ...ENSURE they exist
                CASE "LEFT","TOP","WIDTH","HEIGHT"
                    SELECT CASE ObjType                     ' NOT for these objects
                        CASE "QHEADER","QSTATUSBAR"
                            GOTO SkipProperty
                        CASE "QCOMBOBOX"
                            IF UCASE$(RTRIM$(fldTok))="HEIGHT" THEN GOTO SkipProperty
                    END SELECT
                CASE "COLWIDTHS"
                    FOR i=1 TO TALLY(newTok,",")+1
                        CodeText2.AddStrings SPACE$(4)+lpObjName+"->"+FIELD$(newLine,"|",1) _
                                                        +"("+STR$(i-1)+") = "+FIELD$(newTok,",",i)
                    NEXT i
                    GOTO SkipProperty
            END SELECT
            CodeText2.AddStrings SPACE$(4)+lpObjName+"->"+FIELD$(newLine,"|",1)+" = "+newTok
        END IF
SkipProperty:
        INC newIdx
    LOOP UNTIL newIdx >= FB_InputList.ItemCount
    ' Add any object event calls ... OnClick, OnChange etc
    DO
        newLine=LTRIM$(RTRIM$(FB_InputList.Item(newIdx)))
        IF LEFT$(newLine,1)="#" THEN EXIT DO
        IF LEFT$(newLine,2)="On" THEN
            newTok=LTRIM$(RTRIM$(FIELD$(newLine,"|",2)))
            IF newTok<>"" THEN
                CodeText2.AddStrings SPACE$(4)+lpObjName+"->"+FIELD$(newLine,"|",1)+" = CAST(ANY PTR, @"+newTok+")"
            END IF
        END IF
        INC newIdx
    LOOP UNTIL newIdx >= FB_InputList.ItemCount
END SUB

SUB FB_ConfigureObjects
DEFINT i
DEFSTR lpObjName

    CodeText2.AddStrings "", "'-------------------------------------------------------------------------------"
    CodeText2.AddStrings "' Configure components"
    CodeText2.AddStrings "'-------------------------------------------------------------------------------"
    CodeText2.AddStrings "SUB ConfigureComponents"

    FOR i=0 TO ObjCount-1
        FB_FillConfiguration(i)                                     ' For each component
    NEXT i
    CodeText2.AddStrings "","END SUB"
END SUB

SUB FB_AddNonVisible(idx AS integer)
DEFSTR tstr, tok, parnt

    parnt=RTRIM$(FB_Objects(0).FBObjPtrName)+"->Handle"
    tstr=UCASE$(LTRIM$(RTRIM$(FB_Objects(idx).FBObjType)))
    tok=RTRIM$(FB_Objects(idx).FBObjPtrName)
    SELECT CASE tstr
        CASE "QTIMER"
            CodeText2.AddStrings "","    "+tok+"->Parent="+parnt
            CodeText2.AddStrings "    SetTimer("+parnt+", "+tok+"->ObjIndex, "+tok+"->Interval, 0)"
        CASE "QSOCKET"
            CodeText2.AddStrings "","    "+tok+"->Parent="+parnt
'        CASE "QBITMAP"
'        etc etc but not all components
    END SELECT
END SUB

'
' Currently cannot handle arrays of strings in FreeBasic code generation
' So FUDGE it here and break up into single strings and add each individually.
' QHeader has a "section order" problem (insertafter directive) fudged here too!
'
SUB FB_AddSubItems(idx AS integer)
DEFSTR tstr, tok1, tok2, objPtr
DEFINT i, j, cnt

    objPtr=RTRIM$(FB_Objects(idx).FBObjPtrName)
    tstr=RTRIM$(FB_Objects(idx).FBObjType)
    SELECT CASE tstr
        CASE "QTABCONTROL"
            ' My FUDGE! Add each TAB individually
            i=FB_Objects(idx).FBLineNum+1
            FOR i=i TO FB_InputList.ItemCount-1                         ' Scan this objects properties/methods etc
                tok1=FB_InputList.Item(i)
                IF LEFT$(tok1,1)="#" THEN EXIT SUB                      ' End of this object? Exit
                IF UCASE$(LEFT$(tok1,7))="ADDTABS" THEN                 ' Found it!
                    tok2=TRIM$(FIELD$(tok1,"|",4))
                    IF tok2 <> "" THEN
                        cnt=TALLY(tok2,",")+1
                        j=1
                        DO
                            tok1=FIELD$(tok2,",",j)
                            IF INSTR(tok1,QU)>0 THEN
                                tok1=REPLACESUBSTR$(tok1,QU,"")
                            END IF
                            tok1=TRIM$(tok1)
                            tstr=SPACE$(4)+objPtr+"->AddTabs "+QU+tok1+QU
                            CodeText2.AddStrings tstr
                            INC j
                        LOOP UNTIL j>cnt
                        EXIT SUB                                        ' Exit out of here
                    END IF
                END IF
            NEXT i
        CASE "QLISTBOX","QCOMBOBOX","QLISTVIEW"
            i=FB_Objects(idx).FBLineNum+1
            FOR i=i TO FB_InputList.ItemCount-1                         ' Scan this objects properties/methods etc
                tok1=FB_InputList.Item(i)
                IF LEFT$(tok1,1)="#" THEN EXIT SUB
                IF UCASE$(LEFT$(tok1,10))="ADDCOLUMNS" THEN             ' QListView only
                    tok2=TRIM$(FIELD$(tok1,"|",4))
                    IF tok2 <> "" THEN
                        cnt=TALLY(tok2,",")+1
                        j=1
                        DO
                            tok1=FIELD$(tok2,",",j)
                            IF INSTR(tok1,QU)>0 THEN
                                tok1=REPLACESUBSTR$(tok1,QU,"")
                            END IF
                            tok1=TRIM$(tok1)
                            CodeText2.AddStrings SPACE$(4)+objPtr+"->AddColumns("+objPtr+"->Width\"+STR$(cnt)+"-1,"+QU+tok1+QU+")"
                            INC j
                        LOOP UNTIL j>cnt
                    END IF
                END IF
                IF UCASE$(LEFT$(tok1,8))="ADDITEMS" THEN                ' QListView only
                    tok2=TRIM$(FIELD$(tok1,"|",4))
                    IF tok2 <> "" THEN
                        cnt=TALLY(tok2,",")+1
                        j=1
                        DO
                            tok1=FIELD$(tok2,",",j)
                            IF INSTR(tok1,QU)>0 THEN
                                tok1=REPLACESUBSTR$(tok1,QU,"")
                            END IF
                            tok1=TRIM$(tok1)
                            CodeText2.AddStrings SPACE$(4)+objPtr+"->AddItems "+QU+tok1+QU
                            INC j
                        LOOP UNTIL j>cnt
                        EXIT SUB
                    END IF
                END IF
            NEXT i
        CASE "QHEADER"
            i=FB_Objects(idx).FBLineNum+1
            FOR i=i TO FB_InputList.ItemCount-1                         ' Scan this objects properties/methods etc
                tok1=FB_InputList.Item(i)
                IF LEFT$(tok1,1)="#" THEN EXIT SUB
                IF UCASE$(LEFT$(tok1,11))="ADDSECTIONS" THEN
                    tok2=TRIM$(FIELD$(tok1,"|",4))
                    IF tok2 <> "" THEN
                        cnt=TALLY(tok2,",")+1
                        j=1
                        DO
                            tok1=FIELD$(tok2,",",j)
                            IF INSTR(tok1,QU)>0 THEN
                                tok1=REPLACESUBSTR$(tok1,QU,"")
                            END IF
                            tok1=TRIM$(tok1)
                            tstr=SPACE$(4)+objPtr+"->AddSections "+QU+tok1+QU
                            CodeText2.AddStrings tstr
                            INC j
                        LOOP UNTIL j>cnt
                        EXIT SUB
                    END IF
                END IF
            NEXT i
    END SELECT
END SUB

SUB FB_CreateObjects
DEFINT j, cnt
DEFSTR tstr, tok, parnt, ExtStyle

    CodeText2.AddStrings "", "'-------------------------------------------------------------------------------"
    CodeText2.AddStrings "' Create components"
    CodeText2.AddStrings "'-------------------------------------------------------------------------------"
    CodeText2.AddStrings "SUB CreateMainMenu"
    CodeText2.AddStrings "END SUB",""
    CodeText2.AddStrings "SUB CreatePopupMenus"
    CodeText2.AddStrings "END SUB",""

    CodeText2.AddStrings "SUB CreateComponents"
    CodeText2.AddStrings "DIM AS UINT StdStyle, ExtStyle=0"
    CodeText2.AddStrings "DIM AS string ObjCaption",""
    ' Do the form
    tok=RTRIM$(FB_Objects(0).FBObjPtrName)
    CodeText2.AddStrings SPACE$(4)+tok+"->PreSetConfig()"
    CodeText2.AddStrings SPACE$(4)+"StdStyle="+tok+"->objStyle"
    CodeText2.AddStrings SPACE$(4)+"IF "+tok+"->BorderStyle > bsDialog THEN ExtStyle=WS_EX_TOOLWINDOW",""
    CodeText2.AddStrings SPACE$(4)+tok+"->Handle = CreateWindowEx(ExtStyle, "+QU+ _
                                            "RQClass"+QU+", "+tok+"->Caption, StdStyle, _"
    CodeText2.AddStrings SPACE$(20)+tok+"->Left, "+tok+"->Top, "+tok+"->Width, "+tok+"->Height, _"
    CodeText2.AddStrings SPACE$(20)+"0, 0, Application.hInstance, 0)"
    CodeText2.AddStrings SPACE$(4)+"Application.hMainForm = "+tok+"->Handle"
    CodeText2.AddStrings SPACE$(4)+tok+"->PostConfig()"
    cnt=1
    WHILE cnt < ObjCount
        IF FB_ObjectVisible(cnt) THEN
            tstr=RTRIM$(FB_Objects(cnt).FBObjPtrName)
            tok=SPACE$(4)+tstr+"->PreSetConfig()"                           ' Any pre-configuration?
            CodeText2.AddStrings "",tok
            ' This object have a caption?
            IF FB_HasObjCaption(FB_Objects(cnt).FBObjType) THEN
                tok=SPACE$(4)+"ObjCaption="+tstr+"->Caption"                ' Set caption in code
            ELSE
                tok=SPACE$(4)+"ObjCaption="+QU+QU                           ' No caption this object
            END IF
            CodeText2.AddStrings tok
            ' Get the parent of this object fm MyObjects struc
            j=FB_GetParentIdx(cnt)
            IF UCASE$(RTRIM$(FB_Objects(j).FBObjType))="QSCROLLBOX" THEN    ' Parent scrollbox?
                parnt=RTRIM$(FB_Objects(j).FBObjPtrName)+"->PanHandle"      ' The embedded panel not scrollbox
            ELSE
                parnt=RTRIM$(FB_Objects(j).FBObjPtrName)+"->Handle"         ' The objects parent
            END IF
            ' Accumulate window style parms
            tok=SPACE$(4)+"StdStyle="+tstr+"->objStyle"
            CodeText2.AddStrings tok
            tok=SPACE$(4)+"IF "+tstr+"->Visible THEN StdStyle OR= WS_VISIBLE"
            CodeText2.AddStrings tok
            SELECT CASE FB_Objects(cnt).FBObjID
                CASE Q_EDIT,Q_RICHEDIT,Q_LISTBOX,Q_SCROLLBOX,Q_DIRTREE,Q_LISTVIEW,Q_TREEVIEW         ' Any more?
                    tok="IF "+tstr+"->BorderStyle=bsNone THEN ExtStyle=0 ELSE ExtStyle=WS_EX_CLIENTEDGE"
                    CodeText2.AddStrings SPACE$(4)+tok,""
                CASE Q_FILELISTBOX
                    CodeText2.AddStrings SPACE$(4)+"ExtStyle=WS_EX_CLIENTEDGE",""
                CASE ELSE
                    CodeText2.AddStrings SPACE$(4)+"ExtStyle=0",""
            END SELECT
            tok=SPACE$(4)+tstr+"->Handle=CreateWindowEx(ExtStyle,"
            tok=tok+tstr+"->objClass, ObjCaption, WS_CHILD OR StdStyle, _"
            CodeText2.AddStrings tok
            tok=SPACE$(20)+tstr+"->Left, "+tstr+"->Top, "+tstr+"->Width, "+tstr+"->Height, _"
            CodeText2.AddStrings tok
            tok=SPACE$(20)+parnt+", CAST(HMENU, "+tstr+"->objIndex), 0, 0)"
            CodeText2.AddStrings tok, ""
            CodeText2.AddStrings SPACE$(4)+tstr+"->Parent = "+parnt
            FB_AddSubItems(cnt)                                             ' Any additional config?
            tok=SPACE$(4)+tstr+"->PostConfig()"                             ' Any post-configuration?
            CodeText2.AddStrings tok
            CodeText2.AddStrings SPACE$(4)+"EnableWindow("+tstr+"->Handle, "+tstr+"->Enabled)"
        ELSE
            FB_AddNonVisible(cnt)
        END IF
        INC cnt
    WEND
    CodeText2.AddStrings "", SPACE$(4)+"CreateMainMenu()"
    CodeText2.AddStrings SPACE$(4)+"CreatePopupMenus()"
    CodeText2.AddStrings "END SUB"
END SUB

SUB FB_KillTimerCode
DEFINT i
DEFSTR tstr, tok1, tok2

    tok1=RTRIM$(FB_Objects(0).FBObjPtrName)                         ' Main form ptr
    CodeText2.AddStrings "SUB KillAllTimers"
    CodeText2.AddStrings SPACE$(4)+"KillTimer(Application.hMainForm, SYS_TIMER_IDX)"
    FOR i=1 TO ObjCount-1
        tstr=UCASE$(RTRIM$(FB_Objects(i).FBObjType))
        IF tstr="QTIMER" THEN
            tok2=RTRIM$(FB_Objects(i).FBObjPtrName)
            CodeText2.AddStrings SPACE$(4)+tok2+"->Enabled=False"
            CodeText2.AddStrings SPACE$(4)+"KillTimer("+tok1+"->Handle, "+tok2+"->ObjIndex)"
        END IF
    NEXT i
    CodeText2.AddStrings "END SUB"
END SUB

SUB FB_SysCheckHints
DEFINT i
DEFSTR tstr, tok

    CodeText2.AddStrings "SUB CheckSystemTimers"
    CodeText2.AddStrings "STATIC AS HWND TT_LastHWND"
    CodeText2.AddStrings "DIM AS HWND TT_hWnd"
    CodeText2.AddStrings "DIM AS Point TT_pnt",""
    CodeText2.AddStrings SPACE$(4)+"TT_pnt.X=Screen.MouseX: TT_pnt.Y=Screen.MouseY"
    CodeText2.AddStrings SPACE$(4)+"TT_hWnd=WindowFromPoint(TT_pnt)"
    CodeText2.AddStrings SPACE$(4)+"IF TT_LastHWND<>TT_hWnd THEN"
    CodeText2.AddStrings SPACE$(8)+"TT_LastHWND=TT_hWnd"
    CodeText2.AddStrings SPACE$(8)+"SELECT CASE TT_hWnd"
    FOR i=0 TO ObjCount-1
        IF FB_ObjectVisible(i) THEN
            tstr=RTRIM$(FB_Objects(i).FBObjPtrName)
            CodeText2.AddStrings SPACE$(12)+"CASE "+tstr+"->Handle"
            CodeText2.AddStrings SPACE$(16)+"IF "+tstr+"->ShowHint THEN"
            CodeText2.AddStrings SPACE$(20)+"SysDisplayHint("+tstr+"->Handle, "+tstr+"->Hint)"
            CodeText2.AddStrings SPACE$(16)+"END IF"
        END IF
    NEXT i
    CodeText2.AddStrings SPACE$(8)+"END SELECT"
    CodeText2.AddStrings SPACE$(4)+"END IF"
    CodeText2.AddStrings "END SUB"
END SUB

SUB FB_FillTimerEvents
DEFINT i, AddEm=False
DEFSTR tstr, tok

    CodeText2.AddStrings "","SUB CheckWmTimers(hWnd AS HWND, wMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM)"
    CodeText2.AddStrings SPACE$(4)+"SELECT CASE wParam"
    CodeText2.AddStrings SPACE$(8)+"CASE App_Timer.ObjIndex:  CheckSystemTimers"

    FOR i=1 TO ObjCount-1
        tstr=UCASE$(RTRIM$(FB_Objects(i).FBObjType))
        IF tstr="QTIMER" THEN                                           ' Any user timers?????
            Addem=True
            EXIT FOR
        END IF
    NEXT i
    IF AddEm THEN
        FOR i=1 TO ObjCount-1
            tstr=UCASE$(RTRIM$(FB_Objects(i).FBObjType))
            IF tstr="QTIMER" THEN
                tok=RTRIM$(FB_Objects(i).FBObjPtrName)
                CodeText2.AddStrings SPACE$(8)+"CASE "+tok+"->ObjIndex"
                CodeText2.AddStrings SPACE$(12)+"IF ("+tok+"->Enabled) AND ("+tok+"->OnTimer>0) THEN"
                CodeText2.AddStrings SPACE$(16)+tok+"->OnTimer("+tok+")"
                CodeText2.AddStrings SPACE$(12)+"END IF"
            END IF
        NEXT i
    END IF
    CodeText2.AddStrings SPACE$(4)+"END SELECT"
    CodeText2.AddStrings "END SUB"
END SUB

SUB FB_FillCmdEvents
DEFINT i, ID, Addem=False
DEFSTR tstr, tok

    CodeText2.AddStrings "","SUB CheckWmCommandMsg(wParam AS WPARAM, lParam AS LPARAM)"
    FOR i=1 TO ObjCount-1
        ID=FB_Objects(i).FBObjID
        SELECT CASE ID
            CASE Q_EDIT,Q_RICHEDIT,Q_LABEL,Q_BUTTON,Q_COOLBTN,Q_OVALBTN,Q_CHECKBOX, _
                 Q_RADIOBUTTON,Q_PANEL,Q_SCROLLBOX,Q_CANVAS,Q_IMAGE,Q_LISTBOX, _
                 Q_FILELISTBOX,Q_COMBOBOX,Q_GROUPBOX
                Addem=True
                EXIT FOR
        END SELECT
    NEXT i
    IF Addem=False THEN
        CodeText2.AddStrings "END SUB"
        EXIT SUB
    END IF

    CodeText2.AddStrings SPACE$(4)+"SELECT CASE LOWORD(wParam)"
    FOR i=1 TO ObjCount-1
        ID=FB_Objects(i).FBObjID
        SELECT CASE ID
            CASE Q_EDIT,Q_RICHEDIT
                tok=RTRIM$(FB_Objects(i).FBObjPtrName)                ' eg lpForm1
                CodeText2.AddStrings SPACE$(8)+"CASE "+tok+"->ObjIndex"
                CodeText2.AddStrings SPACE$(12)+"IF "+tok+"->OnChange>0 THEN "+tok+"->OnChange("+tok+")"
            CASE Q_LABEL,Q_BUTTON,Q_COOLBTN,Q_OVALBTN,Q_CHECKBOX,Q_RADIOBUTTON,Q_PANEL,Q_SCROLLBOX,Q_CANVAS,Q_IMAGE,Q_GROUPBOX
                tok=RTRIM$(FB_Objects(i).FBObjPtrName)                ' eg lpForm1
                CodeText2.AddStrings SPACE$(8)+"CASE "+tok+"->ObjIndex"
                CodeText2.AddStrings SPACE$(12)+"IF "+tok+"->OnClick>0 THEN "+tok+"->OnClick("+tok+")"
            CASE Q_LISTBOX,Q_FILELISTBOX
                tok=RTRIM$(FB_Objects(i).FBObjPtrName)
                CodeText2.AddStrings SPACE$(8)+"CASE "+tok+"->ObjIndex"
                CodeText2.AddStrings SPACE$(12)+"IF HIWORD(wParam)=LBN_SELCHANGE THEN"
                CodeText2.AddStrings SPACE$(16)+"IF "+tok+"->OnClick>0 THEN "+tok+"->OnClick("+tok+")"
                CodeText2.AddStrings SPACE$(12)+"END IF"
                CodeText2.AddStrings SPACE$(12)+"IF HIWORD(wParam)=LBN_DBLCLK THEN"
                CodeText2.AddStrings SPACE$(16)+"IF "+tok+"->OnDblClick>0 THEN "+tok+"->OnDblClick("+tok+")"
                CodeText2.AddStrings SPACE$(12)+"END IF"
            CASE Q_COMBOBOX
                tok=RTRIM$(FB_Objects(i).FBObjPtrName)
                CodeText2.AddStrings SPACE$(8)+"CASE "+tok+"->ObjIndex"
                CodeText2.AddStrings SPACE$(12)+"IF HIWORD(wParam)=CBN_SELCHANGE THEN"
                CodeText2.AddStrings SPACE$(16)+"IF "+tok+"->OnChange>0 THEN "+tok+"->OnChange("+tok+")"
                CodeText2.AddStrings SPACE$(12)+"END IF"
        END SELECT
    NEXT i
    CodeText2.AddStrings SPACE$(4)+"END SELECT"
    CodeText2.AddStrings "END SUB"
END SUB

SUB FB_FillNotifyEvents
DEFINT i, Addem=False
DEFSTR tstr, tok

    CodeText2.AddStrings "","SUB CheckWmNotifyMsg(wParam AS WPARAM, lParam AS LPARAM)"

    FOR i=1 TO ObjCount-1
        tstr=UCASE$(RTRIM$(FB_Objects(i).FBObjType))
        SELECT CASE tstr
            CASE "QTABCONTROL","QTRACKBAR","QHEADER","QLISTVIEW","QDIRTREE","QTREEVIEW","QPAGESCROLLER"
                Addem=True
                EXIT FOR
        END SELECT
    NEXT i
    IF AddEm THEN
        CodeText2.AddStrings "DIM AS NMHDR PTR lpNmhdr=CAST(NMHDR PTR, lParam)"
        CodeText2.AddStrings "",SPACE$(4)+"SELECT CASE lpNmhdr->hwndFrom"
        FOR i=1 TO ObjCount-1
            tstr=UCASE$(RTRIM$(FB_Objects(i).FBObjType))
            tok=RTRIM$(FB_Objects(i).FBObjPtrName)
            SELECT CASE tstr
                CASE "QTABCONTROL"
                    CodeText2.AddStrings SPACE$(8)+"CASE "+tok+"->Handle"
                    CodeText2.AddStrings SPACE$(12)+"IF lpNmhdr->Code=TCN_SELCHANGE THEN"
                    CodeText2.AddStrings SPACE$(16)+"IF "+tok+"->OnChange>0 THEN "+tok+"->OnChange("+tok+")"
                    CodeText2.AddStrings SPACE$(12)+"END IF"
                CASE "QTRACKBAR"
                    CodeText2.AddStrings SPACE$(8)+"CASE "+tok+"->Handle"
                    CodeText2.AddStrings SPACE$(12)+"IF lpNmhdr->code=NM_RELEASEDCAPTURE THEN"
                    CodeText2.AddStrings SPACE$(16)+tok+"->Position=SendMessage("+tok+"->Handle, TBM_GETPOS, 0, 0)"
                    CodeText2.AddStrings SPACE$(16)+"IF "+tok+"->OnChange>0 THEN "+tok+"->OnChange("+tok+")"
                    CodeText2.AddStrings SPACE$(12)+"END IF"
                CASE "QHEADER"
                    CodeText2.AddStrings SPACE$(8)+"CASE "+tok+"->Handle"
                    CodeText2.AddStrings SPACE$(12)+"IF lpNmhdr->code = HDN_ITEMCLICK THEN              ' Left button?"
                    CodeText2.AddStrings SPACE$(16)+"DIM AS NMHEADER PTR lpNmHead=CAST(NMHEADER PTR, lParam)"
                    tstr="IF "+tok+"->OnSectionClick>0 THEN "+tok+"->OnSectionClick(lpNmHead->iItem, "+tok+")"
                    CodeText2.AddStrings SPACE$(16)+tstr
                    CodeText2.AddStrings SPACE$(12)+"END IF"
                CASE "QLISTVIEW"
                    CodeText2.AddStrings SPACE$(8)+"CASE "+tok+"->Handle"
                    CodeText2.AddStrings SPACE$(12)+"DIM AS NMITEMACTIVATE PTR lpItmActiv=CAST(NMITEMACTIVATE PTR, lParam)"
                    CodeText2.AddStrings SPACE$(12)+"IF (lpNmhdr->code=NM_CLICK) AND (lpItmActiv->iItem > -1) THEN"
                    CodeText2.AddStrings SPACE$(16)+tok+"->ItemIndex=lpItmActiv->iItem"
                    CodeText2.AddStrings SPACE$(16)+"IF "+tok+"->OnClick>0 THEN "+tok+"->OnClick("+tok+")"
                    CodeText2.AddStrings SPACE$(12)+"END IF"
                    CodeText2.AddStrings SPACE$(12)+"IF (lpNmhdr->code=NM_DBLCLK) AND (lpItmActiv->iItem > -1) THEN"
                    CodeText2.AddStrings SPACE$(16)+tok+"->ItemIndex=lpItmActiv->iItem"
                    CodeText2.AddStrings SPACE$(16)+"IF "+tok+"->OnDblClick>0 THEN "+tok+"->OnDblClick("+tok+")"
                    CodeText2.AddStrings SPACE$(12)+"END IF"
                    CodeText2.AddStrings SPACE$(12)+"IF (lpNmhdr->code=LVN_COLUMNCLICK) AND (lpItmActiv->iItem = -1) AND _"
                    CodeText2.AddStrings SPACE$(64)+"("+tok+"->ColumnClick=True) THEN"
                    tstr="IF "+tok+"->OnColumnClick>0 THEN "+tok+"->OnColumnClick(lpItmActiv->iSubItem, "+tok+")"
                    CodeText2.AddStrings SPACE$(16)+tstr
                    CodeText2.AddStrings SPACE$(12)+"END IF"
                CASE "QDIRTREE"
                    CodeText2.AddStrings SPACE$(8)+"CASE "+tok+"->Handle"
                    CodeText2.AddStrings SPACE$(12)+"IF lpNmhdr->code=TVN_SELCHANGED THEN"
                    CodeText2.AddStrings SPACE$(16)+tok+"->UpdateTreeView(lParam)"
                    CodeText2.AddStrings SPACE$(16)+"IF "+tok+"->OnChange>0 THEN "+tok+"->OnChange("+tok+")"
                    CodeText2.AddStrings SPACE$(12)+"END IF"
                CASE "QTREEVIEW"
                    CodeText2.AddStrings SPACE$(8)+"CASE "+tok+"->Handle"
                    CodeText2.AddStrings SPACE$(12)+"SELECT CASE lpNmhdr->code"
                    CodeText2.AddStrings SPACE$(16)+"CASE TVN_SELCHANGED                 ' NOT NM_CLICK!"
                    CodeText2.AddStrings SPACE$(20)+tok+"->UpdateTreeView(lParam)"
                    CodeText2.AddStrings SPACE$(20)+"IF "+tok+"->OnClick>0 THEN "+tok+"->OnClick("+tok+")"
                    CodeText2.AddStrings SPACE$(16)+"CASE NM_DBLCLK"
                    CodeText2.AddStrings SPACE$(20)+"IF "+tok+"->OnDblClick>0 THEN "+tok+"->OnDblClick("+tok+")"
                    CodeText2.AddStrings SPACE$(12)+"END SELECT"
                CASE "QPAGESCROLLER"
                    CodeText2.AddStrings SPACE$(8)+"CASE "+tok+"->Handle"
                    CodeText2.AddStrings SPACE$(12)+"IF lpNmhdr->Code=PGN_CALCSIZE THEN"
                    CodeText2.AddStrings SPACE$(16)+"ShowMessage("+QU+"Here"+QU+")"
                    CodeText2.AddStrings SPACE$(12)+"END IF"
            END SELECT
        NEXT i
        CodeText2.AddStrings SPACE$(4)+"END SELECT"
    END IF
    CodeText2.AddStrings "END SUB"
END SUB

SUB FB_FillPaintEvents
DEFINT i, AddEm=False
DEFSTR tstr, tok

    CodeText2.AddStrings "","SUB CheckWmPaintMsg(hWnd AS HWND, wMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM)"
    ' Any Components need auto re-sizing?
    FOR i=1 TO ObjCount-1
        tstr=UCASE$(RTRIM$(FB_Objects(i).FBObjType))
        SELECT CASE tstr
            CASE "QHEADER","QSTATUSBAR","QTOOLBAR"
                AddEm=True
                EXIT FOR
        END SELECT
    NEXT i
    IF AddEm=True THEN
        CodeText2.AddStrings "DIM AS RECT frmRect"
        CodeText2.AddStrings "DIM AS integer H, W",""
        CodeText2.AddStrings SPACE$(4)+"GetClientRect(hWnd, @frmRect)"
        CodeText2.AddStrings SPACE$(4)+"W=frmRect.Right-frmRect.Left"
        FOR i=1 TO ObjCount-1
            tstr=UCASE$(RTRIM$(FB_Objects(i).FBObjType))
            SELECT CASE tstr
                CASE "QHEADER","QTOOLBAR"                                           ' Default align to the top
                    tok=RTRIM$(FB_Objects(i).FBObjPtrName)
                    CodeText2.AddStrings "", SPACE$(4)+"H="+tok+"->Height"
                    CodeText2.AddStrings SPACE$(4)+"MoveWindow("+tok+"->Handle, 0, 0, W, H, 1)"
                    IF tstr="QHEADER" THEN
                        CodeText2.AddStrings SPACE$(4)+"IF "+tok+"->OnResize>0 THEN "+tok+"->OnResize("+tok+")"
                    END IF

                CASE "QSTATUSBAR"                                                   ' Default align to the bottom
                    tok=RTRIM$(FB_Objects(i).FBObjPtrName)
'                    CodeText2.AddStrings "", SPACE$(4)+"IF "+tok+"->SizeGrip THEN"
                    CodeText2.AddStrings "", SPACE$(4)+"H="+tok+"->Height"
                    CodeText2.AddStrings SPACE$(4)+"MoveWindow("+tok+"->Handle, 0, frmRect.Bottom-H, W, H, 1)"
                    CodeText2.AddStrings SPACE$(4)+"IF "+tok+"->OnResize>0 THEN "+tok+"->OnResize("+tok+")"
'                    CodeText2.AddStrings SPACE$(4)+"END IF"
            END SELECT
        NEXT i
        CodeText2.AddStrings ""
    END IF
    tstr=RTRIM$(FB_Objects(0).FBObjPtrName)                                         ' Main form
    CodeText2.AddStrings SPACE$(4)+"IF "+tstr+"->OnPaint>0 THEN "+tstr+"->OnPaint("+tstr+")"
    CodeText2.AddStrings "END SUB"
END SUB

' Currently not used
SUB FB_FillHitTestEvents
    CodeText2.AddStrings "","SUB CheckWmHitTest(hWnd AS HWND, wMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM)"
    CodeText2.AddStrings "END SUB"
END SUB

' ScrollBar object as child object of a parent (form) 
SUB FB_FillScrollItems
DEFSTR tstr, tok, objPtr
DEFINT i, Addem=False

    CodeText2.AddStrings "","SUB CheckWmObjScroll(hWnd AS HWND, wMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM)"
    FOR i=1 TO ObjCount-1
        tstr=UCASE$(RTRIM$(FB_Objects(i).FBObjType))
        SELECT CASE tstr
            CASE "QSCROLLBAR"
                Addem=True
                EXIT FOR
        END SELECT
    NEXT i
    IF Addem THEN
        CodeText2.AddStrings "DIM AS integer Posn",""
        CodeText2.AddStrings SPACE$(4)+"SELECT CASE lParam"
        FOR i=1 TO ObjCount-1
            tstr=UCASE$(RTRIM$(FB_Objects(i).FBObjType))
            IF tstr="QSCROLLBAR" THEN
                objPtr=RTRIM$(FB_Objects(i).FBObjPtrName)
                CodeText2.AddStrings SPACE$(8)+"CASE "+objPtr+"->Handle"
                CodeText2.AddStrings SPACE$(12)+"Posn=GetScrollPos("+objPtr+"->Handle, SB_CTL)"
                CodeText2.AddStrings SPACE$(12)+"SELECT CASE LOWORD(wParam)             ' independent of orientation"
                CodeText2.AddStrings SPACE$(16)+"CASE SB_LINERIGHT,SB_LINEDOWN : Posn += "+objPtr+"->SmallChange"
                CodeText2.AddStrings SPACE$(16)+"CASE SB_PAGERIGHT,SB_PAGEDOWN : Posn += "+objPtr+"->LargeChange"
                CodeText2.AddStrings SPACE$(16)+"CASE SB_LINELEFT,SB_LINEUP    : Posn -= "+objPtr+"->SmallChange"
                CodeText2.AddStrings SPACE$(16)+"CASE SB_PAGELEFT,SB_PAGEUP    : Posn -= "+objPtr+"->LargeChange"
                CodeText2.AddStrings SPACE$(12)+"END SELECT"
                CodeText2.AddStrings SPACE$(12)+"IF Posn > "+objPtr+"->Max THEN Posn = "+objPtr+"->Max"
                CodeText2.AddStrings SPACE$(12)+"IF Posn < "+objPtr+"->Min THEN Posn = "+objPtr+"->Min"
                CodeText2.AddStrings SPACE$(12)+objPtr+"->Position=Posn           ' Update the ScrollBar"
                CodeText2.AddStrings SPACE$(12)+"IF "+objPtr+"->OnChange>0 THEN "+objPtr+"->OnChange("+objPtr+")"
            END IF
        NEXT i
        CodeText2.AddStrings SPACE$(4)+"END SELECT"
    END IF
    CodeText2.AddStrings "END SUB"
END SUB

' NOT used - NO Button style components are OWNERDRAW style
SUB FB_CtlColorBtn
DEFSTR tstr, objPtr
DEFINT i, Addem=False

    CodeText2.AddStrings "","FUNCTION CheckBtnColor(wParam AS WPARAM, lParam AS LPARAM) AS LRESULT"
    CodeText2.AddStrings "DIM AS HBRUSH hBrush=NULL"
    CodeText2.AddStrings "' Only for OwnerDraw buttons"
' DIM AS HDC hDC=CAST(HDC, wParam)
' DIM AS HWND Handle=CAST(HWND, lParam)

'     SELECT CASE Handle
'         CASE Obj->Handle                                   ' Obj = BUTTON-TYPE object
'             SetBKMode(hDC, 1)
'             SetTextColor(hDC, CAST(COLORREF, Obj->Font.Color))
'             hBrush=CreateSolidBrush(CAST(COLORREF, Obj->Color))
'     END SELECT
    CodeText2.AddStrings SPACE$(4)+"RETURN CAST(LRESULT, hBrush)"
    CodeText2.AddStrings "END FUNCTION"
END SUB

' Sets color (background & text) for edit controls (NOT RichEdit) and ComboBox
SUB FB_CtlColorEdit
DEFSTR tstr, objPtr
DEFINT i, Addem=False

    CodeText2.AddStrings "","FUNCTION CheckEditColor(wParam AS WPARAM, lParam AS LPARAM) AS LRESULT"
    CodeText2.AddStrings "DIM AS HBRUSH hBrush=NULL"
    FOR i=1 TO ObjCount-1
        tstr=UCASE$(RTRIM$(FB_Objects(i).FBObjType))
        SELECT CASE tstr
            CASE "QEDIT", "QCOMBOBOX"                                       ' Edit panel of a combobox!
                Addem=True
                EXIT FOR
        END SELECT
    NEXT i
    IF Addem THEN
        CodeText2.AddStrings "DIM AS HDC hDC=CAST(HDC, wParam)"
        CodeText2.AddStrings "DIM AS HWND Handle=CAST(HWND, lParam)",""
        CodeText2.AddStrings SPACE$(4)+"SELECT CASE Handle"
        FOR i=1 TO ObjCount-1
            tstr=UCASE$(RTRIM$(FB_Objects(i).FBObjType))
            IF tstr="QEDIT" THEN
                objPtr=RTRIM$(FB_Objects(i).FBObjPtrName)
                CodeText2.AddStrings SPACE$(8)+"CASE "+objPtr+"->Handle"
                CodeText2.AddStrings SPACE$(12)+"SetBKMode(hDC, 1)"
                CodeText2.AddStrings SPACE$(12)+"SetTextColor(hDC, CAST(COLORREF, "+objPtr+"->Font.Color))"
                CodeText2.AddStrings SPACE$(12)+"hBrush=CreateSolidBrush(CAST(COLORREF, "+objPtr+"->Color))"
            ELSEIF tstr="QCOMBOBOX" THEN
                objPtr=RTRIM$(FB_Objects(i).FBObjPtrName)
                CodeText2.AddStrings SPACE$(8)+"CASE "+objPtr+"->hEdit"
                CodeText2.AddStrings SPACE$(12)+"SetBKMode(hDC, 1)"
                CodeText2.AddStrings SPACE$(12)+"SetTextColor(hDC, CAST(COLORREF, "+objPtr+"->Font.Color))"
                CodeText2.AddStrings SPACE$(12)+"hBrush=CreateSolidBrush(CAST(COLORREF, "+objPtr+"->Color))"
            END IF
        NEXT i
        CodeText2.AddStrings SPACE$(4)+"END SELECT"
    END IF
    CodeText2.AddStrings SPACE$(4)+"RETURN CAST(LRESULT, hBrush)"
    CodeText2.AddStrings "END FUNCTION"
END SUB

SUB FB_CtlColorStatic
DEFSTR tstr, objPtr
DEFINT i, Addem=False

    CodeText2.AddStrings "","FUNCTION CheckStaticColor(wParam AS WPARAM, lParam AS LPARAM) AS LRESULT"
    CodeText2.AddStrings "DIM AS HBRUSH hBrush=NULL"
    FOR i=1 TO ObjCount-1
        tstr=UCASE$(RTRIM$(FB_Objects(i).FBObjType))
        SELECT CASE tstr
            CASE "QLABEL","QTRACKBAR"
                Addem=True
                EXIT FOR
        END SELECT
    NEXT i
    IF Addem THEN
        CodeText2.AddStrings "DIM AS HDC hDC=CAST(HDC, wParam)"
        CodeText2.AddStrings "DIM AS HWND Handle=CAST(HWND, lParam)",""
        CodeText2.AddStrings SPACE$(4)+"SELECT CASE Handle"
        FOR i=1 TO ObjCount-1
            tstr=UCASE$(RTRIM$(FB_Objects(i).FBObjType))
            IF (tstr="QLABEL") OR (tstr="QTRACKBAR")  THEN
                objPtr=RTRIM$(FB_Objects(i).FBObjPtrName)
                CodeText2.AddStrings SPACE$(8)+"CASE "+objPtr+"->Handle"
                CodeText2.AddStrings SPACE$(12)+"SetBKMode(hDC, 1)"
                CodeText2.AddStrings SPACE$(12)+"SetTextColor(hDC, CAST(COLORREF, "+objPtr+"->Font.Color))"
                CodeText2.AddStrings SPACE$(12)+"hBrush=CreateSolidBrush(CAST(COLORREF, "+objPtr+"->Color))"
            END IF
        NEXT i
        CodeText2.AddStrings SPACE$(4)+"END SELECT"
    END IF
    CodeText2.AddStrings SPACE$(4)+"RETURN CAST(LRESULT, hBrush)"
    CodeText2.AddStrings "END FUNCTION"
END SUB

SUB FB_CheckForListBox
DEFSTR tstr, tok
DEFINT i, Addem=False

    CodeText2.AddStrings "","FUNCTION CheckListBoxColor(wParam AS WPARAM, lParam AS LPARAM) AS LRESULT"
    CodeText2.AddStrings "DIM AS HBRUSH hBrush=NULL"
    FOR i=1 TO ObjCount-1
        tstr=UCASE$(RTRIM$(FB_Objects(i).FBObjType))
        SELECT CASE tstr
            CASE "QLISTBOX", "QFILELISTBOX", "QCOMBOBOX"
                Addem=True
                EXIT FOR
        END SELECT
    NEXT i
    IF Addem THEN
        CodeText2.AddStrings SPACE$(4)+"SELECT CASE lParam"
        FOR i=1 TO ObjCount-1
            tstr=UCASE$(RTRIM$(FB_Objects(i).FBObjType))
            SELECT CASE tstr
                CASE "QLISTBOX", "QFILELISTBOX"
                    tok=TRIM$(FB_Objects(i).FBObjPtrName)
                    CodeText2.AddStrings SPACE$(8)+"CASE "+tok+"->Handle"
                    CodeText2.AddStrings SPACE$(12)+"DIM AS HDC hDC=CAST(HDC, wParam)"
                    CodeText2.AddStrings SPACE$(12)+"DIM AS HGDIOBJ Original=SelectObject(hDC, GetStockObject(DC_BRUSH))"
                    CodeText2.AddStrings SPACE$(12)+"hBrush=CreateSolidBrush(CAST(COLORREF, "+tok+"->Color))"
                    CodeText2.AddStrings SPACE$(12)+"SetBkMode(hDC, 1)                          ' Transparent"
                    CodeText2.AddStrings SPACE$(12)+"SetTextColor(hDC, CAST(COLORREF,"+tok+"->Font.color))"
                CASE "QCOMBOBOX"
                    tok=TRIM$(FB_Objects(i).FBObjPtrName)
                    CodeText2.AddStrings SPACE$(8)+"CASE "+tok+"->hList"
                    CodeText2.AddStrings SPACE$(12)+"DIM AS HDC hDC=CAST(HDC, wParam)"
                    CodeText2.AddStrings SPACE$(12)+"DIM AS HGDIOBJ Original=SelectObject(hDC, GetStockObject(DC_BRUSH))"
                    CodeText2.AddStrings SPACE$(12)+"hBrush=CreateSolidBrush(CAST(COLORREF, "+tok+"->Color))"
                    CodeText2.AddStrings SPACE$(12)+"SetBkMode(hDC, 1)                          ' Transparent"
                    CodeText2.AddStrings SPACE$(12)+"SetTextColor(hDC, CAST(COLORREF,"+tok+"->Font.color))"
            END SELECT
        NEXT i
        CodeText2.AddStrings SPACE$(4)+"END SELECT"
    END IF
    CodeText2.AddStrings SPACE$(4)+"RETURN CAST(LRESULT, hBrush)"
    CodeText2.AddStrings "END FUNCTION"
END SUB

' Currently not used
SUB FB_FillWmDrawItem
    CodeText2.AddStrings "","SUB CheckWmDrawItem(wParam AS WPARAM, lParam AS LPARAM)"
    CodeText2.AddStrings "END SUB"
END SUB

SUB FB_FillPopup
DEFINT i
DEFSTR tstr

    CodeText2.AddStrings "","' Hook into the Popup menus for all of your components here"
    CodeText2.AddStrings "SUB CheckWmContextMenu(hWnd AS HWND, wMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM)"
'    CodeText2.AddStrings "' DIM AS UINT flags=TPM_RETURNCMD OR TPM_RIGHTBUTTON OR TPM_NOANIMATION"
'    CodeText2.AddStrings "' DIM AS integer Idx",""
'    CodeText2.AddStrings "' "+SPACE$(4)+"SELECT CASE wParam"
'    CodeText2.AddStrings "' "+SPACE$(8)+"CASE Obj->Handle                                           ' Check for objects with a popup menu"
'    CodeText2.AddStrings "' "+SPACE$(12)+"IF (Obj->PopupMenu>0) AND (Obj->OnPopUp>0) THEN"
'    CodeText2.AddStrings "' "+SPACE$(16)+"Idx=TrackPopupMenuEx(Obj->PopupMenu, flags, LOWORD(lParam), HIWORD(lParam), Obj->Handle, 0)"
'    CodeText2.AddStrings "' "+SPACE$(16)+"Obj->OnPopUp(Idx)                                 ' OnPopUp is the event handler"
'    CodeText2.AddStrings "' "+SPACE$(12)+"END IF"
'    CodeText2.AddStrings "' "+SPACE$(4)+"END SELECT"
    CodeText2.AddStrings "END SUB"
END SUB

SUB FB_FillSocketEvents
DEFINT i, Addem=False
DEFSTR tstr

    CodeText2.AddStrings "","SUB CheckSocketEvent(hWnd AS HWND, wMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM)"
    FOR i=1 TO ObjCount-1
        tstr=UCASE$(RTRIM$(FB_Objects(i).FBObjType))
        IF tstr="QSOCKET" THEN
            Addem=True
            EXIT FOR
        END IF
    NEXT i
    IF AddEm THEN
        CodeText2.AddStrings "' Server: Only ONE message of interest on the server master socket - FD_ACCEPT"
        CodeText2.AddStrings "' Client: Only ONE message of interest FD_CLOSE, remainder of messages optional"
        CodeText2.AddStrings "' NB The wParam is the socket descriptor (handle)"
        CodeText2.AddStrings SPACE$(4)+"IF HIWORD(lParam)=0 THEN"
        CodeText2.AddStrings SPACE$(8)+"SELECT CASE lParam"
        CodeText2.AddStrings SPACE$(12)+"CASE FD_ACCEPT"
        CodeText2.AddStrings SPACE$(16)+"lpSocket1->ConnPending=True                ' Remote connection request"
        CodeText2.AddStrings SPACE$(12)+"CASE FD_CONNECT                                '|"
        CodeText2.AddStrings SPACE$(12)+"CASE FD_READ                                   '| Client socket asynch mode,..."
        CodeText2.AddStrings SPACE$(12)+"CASE FD_WRITE                                  '| ...add your dispatch code here"
        CodeText2.AddStrings SPACE$(12)+"CASE FD_CLOSE                                  '|"
        CodeText2.AddStrings SPACE$(16)+"lpSocket1->DiscPending=wParam              ' Remote disconnect notification"
        CodeText2.AddStrings SPACE$(8)+"END SELECT"
        CodeText2.AddStrings SPACE$(4)+"ELSE"
        CodeText2.AddStrings SPACE$(8)+"lpSocket1->SockError=WSAGetLastError()"
        CodeText2.AddStrings SPACE$(4)+"END IF"
    END IF
    CodeText2.AddStrings "END SUB"
END SUB

SUB FB_AddMainFormCode
DEFSTR tok

    tok=RTRIM$(FB_Objects(0).FBObjPtrName)
    CodeText2.AddStrings SPACE$(4)+tok+"->Center                         ' You may wish to delete this line",""
    CodeText2.AddStrings SPACE$(4)+"IF "+tok+"->OnShow>0 THEN "+tok+"->OnShow("+tok+")",""
    CodeText2.AddStrings SPACE$(4)+"Result = "+tok+"->ShowModal             ' Show form & call default message handler",""
    CodeText2.AddStrings SPACE$(4)+"' You may want to change the above line to "+tok+"->Show, and then provide"
    CodeText2.AddStrings SPACE$(4)+"' your own function that, amongst other things, handles all window messages."
END SUB

SUB xFB_AddMainFormCode
DEFSTR tok

    tok=RTRIM$(FB_Objects(0).FBObjPtrName)
    CodeText2.AddStrings SPACE$(4)+"ConfigureComponents()",""
    CodeText2.AddStrings SPACE$(4)+"' Load the Main form"
    CodeText2.AddStrings SPACE$(4)+tok+"->Parent = 0"
    CodeText2.AddStrings SPACE$(4)+tok+"->Handle = CreateWindowEx(0, "+QU+"RQClass"+QU+", "+tok+"->Caption, _"
    CodeText2.AddStrings SPACE$(20)+"WS_OVERLAPPEDWINDOW, _"
    CodeText2.AddStrings SPACE$(20)+tok+"->Left, "+tok+"->Top, "+tok+"->Width, "+tok+"->Height, _"
    CodeText2.AddStrings SPACE$(20)+"0, 0, hInst, 0)"
    CodeText2.AddStrings SPACE$(4)+"Application.hMainForm = "+tok+"->Handle",""
    CodeText2.AddStrings SPACE$(4)+tok+"->Center                ' You may wish to delete this line",""
    CodeText2.AddStrings SPACE$(4)+"' Load remainder of components"
    CodeText2.AddStrings SPACE$(4)+"LoadComponents()",""
    CodeText2.AddStrings SPACE$(4)+"ShowWindow("+tok+"->Handle, iCmdShow)"
    CodeText2.AddStrings SPACE$(4)+"IF "+tok+"->OnShow>0 THEN "+tok+"->OnShow("+tok+")"
END SUB

SUB FB_FillFooter
DIM tList AS QStringList
DEFINT i
DEFSTR tstr, FormPtr

    tList.Clear
    HelpRef.Position=0
    FOR i=1 TO HelpRef.LineCount
        tstr=HelpRef.ReadLine
        IF FIELD$(tstr," ",1)="#9" THEN EXIT FOR                    ' Find the FB Footer section
    NEXT i
    FOR i=i+1 TO HelpRef.LineCount                                  ' Fill the temporary stringlist
        tstr=HelpRef.ReadLine-CHR$(13)-CHR$(10)
        IF LEFT$(tstr,1)="#" THEN EXIT FOR                          ' End of section text
        tList.AddItems tstr
    NEXT i

    FormPtr=RTRIM$(FB_Objects(0).FBObjPtrName)
    i=0
    DO
        tstr=tList.Item(i)
        IF INSTR(tstr,"###")>0 THEN
            SELECT CASE LEFT$(tstr,13)
                CASE "' ### MarkerA": FB_KillTimerCode
                CASE "' ### MarkerB": FB_SysCheckHints
                CASE "' ### MarkerC"
                    FB_FillTimerEvents                              '| Add the hooks code for ...
                    FB_FillCmdEvents                                '| ...these main form messages
                    FB_FillNotifyEvents                             '|
                    FB_FillPaintEvents                              '|
                    FB_CtlColorBtn                                  '| BUTTON types have to be OWNERDRAWN
                    FB_CtlColorStatic                               '|
                    FB_CtlColorEdit                                 '|
                    FB_CheckForListBox                              '|
                    FB_FillScrollItems                              '|
                    FB_FillSocketEvents                             '|
                    FB_FillPopup                                    '| Place to hook any popup menus
                    FB_FillHitTestEvents                            '|
                    FB_FillWmDrawItem                               '|
                CASE "' ### MarkerD":                               ' Spare ATT
                CASE "' ### MarkerE": FB_AddMainFormCode
            END SELECT
        ELSE
            IF INSTR(tstr,"$$ptr$$")>0 THEN tstr=REPLACESUBSTR$(tstr,"$$ptr$$",FormPtr)
            CodeText2.AddStrings tstr
        END IF
        INC i
    LOOP UNTIL i >= tList.ItemCount
    tList.Clear
END SUB

SUB FB_AddUserStubs
DEFINT i, j, idx
DEFSTR tstr, tok1, tok2
DEFSTR MyFuncList=""

    CodeText2.AddStrings "", "'-------------------------------------------------------------------------------"
    CodeText2.AddStrings "' Code for the component events"
    CodeText2.AddStrings "'-------------------------------------------------------------------------------"
    i=0
    DO
        j=FB_Objects(i).FBLineNum+1
        FOR idx=j TO FB_InputList.ItemCount-1
            tstr=LTRIM$(RTRIM$(FB_InputList.Item(idx)))
            IF LEFT$(tstr,1)="#" THEN EXIT FOR
            IF LEFT$(tstr,2)="On" THEN
                tok1=FIELD$(tstr,"|",2)                                 ' User defined sub event
                tok2=FIELD$(tstr,"|",3)                                 ' Sub event parameters
                IF tok1<>"" THEN                                        ' A defined sub event?
                    IF INSTR(MyFuncList,tok1)=0 THEN
                        MyFuncList=MyFuncList+","+tok1                  ' Add to global list
                        tok1="SUB "+tok1
                        IF tok2="" THEN
                            tok1=tok1+"(Sender AS "+RTRIM$(FB_Objects(i).FBObjType)+")"
                        ELSE
                            IF INSTR(tok2,"QRECT")>0 THEN tok2=FixQRect(tok2)           ' Replace RQ QRECT reference
                            tok1=tok1+LEFT$(tok2,LEN(tok2)-1)
                            tok1=tok1+", Sender AS "+RTRIM$(FB_Objects(i).FBObjType)+")"
                        END IF
                        CodeText2.AddStrings tok1,"END SUB",""
                    END IF
                END IF
            END IF
        NEXT idx
        INC i
    LOOP UNTIL i >= ObjCount
END SUB

SUB FB_GetFormData
DEFINT i, j
DEFSTR tstr, tok
DIM NoSupport AS QStringList

    FB_InputList.Clear
    FOR i=0 TO ObjectIndex-1
        FB_InputList.AddItems "#"+STR$(i+1)
        tstr=ObjectList(i)
        FOR j=1 TO TALLY(tstr,"~")+1
            tok=LEFT$(FIELD$(tstr,"~",j),128)
            FB_InputList.AddItems tok
        NEXT j
    NEXT i

    NoSupport.Clear
    FOR i=0 TO FB_InputList.ItemCount-1
        tstr=FB_InputList.Item(i)
        IF LEFT$(tstr,1)="#" THEN
            INC i
            tstr=FB_InputList.Item(i)
            IF FB_HasSupport(FIELD$(tstr,"|",2)) THEN
                FB_Objects(ObjCount).FBLineNum=i
                FB_Objects(ObjCount).FBObjID=VAL(FIELD$(tstr,"|",1))
                FB_Objects(ObjCount).FBObjType=FIELD$(tstr,"|",2)
                FB_Objects(ObjCount).FBObjName=FIELD$(tstr,"|",5)
                FB_Objects(ObjCount).FBObjPtrName="lp"+FIELD$(tstr,"|",5)
                FB_Objects(ObjCount).FBObjHandle=VAL(FIELD$(tstr,"|",3))
                FB_Objects(ObjCount).FBObjParent=VAL(FIELD$(tstr,"|",6))
                FB_Objects(ObjCount).FBChildCount=0
                INC ObjCount
            ELSE
                NoSupport.AddItems FIELD$(tstr,"|",2)
            END IF
        END IF
    NEXT i
    IF NoSupport.ItemCount>0 THEN
        tstr="Sorry, no support for:"+CRLF+CRLF
        FOR i=0 TO NoSupport.ItemCount-1
            tstr=tstr+NoSupport.Item(i)+CRLF
        NEXT i
        tstr=tstr+CRLF+"Code for the remainder of the"+CRLF+"components will be created!"
        MessageBox(tstr,"RapidFRM limitation",&h40)
    END IF
END SUB

SUB FB_SaveAndExit
DEFSTR tstr, outstr
DEFINT i

    CodeText2.SelectAll
    CodeText2.CopyToClipBoard
    CodeText2.SelLength=0                                           ' De-select text
    DoEvents
    ClipBoard.Open
    tstr=ClipBoard.text
    ClipBoard.Clear
    ClipBoard.Text=tstr
    ClipBoard.Close
    ErrorCondition=False: ObjectIndex=ComponentCount                ' 2 Conditions set to exit immediately
    ProgramEnd
END SUB

' RQ format source file as a template for FB
SUB SaveTemplate
DIM PM AS integer
DIM FName AS string
DIM SaveDialog AS QSAVEDIALOG

    PM=ProjectMode: ProjectMode=-1      ' NO header or footer wanted
    BuildFormCode                       ' RQ template code
    ProjectMode=PM
    SaveDialog.InitialDir=CURDIR$
    SaveDialog.Filter="FreeBASIC template file (*.tpl)|*.tpl|You choose (*.*)|*.*"
    IF SaveDialog.Execute THEN
        FName=SaveDialog.FileName
        IF INSTR(FName,".")=0 THEN FName=FName+".tpl"
        CodeText.SaveToFile(FName)
    ELSE
        MessageBox("No file saved!","Save Template File",&h10)
    END IF
END SUB

SUB BuildFBCode
    scan.Enabled=False                  ' Stop everything else that's going on
    ClipBoard.Clear                     ' Signal to IDE?

    CodeText2.Clear                     ' Start with a clean slate
    ObjCount=0                          ' ditto
    FB_GetFormData                      ' Load the global FB_InputList of form design
    FB_FillHeader                       ' Header includes (varies with components)
    FB_AddGlobals                       ' Global component structure & pointer creation
    FB_AddFwdDeclares                   ' Component event subs
    FB_ConfigureObjects                 ' Code to initialise component global structures
    FB_CreateObjects                    ' System load of the components
    FB_FillFooter                       ' Main Form load, component load, msg loop etc
    FB_AddUserStubs                     ' Dummy subs for component event code

    scan.Enabled=True
END SUB

'-----------------------------------------------------------------------------------
'----------------------------- RapidQ Information ----------------------------------
'-----------------------------------------------------------------------------------
'
' Format of FIRST section ("~" separated) of each ObjectList() entry :-
' <Object ID>|<Object descriptor>|<Object Handle>|<Internal name>|***<External name>|<Parent handle>~
' *** If standalone mode internal name else external name derived from users source code
'
' eg Object ID order of components in file "Component.data"
' "0|QFORM|123456|Form1|MyForm|654321~Left = 20|Top = 20|Width....etc"
' "4|QBUTTON|234567|Butn1|ExitBtn|765432~.......etc"
'-----------------------------------------------------------------------------------
